/* sane - Scanner Access Now Easy.
	Copyright (C) 2005 Mustek.
	Originally maintained by Mustek
	Author:Roy 2005.5.24

	This file is part of the SANE package.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston,
	MA 02111-1307, USA.

	As a special exception, the authors of SANE give permission for
	additional uses of the libraries contained in this release of SANE.

	The exception is that, if you link a SANE library with other files
	to produce an executable, this does not by itself cause the
	resulting executable to be covered by the GNU General Public
	License.  Your use of that executable is in no way restricted on
	account of linking the SANE library code into it.

	This exception does not, however, invalidate any other reasons why
	the executable file might be covered by the GNU General Public
	License.

	If you submit changes to SANE to the maintainers to be included in
	a subsequent release, you agree by submitting the changes that
	those changes may be distributed with this exception intact.

	If you write modifications of your own for SANE, it is your choice
	whether to permit this exception to apply to your modifications.
	If you do not wish that, delete this exception notice.

	This file provides a generic USB interface.  */
#include <time.h>
#include "../include/asic.h"
#include "../include/typedef.h"
#include "../include/public.h"
#include "DefineScanner.h"
#include "ScanRegDef.h"

#include "asic_mid.h"
#include "asic_libusb.h"


#ifndef DEVICENAME
#define DEVICENAME "a3p1"
#endif


#ifndef MAX_PATH
#define MAX_PATH 256
#endif




typedef enum tagCALIBRATETYPE {
	CT_FIRMWARE=0,
	CT_SOFTWARE=1,
	CT_PGA=2
} CALIBRATETYPE;


BYTE g_bMeanDarkShading;
BYTE g_bImagePixelAvg;
BYTE g_OutputRawData; 
BYTE g_Lamp0PWM; 
BYTE g_Lamp1PWM; 


BYTE g_PowerOnLampControl;
BYTE g_PowerOnLamp0PWM; 

BYTE g_bMeanDarkShading900dpi;
BYTE g_bImagePixelAvg900dpi;

WORD			wXShift; 
WORD			wYShift; 
CALIBRATETYPE	ctCalibrateType; 
WORD			wCalibrateWidth;
BYTE			bCalibrateWhiteLength; 
BYTE			bCalibrateDarkLength; 
BYTE			bThreshold; 
BOOL			isCompress;
BOOL			isFromADF; 
LIGHTSOURCE		lsLightSource; 
WORD			wCalibrateShift;
BYTE			bScanBits;

BYTE    isMotorMove;


BYTE isHardwareShading;
LPWORD lpWhiteShadingTable; 
LPWORD lpDarkShadingTable;

extern BYTE	PHTGTimingAdjustIN900;
extern BYTE	PH1_0IN900;
extern BYTE	PH1_1IN900;
extern BYTE	PH2_0IN900;
extern BYTE	PH2_1IN900;
extern BYTE	PH1B_0IN900;
extern BYTE	PH1B_1IN900;
extern BYTE	PH2B_0IN900;
extern BYTE	PH2B_1IN900;
extern BYTE	PHRS_0IN900;
extern BYTE	PHRS_1IN900;
extern BYTE	PHCP_0IN900;
extern BYTE	PHCP_1IN900;
extern BYTE	AFE_VSAMP_0IN900;
extern BYTE	AFE_VSAMP_1IN900;
extern BYTE	AFE_RSAMP_0IN900;
extern BYTE	AFE_RSAMP_1IN900;
extern BYTE	PH1234_IN_DUMMY_TGIN900;
extern BYTE	CCD_SETUP_REGISTERIN900;
extern BYTE	HARDWARE_SETTINGIN900;
extern BYTE	AFEControlIN900;
extern BYTE	AFE_CHANNEL_RED_LATCH_POSIN900;
extern BYTE	AFE_CHANNEL_GREEN_LATCH_POSIN900;



extern BYTE	AFE_CHANNEL_BLUE_LATCH_POSIN900;
extern BYTE	AFE_MALC_TIMING_ADJUST_0IN900;
extern BYTE	AFE_MALC_TIMING_ADJUST_1IN900;
extern BYTE	AD9826ConfigurationIN900;
extern BYTE	AD9826MuxConfigIN900;

extern BYTE	PHTGTimingAdjustIN1800;
extern BYTE	PH1_0IN1800;
extern BYTE	PH1_1IN1800;
extern BYTE	PH2_0IN1800;
extern BYTE	PH2_1IN1800;
extern BYTE	PH1B_0IN1800;
extern BYTE	PH1B_1IN1800;
extern BYTE	PH2B_0IN1800;
extern BYTE	PH2B_1IN1800;
extern BYTE	PHRS_0IN1800;
extern BYTE	PHRS_1IN1800;
extern BYTE	PHCP_0IN1800;
extern BYTE	PHCP_1IN1800;
extern BYTE	AFE_VSAMP_0IN1800;
extern BYTE	AFE_VSAMP_1IN1800;
extern BYTE	AFE_RSAMP_0IN1800;
extern BYTE	AFE_RSAMP_1IN1800;
extern BYTE	PH1234_IN_DUMMY_TGIN1800;
extern BYTE	CCD_SETUP_REGISTERIN1800;
extern BYTE	HARDWARE_SETTINGIN1800;
extern BYTE	AFEControlIN1800;
extern BYTE	AFE_CHANNEL_RED_LATCH_POSIN1800;
extern BYTE	AFE_CHANNEL_GREEN_LATCH_POSIN1800;
extern BYTE	AFE_CHANNEL_BLUE_LATCH_POSIN1800;
extern BYTE	AFE_MALC_TIMING_ADJUST_0IN1800;
extern BYTE	AFE_MALC_TIMING_ADJUST_1IN1800;
extern BYTE	AD9826ConfigurationIN1800;
extern BYTE	AD9826MuxConfigIN1800;

WORD ProductID = 0x0401;
WORD VendorID = 0x055f;


STATUS Asic_Open(PAsic chip, LPSTR pDeviceName)
{
	STATUS status=STATUS_GOOD;
	BYTE devname[MAX_PATH];
	int i;
	
	XDBG(DEBUG_ASIC,"Asic_Open: Enter");
	
	if (chip->firmwarestate > FS_OPENED)
	{
		XDBG(DEBUG_ASIC,"chip has been opened. fd=%d",chip->fd);
		return STATUS_INVAL;
	}
     
	/*init usb*/

	if(STATUS_GOOD!=libusb_init(VendorID,ProductID,&chip->fd))

	{
		fprintf(stderr,"Can't find  scanner vid=0x%04x,pid=0x%04x \n",VendorID,ProductID);
		return STATUS_INVAL;
	}

	
	/*open usb*/
	if(STATUS_GOOD!=libusb_open(chip->fd))
	{
		fprintf(stderr,"Can't open scanner! \n");
		return STATUS_INVAL;
	}

	/*open scanner chip*/
	status = OpenScanChip(chip);

	if (status != STATUS_GOOD)
	{
		libusb_close(chip->fd);
		XDBG(DEBUG_ASIC,"Asic_Open: OpenScanChip error");
		return status;
	}

	Mustek_SendData (chip,ES01_94_PowerSaveControl, 0x27);/*add by Roy 2005.10.18*/
	Mustek_SendData (chip,ES01_92_TimerPowerSaveTime , 54*3);/*add by Roy 2005.10.18*/
	
	chip->firmwarestate = FS_OPENED;
	Asic_WaitUnitReady(chip);
	XDBG(DEBUG_ASIC,"Asic_WaitUnitReady");
	status = SafeInitialChip(chip);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_Open: SafeInitialChip error\n");
		return status;
	}
	
	XDBG(DEBUG_ASIC,"Asic_Open: Exit");
	return status;
}


STATUS Asic_Close(PAsic chip)
{
	STATUS status=STATUS_GOOD;
	XDBG(DEBUG_ASIC,"Asic_Close: Enter");
	
	if (chip->firmwarestate < FS_OPENED)
	{
		XDBG(DEBUG_ASIC,"Asic_Close: Scanner is not opened");
		return STATUS_GOOD;
	}
	
	if (chip->firmwarestate > FS_OPENED) 
	{
		XDBG(DEBUG_ASIC,"Asic_Close: Scanner is scanning, try to stop scanning");
		Asic_ScanStop(chip);
	}
	
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_ENABLE);
	
	status = CloseScanChip(chip);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr, "Asic_Close: CloseScanChip error\n");
		return status;
	}
	
	libusb_close(chip->fd);
	chip->firmwarestate = FS_ATTACHED;
	
	XDBG(DEBUG_ASIC,"Asic_Close: Exit");
	return status;
}


STATUS Asic_TurnLamp(PAsic chip, BOOL isLampOn)
{
	STATUS status = STATUS_GOOD;

	XDBG(DEBUG_ASIC,"Asic_TurnLamp: Enter");

	if (chip->firmwarestate < FS_OPENED)
	{
		fprintf(stderr,"Asic_TurnLamp: Scanner is not opened\n");
		return STATUS_INVAL;
	}

	if (chip->firmwarestate > FS_OPENED)
	{
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	}

	if( isLampOn )
	{
		XDBG(DEBUG_ASIC,"ASIC_TurnLamp: Trun lamp0 on");
		if(g_PowerOnLampControl)
		{
			Mustek_SendData(chip,0x90, g_PowerOnLamp0PWM);
			Mustek_SendData(chip,0x91, 0);
			XDBG(DEBUG_ASIC,"Lamp0 PWM = %d", g_PowerOnLamp0PWM);
			XDBG(DEBUG_ASIC,"Lamp1 PWM = 0");
			usleep(800000);
		}

		Mustek_SendData(chip,0x90, g_Lamp0PWM);
		Mustek_SendData(chip,0x91, 0);
		XDBG(DEBUG_ASIC,"Lamp0 PWM = %d", g_Lamp0PWM);
		XDBG(DEBUG_ASIC,"Lamp1 PWM = 0");
	}
	else
	{
		XDBG(DEBUG_ASIC,"ASIC_TurnLamp: Trun lamp0 off");
		Mustek_SendData(chip,0x90, 0);
		Mustek_SendData(chip,0x91, 0);
	}

	chip->firmwarestate = FS_OPENED;
	XDBG(DEBUG_ASIC,"Asic_TurnLamp: Exit");

	return STATUS_GOOD;
}

STATUS Asic_TurnLampEngineTA(PAsic chip, BOOL isLampOn)
{
	XDBG(DEBUG_ASIC,"TurnLampEngineTA: Enter");

	if (chip->firmwarestate < FS_OPENED)
	{
		fprintf(stderr,"Asic_TurnLampEngineTA: Scanner is not opened\n");
		return STATUS_INVAL;
	}


	if (chip->firmwarestate > FS_OPENED)
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);

	if( isLampOn )
	{
		XDBG(DEBUG_ASIC,"ASIC_TurnLampEngineTA: Trun lamp0 and lamp1 on");
		if(g_PowerOnLampControl)
		{
			Mustek_SendData(chip,0x90, g_PowerOnLamp0PWM);
			Mustek_SendData(chip,0x91, g_Lamp1PWM);
			XDBG(DEBUG_ASIC,"Lamp0 PWM = %d", g_PowerOnLamp0PWM);
			XDBG(DEBUG_ASIC,"Lamp1 PWM = %d", g_Lamp1PWM);
			usleep(800000);
		}

		Mustek_SendData(chip,0x90, g_Lamp0PWM);
		Mustek_SendData(chip,0x91, g_Lamp1PWM);
		XDBG(DEBUG_ASIC,"Lamp0 PWM = %d", g_Lamp0PWM);
		XDBG(DEBUG_ASIC,"Lamp1 PWM = %d", g_Lamp1PWM);

	}
	else
	{
		XDBG(DEBUG_ASIC,"ASIC_TurnLampEngineTA: Trun lamp0 and lamp1 off");
		Mustek_SendData(chip,0x90, 0);
		Mustek_SendData(chip,0x91, 0);
	}

	chip->firmwarestate = FS_OPENED;

	XDBG(DEBUG_ASIC,"Asic_TurnLampEngineTA: Exit");
	
	return STATUS_GOOD;
}

STATUS Asic_TurnTA(PAsic chip, BOOL isTAOn)
{
	STATUS status;

	XDBG(DEBUG_ASIC,"Asic_TurnTA: Enter");

	if (chip->firmwarestate < FS_OPENED)
	{
		fprintf(stderr,"Asic_TurnTA: Scanner is not opened\n");
		return STATUS_INVAL;
	}

	if (chip->firmwarestate > FS_OPENED)
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);

	if( isTAOn )
	{
		XDBG(DEBUG_ASIC,"Asic_TurnTA: Turn TA on");
		Mustek_SendData(chip,0x90, 0);
		Mustek_SendData(chip,0x91, g_Lamp1PWM);
		XDBG(DEBUG_ASIC,"Lamp0 PWM = 0");
		XDBG(DEBUG_ASIC,"Lamp1 PWM = g_Lamp1PWM(%d)", g_Lamp1PWM);
	}
	else
	{
		XDBG(DEBUG_ASIC,"Asic_TurnTA: Turn TA off");
		Mustek_SendData(chip,0x90, 0);
		Mustek_SendData(chip,0x91, 0);
	}

	chip->firmwarestate = FS_OPENED;

	XDBG(DEBUG_ASIC,"Asic_TurnTA: Exit");
	
	return STATUS_GOOD;
}

STATUS Asic_WaitUnitReady(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	BYTE temp;
	
	XDBG(DEBUG_ASIC,"Asic_WaitUnitReady: Enter");
	if(chip->firmwarestate < FS_OPENED)
	{
		fprintf(stderr,"Asic_WaitUnitReady: Scanner has not been opened\n");
		return STATUS_INVAL;
	}
	
	do{
		Mustek_SendData(chip,ES01_8B_Status, 0x01);
		
		Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
		temp = ES01_8B_Status;
		Mustek_ReceiveData(chip,&temp);

	} while ((temp & 0x1f) != 0);
	
	chip->motorstate=MS_STILL;
	XDBG(DEBUG_ASIC,"Asic_WaitUnitReady: Exit");
	return status;
}

STATUS Asic_Release(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	
	if (chip->firmwarestate > FS_ATTACHED)
		status = Asic_Close(chip);
	
	chip->firmwarestate=FS_NULL;
	
	if(lpWhiteShadingTable != NULL)
		free(lpWhiteShadingTable);  
	if(lpDarkShadingTable != NULL)
		free(lpDarkShadingTable); 
	
	lpWhiteShadingTable = NULL;
	lpDarkShadingTable = NULL;

	return status;
}


STATUS Asic_Initialize(PAsic chip)
{
	STATUS status;

	
	chip->motorstate = MS_STILL;
	chip->dwBytesCountPerRow = 0;
	Asic_Reset(chip);

	chip->lpGammaTable=NULL;
	isMotorMove = MOTOR_1_ENABLE;
	chip->UsbHost = HT_USB20;

	chip->isFirstOpenChip=TRUE;

	chip->isHardwareShading = SHADING_ENABLE;
	lpWhiteShadingTable = NULL;  
	lpDarkShadingTable = NULL;  
	chip->SWWidth = 0;
	chip->TA_Status = TA_UNKNOW;

	PHTGTimingAdjustIN900 = 253;
	PH1_0IN900 = 0;
	PH1_1IN900 = 30;
	PH2_0IN900 = 4;
	PH2_1IN900 = 255;
	PH1B_0IN900 = 0;
	PH1B_1IN900 = 30;
	PH2B_0IN900 = 4;
	PH2B_1IN900 = 255;
	PHRS_0IN900 = 0;
	PHRS_1IN900 = 4;
	PHCP_0IN900 = 0;
	PHCP_1IN900 = 4;
	AFE_VSAMP_0IN900 = 2;
	AFE_VSAMP_1IN900 = 128;
	AFE_RSAMP_0IN900 = 2;
	AFE_RSAMP_1IN900 = 16;
	PH1234_IN_DUMMY_TGIN900 = 240;
	CCD_SETUP_REGISTERIN900 = 0;
	HARDWARE_SETTINGIN900 = 1;
	AFEControlIN900 = 1;
	AFE_CHANNEL_RED_LATCH_POSIN900 = 54;
	AFE_CHANNEL_GREEN_LATCH_POSIN900 = 155;
	AFE_CHANNEL_BLUE_LATCH_POSIN900 = 255;
	AFE_MALC_TIMING_ADJUST_0IN900 = 156;
	AFE_MALC_TIMING_ADJUST_1IN900 = 115;
	AD9826ConfigurationIN900 = 120;
	AD9826MuxConfigIN900 = 175;

	PHTGTimingAdjustIN1800 = 253;
	PH1_0IN1800 = 0;
	PH1_1IN1800 = 30;
	PH2_0IN1800 = 4;
	PH2_1IN1800 =225;
	PH1B_0IN1800 = 0;
	PH1B_1IN1800 = 30;
	PH2B_0IN1800 = 4;
	PH2B_1IN1800 = 225;
	PHRS_0IN1800 = 0;
	PHRS_1IN1800 = 2;
	PHCP_0IN1800 = 0;
	PHCP_1IN1800 = 8;
	AFE_VSAMP_0IN1800 = 2;
	AFE_VSAMP_1IN1800 = 128;
	AFE_RSAMP_0IN1800 = 2;
	AFE_RSAMP_1IN1800 = 16;
	PH1234_IN_DUMMY_TGIN1800 = 240;
	CCD_SETUP_REGISTERIN1800 = 32;
	HARDWARE_SETTINGIN1800 = 1;
	AFEControlIN1800 = 1;
	AFE_CHANNEL_RED_LATCH_POSIN1800 = 54;
	AFE_CHANNEL_GREEN_LATCH_POSIN1800 = 155;
	AFE_CHANNEL_BLUE_LATCH_POSIN1800 = 225;
	AFE_MALC_TIMING_ADJUST_0IN1800 = 156;
	AFE_MALC_TIMING_ADJUST_1IN1800 = 115;
	AD9826ConfigurationIN1800 = 120;
	AD9826MuxConfigIN1800 = 175;

	g_bMeanDarkShading = 1;
	g_bImagePixelAvg = 1;
	g_OutputRawData = 0; 
	g_Lamp0PWM = 200; 
	g_Lamp1PWM = 127; 

	g_PowerOnLampControl = 0;
	g_PowerOnLamp0PWM = 200; 

	g_bMeanDarkShading900dpi = 1;
	g_bImagePixelAvg900dpi = 1;

	chip->UsbHost = HT_USB10;
  
	status = ResetGpio(chip);
	
	return status;
}

STATUS Asic_SetWindow(PAsic chip, BYTE bScanBits, 
					  WORD wXResolution, WORD wYResolution, 
					  WORD wX, WORD wY, WORD wWidth, WORD wLength)
{
	LPBYTE				lpMotorTable;
	BYTE				isMotorMoveToFirstLine;
	BYTE				isUniformSpeedToScan;
	BYTE				isScanBackTracking;
	
	XDBG(DEBUG_ASIC,"Asic_SetWindow: Enter");
	XDBG(DEBUG_ASIC,"Asic_SetWIndow: bScanBits=%d,wXResolution=%d,wYResolution=%d,"
	"wX=%d,wY=%d,wWidth=%d,wLength=%d",bScanBits,wXResolution,wYResolution,wX,wY,wWidth,wLength);

	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_SetWindow: Scanner is not opened\n");
		return STATUS_INVAL;
	}
	
	switch (bScanBits){
	case 1:
	case 8:
		chip->Scan.ScanMode=1;
		chip->dwBytesCountPerRow=(DWORD)(wWidth);
		chip->dwCalibrationBytesCountPerRow = chip->dwBytesCountPerRow;
		break;
	case 16:
		chip->Scan.ScanMode=2;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*2;
		chip->dwCalibrationBytesCountPerRow = chip->dwBytesCountPerRow;
		break;
	case 24:	
		chip->Scan.ScanMode=3;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*3;
		chip->dwCalibrationBytesCountPerRow = chip->dwBytesCountPerRow;
		break;
	case 48:
		chip->Scan.ScanMode=4;
		chip->dwBytesCountPerRow=(DWORD)(wWidth)*6;
		chip->dwCalibrationBytesCountPerRow = chip->dwBytesCountPerRow;
		break;
	default:
		chip->dwBytesCountPerRow=(DWORD)(wWidth);
		chip->dwCalibrationBytesCountPerRow = chip->dwBytesCountPerRow;
		break;
	}

	XDBG(DEBUG_ASIC,"Asic_SetWindow: Scan.ScanMode=%d, dwBytesCountPerRow=%d, dwCalibrationBytesCountPerRow=%d",
		chip->Scan.ScanMode, chip->dwBytesCountPerRow, chip->dwCalibrationBytesCountPerRow);
	chip->Scan.StartPixel = wX;
	chip->Scan.StartLine  = wY;

	switch (wYResolution)
	{
	case 1800:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,1800);
		break;
	case 900:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_900_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,900);
		break;
	case 450:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_450_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,450);
		break;
	case 225:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_225_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,225);
		break;
	case 112:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_112_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,112);
		break;
	case 72:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_72_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,72);
		break;
	case 56:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_56_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,56);
		break;
	case 50:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetScanParam(chip,50);
		break;
	}

	chip->Scan.PicHeight  = wLength * 1800 / wYResolution;
	chip->Scan.LongX		= chip->Scan.PicWidth;

	XDBG(DEBUG_ASIC,"Asic_SetWindow: chip->Scan.PicWidth=%d, chip->Scan.StartPixel=%d chip->Scan.StartLine=%d"
		"chip->Scan.PicHeight=%d chip->Scan.LongX=%d chip->Scan.CCD_Pixel_Length=%d",
	chip->Scan.PicWidth, chip->Scan.StartPixel ,chip->Scan.StartLine,chip->Scan.PicHeight, chip->Scan.LongX,chip->Scan.CCD_Pixel_Length);
	
	
	
#ifdef HARDWARE_SHADING_SWITCH_ON
    if((wXResolution > 900) && (wWidth > HARDWARE_SHADING_WIDTH * wXResolution) && (bScanBits >= 8)) 
#else
	if((wXResolution > 900) && (bScanBits >= 8))
#endif
		{
			chip->Temp.Shading_Table_Size = 0;
			chip->isHardwareShading = SHADING_DISABLE; 
		}
		else
		{
			chip->Temp.Shading_Table_Size	= (((int)(wWidth * 3 * 2 / 510) * 512) + ((wWidth * 3 * 2) % 510) ) * 2;
			chip->isHardwareShading = SHADING_ENABLE;  
		}

	
		XDBG(DEBUG_ASIC,"SetWindow the shading table size (%d)", chip->Temp.Shading_Table_Size);

		chip->RamPositions.Shading	= 64 - (chip->Temp.Shading_Table_Size / 8192);
		if((chip->Temp.Shading_Table_Size % 8192) != 0)
			chip->RamPositions.Shading--;
	
		XDBG(DEBUG_ASIC,"Temp.Shading_Table_Size = %d,RamPositions.Shading = %d", 
			chip->Temp.Shading_Table_Size,chip->RamPositions.Shading);
	
		chip->RamPositions.Shading_0	= 0;
		chip->RamPositions.Shading_1	= (BYTE) ((chip->RamPositions.Shading << 4) & 0xf0);
		chip->RamPositions.Shading_2	= (BYTE) ((chip->RamPositions.Shading >> 4) & 0x0f);


		
		chip->RamPositions.Motor		= chip->RamPositions.Shading - 1;
		chip->RamPositions.Motor_0	= 0;
		chip->RamPositions.Motor_1	= (BYTE) ((chip->RamPositions.Motor << 4) & 0xf0);
		chip->RamPositions.Motor_2	= (BYTE) ((chip->RamPositions.Motor >> 4) & 0x0f);

		chip->Temp.Image_Buffer_Size	= chip->RamPositions.Motor * 4096 - 1;
		chip->RamPositions.ImageEndAddr_0	= LOBYTE(chip->Temp.Image_Buffer_Size);
		chip->RamPositions.ImageEndAddr_1	= HIBYTE(chip->Temp.Image_Buffer_Size);
		chip->RamPositions.ImageEndAddr_2	= (BYTE)((chip->Temp.Image_Buffer_Size & 0x00ff0000) >> 16);
		XDBG(DEBUG_ASIC,"SetWindow: chip->Temp.Image_Buffer_Size=%d",chip->Temp.Image_Buffer_Size);
		XDBG(DEBUG_ASIC,"SetWindow: chip->RamPositions.ImageEndAddr_0=%d chip->RamPositions.ImageEndAddr_1=%d"
			"chip->RamPositions.ImageEndAddr_2=%d",chip->RamPositions.ImageEndAddr_0,chip->RamPositions.ImageEndAddr_1,
			chip->RamPositions.ImageEndAddr_2);
		if(wXResolution == 1800)
		{
			DWORD dwTemp;
			dwTemp = chip->Scan.LongX * chip->Scan.ScanMode;
			if(chip->Temp.Image_Buffer_Size > dwTemp)
				chip->Temp.Full_Bank = dwTemp/16+1;
			else
				chip->Temp.Full_Bank = (chip->Scan.LongX * chip->Scan.ScanMode+1)/16;
		}
		else
		{
			DWORD dwTemp;
			dwTemp = chip->Scan.LongX * chip->Scan.ScanMode*2;

			if(chip->Temp.Image_Buffer_Size > dwTemp)
				chip->Temp.Full_Bank = (chip->Temp.Image_Buffer_Size - dwTemp) / 16;
			else
				chip->Temp.Full_Bank = (chip->Scan.LongX * chip->Scan.ScanMode+1)/16;
		}	

		XDBG(DEBUG_ASIC,"Asic_SetWindow: Scan.ScanMode=%d",chip->Scan.ScanMode);
		XDBG(DEBUG_ASIC,"Asic_SetWindow: Temp.Full_Bank=%d",chip->Temp.Full_Bank);
		
		chip->RamPositions.ImageFullBank_0 = LOBYTE(chip->Temp.Full_Bank);
		chip->RamPositions.ImageFullBank_1 = HIBYTE(chip->Temp.Full_Bank);
		
		SafeInitialChip(chip);
		SetPowerSave(chip);
		Mustek_SendData(chip,ES01_97_GPIOControl0_7, 0x7c);	

		if(chip->lightsource==LS_REFLECTIVE)
		{
			Mustek_SendData(chip,ES01_95_GPIOValue0_7, GPIO_95_Config);
		}
		else
		{
			Mustek_SendData(chip,ES01_95_GPIOValue0_7, GPIO_95_Config & 0xdf); 
		}
		
		Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x00);
		Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);
		
		
		SetLineTimeAndExposure(chip);	
		if(wXResolution > 50)
			chip->Timing.CCD_SETUP_REGISTER = CCD_SETUP_REGISTERIN1800; 
		else
			chip->Timing.CCD_SETUP_REGISTER = CCD_SETUP_REGISTERIN900; 
		
		XDBG(DEBUG_ASIC,"Timing.CCD_SETUP_REGISTER = 0x%x", chip->Timing.CCD_SETUP_REGISTER);
		
		CCDTiming(chip);
		SetADConverter(chip);
		Mustek_SendData(chip,ES01_B4_StartPixelLSB, LOBYTE(chip->Scan.StartPixel));
		Mustek_SendData(chip,ES01_B5_StartPixelMSB, HIBYTE(chip->Scan.StartPixel));
	
		XDBG(DEBUG_ASIC,"SetWindows StartPixel = %d", chip->Scan.StartPixel);
		
		Mustek_SendData(chip,ES01_B6_LineWidthPixelLSB, LOBYTE(chip->Scan.LongX));
		Mustek_SendData(chip,ES01_B7_LineWidthPixelMSB, HIBYTE(chip->Scan.LongX));	
		
		XDBG(DEBUG_ASIC,"SetWindows Scan.LongX = %d", chip->Scan.LongX);


		Mustek_SendData(chip,ES01_F7_DigitalControl, 0);

		Mustek_SendData(chip,ES01_9E_HorizontalRatio1to15LSB, LOBYTE(chip->Scan.Dpi));
		Mustek_SendData(chip,ES01_9F_HorizontalRatio1to15MSB, HIBYTE(chip->Scan.Dpi));
		XDBG(DEBUG_ASIC,"SetWindows: Scan.Dpi=%d ",chip->Scan.Dpi);
		
		Mustek_SendData(chip,ES01_9B_ShadingTableAddrA12_A19, (BYTE)(chip->RamPositions.Shading));
		XDBG(DEBUG_ASIC,"SetWindows: chip->RamPositions.Shading=%d ",(BYTE)chip->RamPositions.Shading);


		lpMotorTable = (unsigned char *) malloc (512 * 8 * 2);
		memset(lpMotorTable, 0, 512 * 8 * sizeof(WORD));
		
		ReadMotorTable(chip,lpMotorTable,wYResolution);
		WriteMotorTable(chip,lpMotorTable);

		free (lpMotorTable);

		
		isMotorMoveToFirstLine=MOTOR_MOVE_TO_FIRST_LINE_ENABLE;    
		isUniformSpeedToScan=UNIFORM_MOTOR_AND_SCAN_SPEED_DISABLE; 
		
	
		if(wXResolution == 1800)
		{
		    isScanBackTracking=SCAN_BACK_TRACKING_DISABLE;              
		    isMotorMoveToFirstLine=MOTOR_MOVE_TO_FIRST_LINE_ENABLE;  
			isUniformSpeedToScan=UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE;
		}
		else
			isScanBackTracking=SCAN_BACK_TRACKING_ENABLE;      

	
		Mustek_SendData(chip,ES01_A6_MotorOption, 
			MOTOR_1_ENABLE | 
			MOTOR_0_ENABLE |
			HOME_SENSOR_0_ENABLE |
			MOTOR_TYPE);
	
		Mustek_SendData(chip,ES01_F6_MorotControl1, SPEED_NUIT_1_PIXEL_TIME |
			MOTOR_SYNC_NUIT_1_PIXEL_TIME);
		
		MotorStepTableForScan(chip,wYResolution, wY, (DWORD)(chip->Scan.PicHeight));
		
		Mustek_SendData(chip,ES01_F8_DRAMType, DRAM_TIMING_1 | DRAM_256Kx16);
		
		Mustek_SendData(chip,ES01_A0_HostStartAddr0_7, 0x00);
		
		Mustek_SendData(chip,ES01_A1_HostStartAddr8_15, 0x00);
		Mustek_SendData(chip,ES01_A2_HostStartAddr16_19, 0x00 | ACCESS_DRAM);
		Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, chip->RamPositions.ImageEndAddr_0);
		Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, chip->RamPositions.ImageEndAddr_1);
		Mustek_SendData(chip,ES01_A5_HostEndAddr16_19, chip->RamPositions.ImageEndAddr_2);
		
		XDBG(DEBUG_ASIC,"Asic_SetWindow: chip->RamPositions.ImageEndAddr_0=%d\n"
						"\tchip->RamPositions.ImageEndAddr_1=%d\n"
						"\tchip->RamPositions.ImageEndAddr_2=%d",
						chip->RamPositions.ImageEndAddr_0,
						chip->RamPositions.ImageEndAddr_1,
						chip->RamPositions.ImageEndAddr_2);
		
		Mustek_SendData(chip,ES01_FB_BufferEmptySize16WordLSB, 0x00);
		Mustek_SendData(chip,ES01_FC_BufferEmptySize16WordMSB, 0x00);
		
		Mustek_SendData(chip,ES01_F9_BufferFullSize16WordLSB, chip->RamPositions.ImageFullBank_0);
		Mustek_SendData(chip,ES01_FA_BufferFullSize16WordMSB, chip->RamPositions.ImageFullBank_1);
		
		if(chip->isHardwareShading != SHADING_ENABLE)
			XDBG(DEBUG_ASIC,"The HardwareShading is false");
		else
			XDBG(DEBUG_ASIC,"The HardwareShading is true");
#ifdef _CLOSE_SHADING
		chip->isHardwareShading = SHADING_DISABLE;
#endif
		
		if(g_OutputRawData)
			chip->isHardwareShading = SHADING_DISABLE;

		switch(chip->Scan.ScanMode)
		{
		case 1:  /*gray 8 bit*/
			Mustek_SendData(chip,ES01_F5_ScanDataFormat, GRAY |
				_8_BITS |
				GRAY_GREEN|
				SHADING_2_INT_14_DEC |
				chip->isHardwareShading |
				GAMMA_DISABLE
				);
			break;
		case 2:  /*gray 16 bit*/
			Mustek_SendData(chip,ES01_F5_ScanDataFormat, GRAY |
				_16_BITS |
				GRAY_GREEN|
				SHADING_2_INT_14_DEC |
				chip->isHardwareShading |
				GAMMA_DISABLE
				);
			break;
		case 3:  /* color 8 bits*/
			Mustek_SendData(chip,ES01_F5_ScanDataFormat, COLOR |
				_8_BITS |
				GRAY_RED|
				SHADING_2_INT_14_DEC |
				chip->isHardwareShading |
				GAMMA_DISABLE
				);
			break;
		case 4:  /*color 16 bits*/
			Mustek_SendData(chip,ES01_F5_ScanDataFormat, COLOR |
				_16_BITS |
				GRAY_RED|
				SHADING_2_INT_14_DEC |
				chip->isHardwareShading |
				GAMMA_DISABLE
				);
			break;
		}
		Mustek_SendData(chip,ES01_FF_TestControl, BYPASS_AFE_DISABLE);
		
		Mustek_SendData(chip,ES01_F3_ActionOption, 
			isMotorMoveToFirstLine |
			MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
			SCAN_ENABLE |
			isScanBackTracking |
			INVERT_MOTOR_DIRECTION_DISABLE |
			isUniformSpeedToScan);
		XDBG(DEBUG_ASIC,"SetWindow: isMotorMoveToFirstLine=%d, isScanBackTracking=%d, isUniformSpeedToScan=%d",
		isMotorMoveToFirstLine, isScanBackTracking, isUniformSpeedToScan);
		
		chip->firmwarestate=FS_OPENED;
		
		return STATUS_GOOD;
}

STATUS Asic_Reset(PAsic chip)
{
	STATUS status;

	XDBG(DEBUG_ASIC,"Asic_Reset: Enter");

	wXShift=0;
	wYShift=0;
	ctCalibrateType=CT_FIRMWARE;
	wCalibrateWidth=5312;
	bCalibrateWhiteLength=0;
	bCalibrateDarkLength=0;
	bThreshold=128;
	isCompress=FALSE;
	isFromADF=FALSE;
	lsLightSource=LS_REFLECTIVE;
	wCalibrateShift=0;

	chip->dwBytesCountPerRow=0;
	chip->AD.DirectionR=0;
	chip->AD.DirectionG=1;
	chip->AD.DirectionB=1;
	chip->AD.GainR=40;
	chip->AD.GainG=40;
	chip->AD.GainB=40;
	chip->AD.OffsetR=120;
	chip->AD.OffsetG=40;
	chip->AD.OffsetB=50;

	chip->Scan.TotalMotorSteps=60000;
	chip->Scan.StartLine=0;
	chip->Scan.StartPixel=0;

	XDBG(DEBUG_ASIC,"Asic_Reset: Exit");

	return STATUS_GOOD;
}

STATUS Asic_SetSource(PAsic chip, LIGHTSOURCE lsLightSource)
{
	XDBG(DEBUG_ASIC,"Asic_SetSource: Enter");

	chip->lightsource = lsLightSource;
	switch(lsLightSource)
	{
	case 1:
		XDBG(DEBUG_ASIC, "Asic_SetSource: Source is Reflect");
		break;
	case 2:
		XDBG(DEBUG_ASIC, "Asic_SetSource: Source is Postion");
		break;
	case 4:
		XDBG(DEBUG_ASIC, "Asic_SetSource: Source is Negtive");
		break;
	default:
		XDBG(DEBUG_ASIC,"Asic_SetSource: Source error");
	}

	XDBG(DEBUG_ASIC,"Asic_SetSource: Exit");

	return STATUS_GOOD;
}


STATUS Asic_ScanStart(PAsic chip)
{

	XDBG(DEBUG_ASIC,"Asic_ScanStart: Enter");

	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_ScanStart: Scanner is not opened\n");
		return STATUS_INVAL;
	}

	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_ENABLE);
	Mustek_SetDMAReadScanMode(chip);	

	chip->firmwarestate = FS_SCANNING;

	XDBG(DEBUG_ASIC,"Asic_ScanStart: Exit");

	return STATUS_GOOD;
}

STATUS Asic_ScanStop(PAsic chip)
{
	STATUS status = STATUS_GOOD;

	BYTE temps[2];
	BYTE buf[5];  

	XDBG(DEBUG_ASIC,"Asic_ScanStop: Enter");

	if (chip->firmwarestate < FS_SCANNING)
	{
		return status;
	}
	
	buf[0] = 0x02;
	status = WriteIOControl(chip,0xc0,0,1,buf);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_ScanStop: Stop scan error\n");
		return status;
	}

	buf[0] = 0x00;
	status = WriteIOControl(chip,0xc0,0,1,buf);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_ScanStop: Clear scan error\n");
		return status;
	}

	Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	
	status = Mustek_DMAReadGeneralMode(chip,2, temps);
	if (status != STATUS_GOOD)
	{
		fprintf(stderr,"Asic_ScanStop: DMAReadGeneralMode error\n");
		return status;
	}

	Mustek_SendData (chip,ES01_94_PowerSaveControl, 0x01);
	Mustek_SendData (chip,ES01_92_TimerPowerSaveTime , 54);


	chip->firmwarestate = FS_OPENED;

	XDBG(DEBUG_ASIC,"Asic_ScanStop: Exit");

	return status;
}

STATUS Asic_ReadImage(PAsic chip, LPBYTE pBuffer, 
										WORD LinesCount)
{
	STATUS status = STATUS_GOOD;
	DWORD dwXferBytes;

	XDBG(DEBUG_ASIC,"Asic_ReadImage: Enter : LinesCount = %d",LinesCount);

	if (chip->firmwarestate != FS_SCANNING)
	{
		fprintf(stderr,"Asic_ReadImage: Scanner is not scanning\n");
		return STATUS_INVAL;
	}

	dwXferBytes = (DWORD)(LinesCount) *chip->dwBytesCountPerRow;
	
	XDBG(DEBUG_ASIC,"Asic_ReadImage: chip->dwBytesCountPerRow = %d",chip->dwBytesCountPerRow);
	
	if (dwXferBytes <0)
	{
		XDBG(DEBUG_ASIC,"Asic_ReadImage: dwXferBytes <0");
		return STATUS_INVAL;
	}
	
	if (dwXferBytes == 0)
	{
		XDBG(DEBUG_ASIC,"Asic_ReadImage: dwXferBytes == 0");
		return STATUS_GOOD;
	}

	status = Mustek_DMAReadScanMode(chip,dwXferBytes, pBuffer);
	if (status != STATUS_GOOD)
	{
		XDBG(DEBUG_ASIC,"Asic_ReadImage: Mustek_DMAReadScanMode return not Status good");
	}

	XDBG(DEBUG_ASIC,"Asic_ReadImage: Exit");
	return status;
}

STATUS Asic_ReadNoUseImage(PAsic chip, WORD wXferBytes)
{
	STATUS status = STATUS_GOOD;
	LPBYTE pBuffer;

	XDBG(DEBUG_ASIC,"Asic_ReadNoUseImage: Enter: wXferBytes=%d",wXferBytes);

	if (chip->firmwarestate != FS_SCANNING)
	{
		fprintf(stderr,"Asic_ReadNoUseImage: Scanner is not scanning\n");
		return STATUS_INVAL;
	}

	if(wXferBytes==0 || wXferBytes==512)
		return status; 

	pBuffer = (LPBYTE)malloc(wXferBytes);

	Mustek_DMAReadScanMode(chip, wXferBytes, pBuffer);

	free(pBuffer);

	XDBG(DEBUG_ASIC,"Asic_ReadNoUseImage: Exit");

	return status;
}

STATUS Asic_CheckFunctionKey(PAsic chip,LPBYTE key)
{
	STATUS status = STATUS_GOOD;
	BYTE bBuffer=0xff;

	XDBG(DEBUG_ASIC,"Asic_CheckFunctionKey: Enter");

	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_CheckFunctionKey: Scanner is not Opened\n");
		return STATUS_INVAL;
	}

	Mustek_SendData(chip,ES01_97_GPIOControl0_7, 0x24);
	Mustek_SendData(chip,ES01_95_GPIOValue0_7, 0xff);  
	Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x00);
	Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);

	Mustek_SendData(chip,ES01_8B_Status, 0x02);
	Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	bBuffer = ES01_8B_Status;
	Mustek_ReceiveData(chip,&bBuffer);

	*key = 0xff - bBuffer;

	XDBG(DEBUG_ASIC,"Asic_CheckFunctionKey: Exit : key=%d",*key);

	return status;
}

STATUS Asic_IsTAConnected(PAsic chip,PBOOL hasTA)
{
	STATUS status = STATUS_GOOD;
	BYTE bBuffer=0xff;
	BOOL isTAHome;

	XDBG(DEBUG_ASIC,"Asic_IsTAConnected: Enter");

	if (chip->TA_Status == TA_NOT_PLUGIN)
	{
		XDBG(DEBUG_ASIC,"Asic_IsTAConnected: No TA TA_Status = TA_NOT_PLUGIN");
		*hasTA = FALSE;
		return status;
	}
	else if	(chip->TA_Status == TA_PLUGIN)
	{
		XDBG(DEBUG_ASIC,"Asic_IsTAConnected: Have TA TA_Status = TA_PLUGIN");
		*hasTA = TRUE;
		return status;
	}
	
	XDBG(DEBUG_ASIC,"Testing TA...\n");
	status = Asic_IsTAHomePosition(chip, &isTAHome);
	if (status != STATUS_GOOD)
	{
		XDBG(DEBUG_ASIC,"Asic_IsTAConnected: No TA");
		*hasTA = FALSE;
		chip->TA_Status = TA_NOT_PLUGIN;
		return STATUS_GOOD;
	}
	if (!isTAHome)
	{
		status = DoCarriageHome(chip,TRUE);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"Asic_IsTAConnected: No TA");
			*hasTA = FALSE;
			chip->TA_Status = TA_NOT_PLUGIN;
			return STATUS_GOOD;
		}
		status = Asic_WaitCarriageHome(chip,TRUE);	
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"Asic_IsTAConnected: No TA");
			*hasTA = FALSE;
			chip->TA_Status = TA_NOT_PLUGIN;
			return STATUS_GOOD;
		}
	}

	XDBG(DEBUG_ASIC,"Asic_IsTAConnected: Have TA");
	*hasTA = TRUE;

	chip->TA_Status = TA_PLUGIN;
	
	return STATUS_GOOD;
}

STATUS Asic_DownloadGammaTable(PAsic chip, LPVOID lpBuffer)
{
	STATUS status = STATUS_GOOD;

	XDBG(DEBUG_ASIC,"Asic_DownloadGammaTable: Enter");
	
	chip->lpGammaTable = lpBuffer;

	XDBG(DEBUG_ASIC,"Asic_DownloadGammaTable: Exit");

	return status;
}

STATUS Asic_ReadCalibrationData(PAsic chip, LPVOID pBuffer, 
								DWORD dwXferBytes,BYTE bScanBits)
{
	STATUS status = STATUS_GOOD;

	LPBYTE	pCalBuffer;
	DWORD	dwTotalReadData;

	DWORD	dwReadImageData;

	XDBG(DEBUG_ASIC,"Asic_ReadCalibrationData: Enter");

	if (chip->firmwarestate != FS_SCANNING)
	{
		fprintf(stderr,"Asic_ReadCalibrationData: Scanner is not scanning\n");
		return STATUS_INVAL;
	}



	if(bScanBits==24)
	{
		DWORD i;
		pCalBuffer = (LPBYTE)malloc(dwXferBytes);
		if(pCalBuffer==NULL)
		{
			fprintf(stderr,"Asic_ReadCalibrationData: Can't malloc bCalBuffer memory\n");
			return STATUS_MEM_ERROR;

		}

		for(dwTotalReadData=0; dwTotalReadData<dwXferBytes; )
		{
			dwReadImageData=(dwXferBytes-dwTotalReadData)< 65536 ? 
				            (dwXferBytes-dwTotalReadData): 65536 ;
			
			Mustek_DMAReadScanMode(chip,dwReadImageData, (LPBYTE)(pCalBuffer+dwTotalReadData));
			dwTotalReadData+=dwReadImageData;
		}

		dwXferBytes/=3;
		for(i=0; i<dwXferBytes; i++)
		{
			*((LPBYTE)pBuffer+i)  =*(pCalBuffer+i*3);
			*((LPBYTE)pBuffer+dwXferBytes+i)=*(pCalBuffer+i*3+1);
			*((LPBYTE)pBuffer+dwXferBytes*2+i)=*(pCalBuffer+i*3+2);
		}
		free(pCalBuffer);
	}
	else if(bScanBits==8)
	{
		for(dwTotalReadData=0; dwTotalReadData<dwXferBytes; )
		{
			dwReadImageData=(dwXferBytes-dwTotalReadData)< 65536 ? 
				            (dwXferBytes-dwTotalReadData): 65536 ;
			
			Mustek_DMAReadScanMode(chip,dwReadImageData, (LPBYTE)pBuffer+dwTotalReadData);
			dwTotalReadData+=dwReadImageData;
		}
	}

	XDBG(DEBUG_ASIC,"Asic_ReadCalibrationData: Exit");

	return status;

}

STATUS Asic_CalculateMaxMin(PAsic chip, LPBYTE pBuffer,LPWORD lpMaxValue,
							LPWORD lpMinValue,WORD wResolution)
{
	STATUS status = STATUS_GOOD;

	int     nWhiteSecLength, nBlackSecLength; 
	int		nWhiteSecNum, nBlackSecNum;

	WORD	*wWhiteSecData=NULL, *wBlackSecData=NULL ;
	int		i,j;
	WORD	wCalWidth;
    WORD    wBlackCalWidth = 48; 

	WORD wBlackStartPosition = 14;


	XDBG(DEBUG_ASIC,"Asic_CalculateMaxMin: Enter");

	nBlackSecLength = 10;

	if(wResolution<=900)
	{
		wCalWidth=((10240*wResolution/900+511)>>9)<<9;

		
		if(wResolution<200)
			nWhiteSecLength = 10;
		else
			nWhiteSecLength = 20;

		wBlackCalWidth = wBlackCalWidth * wResolution / 1800;
		nBlackSecLength = nBlackSecLength * wResolution / 1800;

		if(nBlackSecLength <= 0)
			nBlackSecLength = 1;
	}
	else
	{
		nWhiteSecLength = 30;
		wCalWidth=10240;
	}
	
	nWhiteSecNum = (int)(wCalWidth/nWhiteSecLength);
	nBlackSecNum = (int)(wBlackCalWidth/nBlackSecLength);

	wWhiteSecData=(LPWORD)malloc(nWhiteSecNum*sizeof(WORD));
	wBlackSecData=(LPWORD)malloc(nBlackSecNum*sizeof(WORD));

	if(wWhiteSecData==NULL || wBlackSecData==NULL)
	{
		fprintf(stderr,"Asic_CalculateMaxMin: Malloc Memory Error\n");
		return STATUS_MEM_ERROR;
	}
	else
	{
		memset(wWhiteSecData,0,nWhiteSecNum*2);
		memset(wBlackSecData,0,nBlackSecNum*2);
	}

	wBlackStartPosition  = wBlackStartPosition * wResolution / 1800 + 1;
	
	for(i=0 ; i<nBlackSecNum ; i++){
		for (j=0 ;j<nBlackSecLength ; j++)
			wBlackSecData[i]+=*(pBuffer+i*nBlackSecLength+j + wBlackStartPosition);
		wBlackSecData[i]/=nBlackSecLength;
	}

	for(i=0 ; i<nWhiteSecNum ; i++){
		for (j=0 ;j<nWhiteSecLength ; j++)
			wWhiteSecData[i]+=*(pBuffer+i*nWhiteSecLength+j);
		wWhiteSecData[i]/=nWhiteSecLength;
	}

	*lpMaxValue=wWhiteSecData[0];
	*lpMinValue=wBlackSecData[0];
	
	for(i=0; i<nBlackSecNum; i++)
	{
		if(*lpMinValue>wBlackSecData[i])  *lpMinValue=wBlackSecData[i];
	}

	for(i=0; i<nWhiteSecNum; i++)
	{
		if(*lpMaxValue<wWhiteSecData[i])  *lpMaxValue=wWhiteSecData[i];
	}

	free(wWhiteSecData);
	free(wBlackSecData);

	XDBG(DEBUG_ASIC,"Asic_CalculateMaxMin: Exit");

	return status;
}

STATUS Asic_TACalculateMaxMin(PAsic chip, LPBYTE pBuffer,LPWORD lpMaxValue,
						 LPWORD lpMinValue,WORD wResolution)
{
	STATUS status = STATUS_GOOD;

	int     nWhiteSecLength, nBlackSecLength;
	int		nWhiteSecNum, nBlackSecNum;

	WORD	*wWhiteSecData=NULL, *wBlackSecData=NULL ; 
	int		i,j;
	WORD	wCalWidth;
    WORD    wBlackCalWidth = 48;

	WORD wBlackStartPosition = 14;

	XDBG(DEBUG_ASIC,"Asic_TACalculateMaxMin: Enter");

	nBlackSecLength = 10;

	if(wResolution<=900)
	{
		wCalWidth=((10240*wResolution/900+511)>>9)<<9;
		
		if(wResolution<200)
			nWhiteSecLength = 10;
		else
			nWhiteSecLength = 20;

		wBlackCalWidth = wBlackCalWidth * wResolution / 1800;
		nBlackSecLength = nBlackSecLength * wResolution / 1800;

		if(nBlackSecLength <= 0)
			nBlackSecLength = 1;
	}
	else
	{
		nWhiteSecLength = 30;
		wCalWidth=10240;
	}
	
	nWhiteSecNum = (int)(wCalWidth/nWhiteSecLength);
	nBlackSecNum = (int)(wBlackCalWidth/nBlackSecLength);

	wWhiteSecData=(LPWORD)malloc(nWhiteSecNum*sizeof(WORD));
	wBlackSecData=(LPWORD)malloc(nBlackSecNum*sizeof(WORD));

	if(wWhiteSecData==NULL || wBlackSecData==NULL)
	{
		fprintf(stderr,"Asic_TACalculateMaxMin: Malloc Memory Error\n");
		return STATUS_MEM_ERROR;
	}
	else
	{
		memset(wWhiteSecData,0,nWhiteSecNum*2);
		memset(wBlackSecData,0,nBlackSecNum*2);
	}

	wBlackStartPosition  = wBlackStartPosition * wResolution / 1800 + 1;
	
	for(i=0 ; i<nBlackSecNum ; i++){
		for (j=0 ;j<nBlackSecLength ; j++)
			wBlackSecData[i]+=*(pBuffer+i*nBlackSecLength+j + wBlackStartPosition);
		wBlackSecData[i]/=nBlackSecLength;

	}

	for(i=0 ; i<nWhiteSecNum ; i++){
		for (j=0 ;j<nWhiteSecLength ; j++)
			wWhiteSecData[i]+=*(pBuffer+i*nWhiteSecLength+j); 
		wWhiteSecData[i]/=nWhiteSecLength;
	}

	*lpMaxValue=wWhiteSecData[0];
	*lpMinValue=wBlackSecData[0];
	
	for(i=0; i<nBlackSecNum; i++)
	{
		if(*lpMinValue>wBlackSecData[i])  *lpMinValue=wBlackSecData[i];
	}

	for(i=0; i<nWhiteSecNum; i++)
	{
		if(*lpMaxValue<wWhiteSecData[i])  *lpMaxValue=wWhiteSecData[i];
	}

	free(wWhiteSecData);
	free(wBlackSecData);
	

	XDBG(DEBUG_ASIC,"Asic_TACalculateMaxMin: Exit");
	return status;
}

STATUS Asic_SetADConverter(PAsic chip)
{
	STATUS status = STATUS_GOOD;

	XDBG(DEBUG_ASIC,"Asic_SetADConverter: Enter");
	XDBG(DEBUG_ASIC,"Asic_SetADConverter: GainR=%d, GainG=%d, GainB=%d\n"
		"\tOffsetR=%d, OffsetG=%d OffsetB=%d",
		chip->AD.GainR,chip->AD.GainG,chip->AD.GainB,
		chip->AD.OffsetR,chip->AD.OffsetG,chip->AD.OffsetB);

	if (chip->firmwarestate < FS_OPENED)
		OpenScanChip(chip);
	if (chip->firmwarestate > FS_OPENED)
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);

#ifdef _DO_HARDWARE_SETTING_
	Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, chip->Timing.HARDWARE_SETTING);
#endif
	
	Mustek_SendData(chip,ES01_9A_AFEControl, chip->Timing.AFEControl);
	
	Mustek_SendData(chip,ES01_81_AFE_CHANNEL_RED_LATCH_POS, chip->Timing.AFE_CHANNEL_RED_LATCH_POS);
	Mustek_SendData(chip,ES01_82_AFE_CHANNEL_GREEN_LATCH_POS, chip->Timing.AFE_CHANNEL_GREEN_LATCH_POS);
	Mustek_SendData(chip,ES01_83_AFE_CHANNEL_BLUE_LATCH_POS, chip->Timing.AFE_CHANNEL_BLUE_LATCH_POS);	
	Mustek_SendData(chip,ES01_84_AFE_MALC_TIMING_ADJUST_0, chip->Timing.AFE_MALC_TIMING_ADJUST_0);
	Mustek_SendData(chip,ES01_85_AFE_MALC_TIMING_ADJUST_1, chip->Timing.AFE_MALC_TIMING_ADJUST_1);
	
	Mustek_SendData(chip,ES01_00_AD9826Configuration, chip->Timing.AD9826Configuration);
	Mustek_SendData(chip,ES01_02_AD9826MuxConfig, chip->Timing.AD9826MuxConfig);
	
	Mustek_SendData(chip,ES01_04_AD9826PGARed, chip->AD.GainR);
	Mustek_SendData(chip,ES01_06_AD9826PGAGreen, chip->AD.GainG);
	Mustek_SendData(chip,ES01_08_AD9826PGABlue, chip->AD.GainB);
	
	if (chip->AD.DirectionR)
		Mustek_SendData(chip,ES01_0B_AD9826OffsetRedN, chip->AD.OffsetR);
	else
		Mustek_SendData(chip,ES01_0A_AD9826OffsetRedP, chip->AD.OffsetR);
	
	if (chip->AD.DirectionG)
		Mustek_SendData(chip,ES01_0D_AD9826OffsetGreenN, chip->AD.OffsetG);
	else
		Mustek_SendData(chip,ES01_0C_AD9826OffsetGreenP, chip->AD.OffsetG);
	
	if (chip->AD.DirectionB)
		Mustek_SendData(chip,ES01_0F_AD9826OffsetBlueN, chip->AD.OffsetB);
	else
		Mustek_SendData(chip,ES01_0E_AD9826OffsetBlueP, chip->AD.OffsetB);
	
	Mustek_SendData(chip,ES01_A0_HostStartAddr0_7, 0x00);
	Mustek_SendData(chip,ES01_A1_HostStartAddr8_15, 0x00);
	Mustek_SendData(chip,ES01_A2_HostStartAddr16_19, 0x00 | ACCESS_DRAM);
	
	Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, chip->RamPositions.ImageEndAddr_0);
	Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, chip->RamPositions.ImageEndAddr_1);
	Mustek_SendData(chip,ES01_A5_HostEndAddr16_19, chip->RamPositions.ImageEndAddr_2);
	
	chip->firmwarestate = FS_OPENED;


	XDBG(DEBUG_ASIC,"Asic_SetADConverter: Exit");
	return status;
}

STATUS Asic_SetMotorType(PAsic chip, 
						BOOL isMotorMove,BOOL isUniformSpeed)
{
	STATUS status = STATUS_GOOD;

	XDBG(DEBUG_ASIC,"Asic_SetMotorType: Enter");

	if(isMotorMove)
	{
		chip->isMotorMove=MOTOR_1_ENABLE; 
	}
	else
		chip->isMotorMove=MOTOR_1_DISABLE; 

	XDBG(DEBUG_ASIC,"Asic_SetMotorType: Exit");
	return status;
}

STATUS Asic_MotorMove(PAsic chip, BOOL isForward,DWORD dwTotalSteps)
{
	STATUS status = STATUS_GOOD;
	BOOL isUniformSpeedForward = FALSE;
	LPBYTE lpMotorTable;
	DWORD dwForwardSteps;

	XDBG(DEBUG_ASIC,"Asic_MotorMove: Enter");

#ifdef _TA_TESTAP
    chip->lightsource = LS_POSITIVE;
#endif



	if(isForward && (dwTotalSteps < 752))          
		isUniformSpeedForward = TRUE;

	Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);

	Mustek_SendData(chip,ES01_97_GPIOControl0_7, 0x7c);

	if(chip->lightsource == LS_REFLECTIVE)
	{
		Mustek_SendData(chip,ES01_95_GPIOValue0_7, GPIO_95_Config);
		XDBG(DEBUG_ASIC,"GPIO_95_Config = %x", GPIO_95_Config);
	}
	else
	{
		Mustek_SendData(chip,ES01_95_GPIOValue0_7, GPIO_95_Config & 0xdf);
		XDBG(DEBUG_ASIC,"GPIO_95_Config = %x", GPIO_95_Config & 0xdf);
	}


	Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x00);
	Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00); 



	lpMotorTable = (unsigned char *) malloc (512 * 8 * 2);
	memset(lpMotorTable, 0, 512 * 8 * sizeof(WORD));

#ifdef _DO_HARDWARE_SETTING_
	Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, chip->Timing.HARDWARE_SETTING);
#endif

	chip->ibase1=2000;
	chip->ibase2=600;
	ReadMotorTable(chip,lpMotorTable,0);
	Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(800));
	Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(800));


	Mustek_SendData(chip,ES01_A6_MotorOption, 
		                             MOTOR_1_ENABLE |
									 MOTOR_0_ENABLE |
									 HOME_SENSOR_0_ENABLE |
									 MOTOR_TYPE);


	Mustek_SendData(chip,ES01_F6_MorotControl1, SPEED_NUIT_1_PIXEL_TIME |
									   MOTOR_SYNC_NUIT_1_PIXEL_TIME);


	if(isForward)
	{
		if(isUniformSpeedForward)
		{
			Mustek_SendData(chip,ES01_E0_MotorAccStep0_7, LOBYTE(0));
			Mustek_SendData(chip,ES01_E1_MotorAccStep8_8, HIBYTE(0));

			Mustek_SendData(chip,ES01_E5_MotorDecStep, 0);
		}
		else
		{
			Mustek_SendData(chip,ES01_E0_MotorAccStep0_7, LOBYTE(504));
			Mustek_SendData(chip,ES01_E1_MotorAccStep8_8, HIBYTE(504));

			Mustek_SendData(chip,ES01_E5_MotorDecStep, 248);
		}
	}
	else
	{
		Mustek_SendData(chip,ES01_E0_MotorAccStep0_7, LOBYTE(0));
		Mustek_SendData(chip,ES01_E1_MotorAccStep8_8, HIBYTE(0));

		Mustek_SendData(chip,ES01_E5_MotorDecStep, 0);
	}

	if(isForward)
		if(isUniformSpeedForward)
			dwForwardSteps=dwTotalSteps;
		else
			dwForwardSteps=dwTotalSteps-504-248;
	else
		dwForwardSteps=dwTotalSteps;

	Mustek_SendData(chip,ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(dwForwardSteps));
	Mustek_SendData(chip,ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(dwForwardSteps));
	Mustek_SendData(chip,ES01_E4_MotorStepOfMaxSpeed16_19, (BYTE)((dwForwardSteps & 0x00ff0000) >> 16));

	Mustek_SendData(chip,ES01_F0_ScanImageStep0_7, LOBYTE(dwTotalSteps));
	Mustek_SendData(chip,ES01_F1_ScanImageStep8_15, HIBYTE(dwTotalSteps));
	Mustek_SendData(chip,ES01_F2_ScanImageStep16_19, (BYTE) (((dwTotalSteps) & 0x00ff0000) >> 16));

	WriteMotorTable(chip,lpMotorTable);

	if(isForward)
	{
		if(isUniformSpeedForward)
			Mustek_SendData(chip,ES01_F3_ActionOption, 
		                                 MOTOR_MOVE_TO_FIRST_LINE_ENABLE |
										 MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
									     SCAN_DISABLE |
									     SCAN_BACK_TRACKING_DISABLE |
									     INVERT_MOTOR_DIRECTION_ENABLE |
				 					     UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE|
									     MOTOR_TEST_LOOP_DISABLE);

		else
			Mustek_SendData(chip,ES01_F3_ActionOption, 
		                                 MOTOR_MOVE_TO_FIRST_LINE_ENABLE |
									     MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
									     SCAN_DISABLE |
									     SCAN_BACK_TRACKING_DISABLE |
									     INVERT_MOTOR_DIRECTION_DISABLE |
				 					     UNIFORM_MOTOR_AND_SCAN_SPEED_DISABLE|
									     MOTOR_TEST_LOOP_DISABLE);
	}
	else
		Mustek_SendData(chip,ES01_F3_ActionOption, 
                                     MOTOR_MOVE_TO_FIRST_LINE_ENABLE |
									 MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
									 SCAN_DISABLE |
									 SCAN_BACK_TRACKING_DISABLE |
									 INVERT_MOTOR_DIRECTION_ENABLE |
				 					 UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE|
									 MOTOR_TEST_LOOP_DISABLE);


	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_ENABLE);

	free(lpMotorTable);


	chip->motorstate = MS_MOVED;

	Asic_WaitCarriageHome(chip,FALSE);

	XDBG(DEBUG_ASIC,"Asic_MotorMove: Exit");
	return status;
}

STATUS Asic_CarriageHome(PAsic chip, BOOL isTA)
{
	STATUS status = STATUS_GOOD;
	BOOL LampHome, TAHome;
	BOOL hasTA;
	LIGHTSOURCE ls = chip->lightsource;
	
	XDBG(DEBUG_ASIC,"Asic_CarriageHome: Enter : isTA=%d",isTA);
	
	Asic_IsTAConnected(chip, &hasTA);
	
	
	if (hasTA)
	{
		XDBG(DEBUG_ASIC,"Asic_CarriageHome: has TA");
		IsCarriageHome(chip,&LampHome,&TAHome);
		if (LampHome && TAHome)
		{
			XDBG(DEBUG_ASIC,"Asic_CarriageHome: All is home");
			return STATUS_GOOD;
		}
	
		if (!LampHome && !TAHome)
		{
			XDBG(DEBUG_ASIC,"Asic_CarriageHome: None in home");
			chip->lightsource = LS_POSITIVE;
			DoCarriageHome(chip,FALSE);

			TALAMPWaitCarriageHome(chip);
			chip->lightsource = ls;
		}
	
		IsCarriageHome(chip,&LampHome,&TAHome);
		if (LampHome && TAHome)
			return STATUS_GOOD;
	
		if (!LampHome)
		{
			chip->lightsource = LS_REFLECTIVE;
			DoCarriageHome(chip,FALSE);
			Asic_WaitCarriageHome(chip,FALSE);
			chip->lightsource = ls;
		}
	
		if (!TAHome)
		{
			chip->lightsource = LS_POSITIVE;
			DoCarriageHome(chip,TRUE);
			Asic_WaitCarriageHome(chip,TRUE);
			chip->lightsource = ls;
		}
	}
	else
	{
		XDBG(DEBUG_ASIC,"Asic_CarriageHome: No TA");
		IsCarriageHome(chip,&LampHome,&TAHome);
		if (!LampHome)
		{
			DoCarriageHome(chip,FALSE);
			Asic_WaitCarriageHome(chip,FALSE);

		}
	}
	
	
	
	XDBG(DEBUG_ASIC,"Asic_CarriageHome: Exit");
	return status;
}

STATUS Asic_SetShadingTable(PAsic chip, LPWORD lpWhiteShading, 
								LPWORD lpDarkShading, 
								WORD wXResolution, WORD wWidth, 
								WORD wX)
{
	STATUS status = STATUS_GOOD;
	LPBYTE	lpShadingTable;
	WORD i,j,n;
	WORD wIndex=1;

	XDBG(DEBUG_ASIC,"Asic_SetShadingTable: Enter:Shading StartX=%d",wX);

	if(chip->firmwarestate < FS_OPENED)
		OpenScanChip(chip);

	if(chip->firmwarestate==FS_SCANNING)
		Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);

	lpShadingTable = (LPBYTE)malloc(chip->Temp.Shading_Table_Size);

	n = 0;
	for (i = 0 ; i <= (wWidth / 85); i++)
	{
		if( i < (wWidth / 85))
		{			
			for (j = 0; j < 85; j++)
			{
				*((LPWORD)lpShadingTable + i * 512 + j * 6	  ) = *((LPWORD)lpDarkShading +(wX + n) * 3);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 2) = *((LPWORD)lpDarkShading +(wX + n) * 3 + 1);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 4) = *((LPWORD)lpDarkShading +(wX + n) * 3 + 2);

				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 1) = *((LPWORD)lpWhiteShading +(wX + n) * 3);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 3) = *((LPWORD)lpWhiteShading +(wX + n) * 3 + 1);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 5) = *((LPWORD)lpWhiteShading +(wX + n) * 3 + 2);
				n++;
			}
		}
		else
		{
			for (j = 0; j < (wWidth % 85); j++)
			{
				*((LPWORD)lpShadingTable + i * 512 + j * 6	  ) = *((LPWORD)lpDarkShading +(wX + n) * 3);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 2) = *((LPWORD)lpDarkShading +(wX + n) * 3 + 1);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 4) = *((LPWORD)lpDarkShading +(wX + n) * 3 + 2);

				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 1) = *((LPWORD)lpWhiteShading +(wX + n) * 3);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 3) = *((LPWORD)lpWhiteShading +(wX + n) * 3 + 1);
				*((LPWORD)lpShadingTable + i * 512 + j * 6 + 5) = *((LPWORD)lpWhiteShading +(wX + n) * 3 + 2);

				n++;
			}
		}
	}

	Mustek_SendData(chip,ES01_F8_DRAMType, DRAM_TIMING_1 | DRAM_256Kx16);

	Mustek_SendData(chip,ES01_A0_HostStartAddr0_7, chip->RamPositions.Shading_0);
	Mustek_SendData(chip,ES01_A1_HostStartAddr8_15, chip->RamPositions.Shading_1);
	Mustek_SendData(chip,ES01_A2_HostStartAddr16_19, chip->RamPositions.Shading_2 | ACCESS_DRAM);

	Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, 0xff);
	Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, 0xff);
	Mustek_SendData(chip,ES01_A5_HostEndAddr16_19, 0xff);

	Mustek_DMAWrite(chip,chip->Temp.Shading_Table_Size, (LPBYTE)(lpShadingTable));
	free(lpShadingTable);

	/*Steal read 2 byte,by Roy 2005.10.7*/
	usleep(2000);
	BYTE temps[2];
	Mustek_DMAReadGeneralMode(chip, 2, temps);  

	chip->firmwarestate = FS_OPENED;

	XDBG(DEBUG_ASIC,"Asic_SetShadingTable: Exit");

	return status;
}

STATUS Asic_SetSoftwareShadingTable(PAsic chip, LPWORD lpWhiteShading, LPWORD lpDarkShading, WORD wXResolution, WORD wWidth, WORD wX)
{
	STATUS status = STATUS_GOOD;
	DWORD SizeOfTable;

	XDBG(DEBUG_ASIC,"Asic_SetSoftwareShadingTable: Enter");

	if(lpWhiteShadingTable)
		free(lpWhiteShadingTable);
	if(lpDarkShadingTable)
		free(lpDarkShadingTable);
	
	SizeOfTable = wWidth * 3;   /* word*/

	lpWhiteShadingTable = (LPWORD)malloc(SizeOfTable*sizeof(WORD));
	if(lpWhiteShadingTable == NULL)
	{
		fprintf(stderr,"Asic_SetSoftwareShadingTable: Can't malloc memory\n");
		return STATUS_MEM_ERROR;
	}

	lpDarkShadingTable = (LPWORD)malloc(SizeOfTable*sizeof(WORD));
	if(lpDarkShadingTable == NULL)
	{
		free(lpWhiteShadingTable);
		lpWhiteShadingTable = NULL;
		fprintf(stderr,"Asic_SetSoftwareShadingTable: Can't malloc memory\n");
		return STATUS_MEM_ERROR;
	}

	memcpy(lpWhiteShadingTable, lpWhiteShading, SizeOfTable * 2);
	memcpy(lpDarkShadingTable, lpDarkShading, SizeOfTable * 2);

	XDBG(DEBUG_ASIC,"Asic_SetSoftwareShadingTable: Exit");

	return status;
}

STATUS Asic_DoSoftwareShadingRGB24Bit(PAsic chip, LPBYTE lpImageHead, WORD wWidth, WORD wStartLines, WORD wNumberOfLines)
{
	STATUS status = STATUS_GOOD;
	LPBYTE lpBuffer;
	WORD wWhiteShading, wDarkShading;
	float fTemp;
	DWORD dwBytePerLine;
	DWORD i,j;

	XDBG(DEBUG_ASIC,"Asic_DoSoftwareShadingRGB24Bit: Enter");

	lpBuffer = lpImageHead + wWidth * wStartLines * 3;



	dwBytePerLine = (DWORD) wWidth * 3;

	for(i = 0; i < wNumberOfLines; i++)
	{
		for(j = 0; j < dwBytePerLine; j++)
		{
			wWhiteShading = lpWhiteShadingTable[j] ;
			wDarkShading = lpDarkShadingTable[j];

			fTemp = (float) (*lpBuffer - (BYTE) (wDarkShading >> 8));

			if(fTemp <= 0.0)
				*lpBuffer = 0;
			else
			{
				fTemp = fTemp / (float)(wWhiteShading>>8) * 256.0;

				if(fTemp < 0.0) *lpBuffer = 0;
				else
				{
					if(fTemp > 255.0)
						*lpBuffer = 255;
					else
						*lpBuffer = (BYTE)fTemp;
				}
			}
			lpBuffer++;
		}
	}

	XDBG(DEBUG_ASIC,"Asic_DoSoftwareShadingRGB24Bit: Exit");

	return status;
}

STATUS Asic_DoSoftwareShadingGray8Bit(PAsic chip, LPBYTE lpImageHead, WORD wWidth, WORD wStartLines, WORD wNumberOfLines)
{
	STATUS status = STATUS_GOOD;
	
	LPBYTE lpBuffer = lpImageHead + wWidth * wStartLines;

	WORD wWhiteShading, wDarkShading;
	float fTemp;
	DWORD i,j;

	XDBG(DEBUG_ASIC,"DoSoftwareShadingGray8Bit: Enter");

	for(i = 0; i < wNumberOfLines; i++)
	{
		for(j = 0; j < wWidth ; j++)
		{
			wWhiteShading = lpWhiteShadingTable[j * 3 + 1] ;
			wDarkShading = lpDarkShadingTable[j * 3 + 1];

			fTemp = (float) (*lpBuffer - (BYTE) (wDarkShading >> 8));
			fTemp = fTemp * 255.0 / (float)(wWhiteShading>>8);

			if(fTemp < 0.0)
				*lpBuffer = 0;
			else
			{
				if(fTemp > 255.0)
					*lpBuffer = 255;
				else
					*lpBuffer = (BYTE)(fTemp + 0.5);
			}

			lpBuffer++;
		}
	}

	XDBG(DEBUG_ASIC,"DoSoftwareShadingGray8Bit: Exit");

	return status;
}

STATUS Asic_DoSoftwareShadingRGB48Bit(PAsic chip, LPBYTE lpImageHead, WORD wWidth, WORD wStartLines, WORD wNumberOfLines)
{
	STATUS status = STATUS_GOOD;
	LPBYTE lpBuffer = lpImageHead + wWidth * wStartLines * 6;

	WORD wWhiteShading, wDarkShading;
	float fTemp;
	WORD wData;

	DWORD dwBytePerLine = (DWORD) wWidth * 3;
	DWORD i, j;


	XDBG(DEBUG_ASIC,"Asic_DoSoftwareShadingRGB48Bit: Enter");

	for(i = 0; i < wNumberOfLines; i++)
	{
		for(j = 0; j < dwBytePerLine; j++)
		{
			wWhiteShading = lpWhiteShadingTable[j] ;
			wDarkShading = lpDarkShadingTable[j];

			wData = (WORD)*lpBuffer;     /* get low byte data*/
			lpBuffer++;
			wData = wData  + (((WORD)*lpBuffer) << 8);

			fTemp = (float) (wData - wDarkShading);
			fTemp = fTemp / (float)(wWhiteShading) * 65536.0;

			if(fTemp < 0.0)
			{
				*lpBuffer = 0;
			     lpBuffer--;
				*lpBuffer = 0;
			}
			else
			{
				if(fTemp > 65535.0)
				{
					*lpBuffer = 255;
					lpBuffer--;
					*lpBuffer = 255;
				}
				else
				{
					*lpBuffer = HIBYTE((WORD)fTemp);
					lpBuffer--;
					*lpBuffer = LOBYTE((WORD)fTemp);
				}
			}


			lpBuffer = lpBuffer + 2;
		}
	}
	

	XDBG(DEBUG_ASIC,"Asic_DoSoftwareShadingRGB48Bit: Exit");
	return status;
}

STATUS Asic_DoSoftwareShadingGray16Bit(PAsic chip, LPBYTE lpImageHead, WORD wWidth, WORD wStartLines, WORD wNumberOfLines)
{
	STATUS status = STATUS_GOOD;
	LPBYTE lpBuffer = lpImageHead + wWidth * wStartLines * 2;

	WORD wWhiteShading, wDarkShading;
	WORD wData;
	float fTemp;
	DWORD i,j;

	XDBG(DEBUG_ASIC,"Asic_DoSoftwareShadingGray16Bit: Enter");

		for(i = 0; i < wNumberOfLines; i++)
	{
		for(j = 0; j < wWidth ; j++)
		{
			wWhiteShading = lpWhiteShadingTable[j * 3 + 1] ;
			wDarkShading = lpDarkShadingTable[j * 3 + 1];

			wData = (WORD)*lpBuffer;    
			lpBuffer++;
			wData = wData  + (((WORD)*lpBuffer) << 8);

			fTemp = (float) (wData - wDarkShading );
			fTemp = fTemp / ((float) wWhiteShading ) * 65536;

			if(fTemp < 0.0)
			{
				*lpBuffer = 0;   
			     lpBuffer--;
				*lpBuffer = 0;  
			}
			else
			{
				if(fTemp > 65535.0)
				{
					*lpBuffer = 255;  
					lpBuffer--;
					*lpBuffer = 255; 
				}
				else
				{
					*lpBuffer = HIBYTE((WORD)fTemp);
					lpBuffer--;
					*lpBuffer = LOBYTE((WORD)fTemp);
				}
			}

			lpBuffer = lpBuffer + 2;


		}
	}	

	XDBG(DEBUG_ASIC,"Asic_DoSoftwareShadingGray16Bit: Exit");
	return status;
}

STATUS Asic_WaitCarriageHome(PAsic chip, BOOL isTA)
{
	STATUS status = STATUS_GOOD;

	BYTE temp;
	time_t time_start;
	time_t time_end;

	XDBG(DEBUG_ASIC, "Asic_WaitCarriageHome: Enter : isTA=%d",isTA);

	if(chip->motorstate == MS_STILL)
		return status;

	if (isTA)
		ResetTAGpio(chip);
	
	time_start = time(0);
	do{
		if (isTA)
		{
			status = Mustek_SendData(chip,ES01_8B_Status, 0x02);
		}
		else
		{
			status = Mustek_SendData(chip,ES01_8B_Status, 0x01);
		}
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"Asic_WaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
		status = Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"Asic_WaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
		temp = ES01_8B_Status;
		status = Mustek_ReceiveData(chip,&temp);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"Asic_WaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
		if (!isTA)
			usleep(200000);
		
		if (isTA)

		{
			time_end = time(0);
			if ((time_end - time_start) > 20)
			{
				return STATUS_INVAL;
			}
		}
	} while (isTA?((temp&0x10)== 0):((temp&0x1f)!=0));
	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);


	chip->firmwarestate = FS_OPENED;
	chip->motorstate = MS_STILL;

	XDBG(DEBUG_ASIC,"Asic_WaitCarriageHome: Exit");
	return status;
}


STATUS Asic_IsTAHomePosition(PAsic chip, PBOOL isTAHome)
{
	STATUS status = STATUS_GOOD;
	BYTE temp;
	XDBG(DEBUG_ASIC,"Asic_IsTAHomePosition: Enter");
	
	
	Mustek_SendData(chip,ES01_8B_Status, 0x02);
	Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	temp = ES01_8B_Status;
	status = Mustek_ReceiveData(chip,&temp);
	if(STATUS_GOOD != status)
	{
		fprintf(stderr,"Asic_IsTAHomePostion: CarriageHome Failed\n");
		return STATUS_IO_ERROR;
	}

	if((temp & 0x10))
		*isTAHome = TRUE;
	else 
	{
		*isTAHome = FALSE;
		chip->firmwarestate = FS_OPENED;
		chip->motorstate = MS_STILL;
	}

	XDBG(DEBUG_ASIC,"Asic_IsTAHomePosition: Exit");
	return status;
}

STATUS Asic_SetCalibrate(PAsic chip,BYTE bScanBits, WORD wXResolution, 
						 WORD wYResolution, WORD wX, WORD wY, 
						 WORD wWidth, WORD wLength, BOOL isShading)
{
	STATUS status = STATUS_GOOD;
	DWORD TotalStep; 

	XDBG(DEBUG_ASIC,"Asic_SetCalibrate: Enter");
	
	if (chip->firmwarestate != FS_OPENED)
	{
		fprintf(stderr,"Asic_SetCalibrate: Scanner is not opened\n");
		return STATUS_INVAL;
	}

	chip->Scan.StartPixel = wX;
	chip->Scan.StartLine  = wY;

	switch (wXResolution)
	{
	case 1800:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,1800);
		break;

	case 900:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_900_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,900);
		break;
	case 450:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_450_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,450);
		break;
	case 225:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_225_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,225);
		break;
	case 112:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_112_PIXEL_WIDTH; 
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,112);
		break;
	case 72:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_72_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,72);
		break;
	case 56:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_56_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,56);
		break;

	case 50:
		chip->Scan.CCD_Pixel_Length	= CALIBRATION_PIXEL_WIDTH;
		chip->Scan.PicWidth	= wWidth;
		SetCalibrateParam(chip,50);
		break;
	}

	chip->Scan.PicHeight  = wLength * 1800 / wYResolution;
	chip->Scan.LongX		= chip->Scan.PicWidth;

	if(bScanBits==24 || bScanBits==48)
		chip->Scan.ScanMode = 3;
	else if(bScanBits==8)
		chip->Scan.ScanMode = 1;

	if(isShading)
	{
		chip->Temp.Shading_Table_Size	= (((int)(wWidth * 3 * 2 / 510) * 512) + ((wWidth * 3 * 2) % 510) ) * 2;
	}
	else
	{
		chip->Temp.Shading_Table_Size = 0;
	}

	chip->RamPositions.Shading	= 64 - (chip->Temp.Shading_Table_Size / 8192);
	if((chip->Temp.Shading_Table_Size % 8192) != 0)
		chip->RamPositions.Shading--;

	chip->RamPositions.Shading_0	= 0;
	chip->RamPositions.Shading_1	= (BYTE) ((chip->RamPositions.Shading << 4) & 0xf0);
	chip->RamPositions.Shading_2	= (BYTE) ((chip->RamPositions.Shading >> 4) & 0x0f);

	chip->RamPositions.Motor		= chip->RamPositions.Shading - 1;

	chip->RamPositions.Motor_0	= 0;
	chip->RamPositions.Motor_1	= (BYTE) ((chip->RamPositions.Motor << 4) & 0xf0);
	chip->RamPositions.Motor_2	= (BYTE) ((chip->RamPositions.Motor >> 4) & 0x0f);
	
	chip->Temp.Image_Buffer_Size	= chip->RamPositions.Motor * 4096 - 1;  

	chip->RamPositions.ImageEndAddr_0	= LOBYTE(chip->Temp.Image_Buffer_Size);
	chip->RamPositions.ImageEndAddr_1	= HIBYTE(chip->Temp.Image_Buffer_Size);
	chip->RamPositions.ImageEndAddr_2	= (BYTE)((chip->Temp.Image_Buffer_Size & 0x00ff0000) >> 16);
	chip->Temp.Full_Bank = (chip->Temp.Image_Buffer_Size - ((chip->Scan.LongX * chip->Scan.ScanMode) * 2 + 2)) / 16;/*1 Bank = 16 Words*/

	chip->RamPositions.ImageFullBank_0 = LOBYTE(chip->Temp.Full_Bank);
	chip->RamPositions.ImageFullBank_1 = HIBYTE(chip->Temp.Full_Bank);

	SafeInitialChip(chip);
	Mustek_SendData(chip,ES01_97_GPIOControl0_7, 0x7c);

	if(chip->lightsource == LS_REFLECTIVE)
	{
		Mustek_SendData(chip,ES01_95_GPIOValue0_7, GPIO_95_Config);
	}
	else
	{
		Mustek_SendData(chip,ES01_95_GPIOValue0_7, GPIO_95_Config & 0xdf); 
	}

	Mustek_SendData(chip,ES01_98_GPIOControl8_15, 0x00);
	Mustek_SendData(chip,ES01_96_GPIOValue8_15, 0x00);

	SetLineTimeAndExposure(chip);	
	if(wXResolution > 50)
	    chip->Timing.CCD_SETUP_REGISTER = CCD_SETUP_REGISTERIN1800; 
	else
	    chip->Timing.CCD_SETUP_REGISTER = CCD_SETUP_REGISTERIN900; 
	CCDTiming(chip);
#ifdef _DO_HARDWARE_SETTING_
	Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, chip->Timing.HARDWARE_SETTING);
#endif

	Mustek_SendData(chip,ES01_9A_AFEControl, chip->Timing.AFEControl);

	Mustek_SendData(chip,ES01_81_AFE_CHANNEL_RED_LATCH_POS, chip->Timing.AFE_CHANNEL_RED_LATCH_POS);
	Mustek_SendData(chip,ES01_82_AFE_CHANNEL_GREEN_LATCH_POS, chip->Timing.AFE_CHANNEL_GREEN_LATCH_POS);
	Mustek_SendData(chip,ES01_83_AFE_CHANNEL_BLUE_LATCH_POS, chip->Timing.AFE_CHANNEL_BLUE_LATCH_POS);	
	Mustek_SendData(chip,ES01_84_AFE_MALC_TIMING_ADJUST_0, chip->Timing.AFE_MALC_TIMING_ADJUST_0);
	Mustek_SendData(chip,ES01_85_AFE_MALC_TIMING_ADJUST_1, chip->Timing.AFE_MALC_TIMING_ADJUST_1);

	Mustek_SendData(chip,ES01_00_AD9826Configuration, chip->Timing.AD9826Configuration);
	Mustek_SendData(chip,ES01_02_AD9826MuxConfig, chip->Timing.AD9826MuxConfig);

	Mustek_SendData(chip,ES01_04_AD9826PGARed, chip->AD.GainR);
	Mustek_SendData(chip,ES01_06_AD9826PGAGreen, chip->AD.GainG);
	Mustek_SendData(chip,ES01_08_AD9826PGABlue, chip->AD.GainB);

	if (chip->AD.DirectionR)
		Mustek_SendData(chip,ES01_0B_AD9826OffsetRedN, chip->AD.OffsetR);
	else
		Mustek_SendData(chip,ES01_0A_AD9826OffsetRedP, chip->AD.OffsetR);

	if (chip->AD.DirectionG)
		Mustek_SendData(chip,ES01_0D_AD9826OffsetGreenN, chip->AD.OffsetG);
	else
		Mustek_SendData(chip,ES01_0C_AD9826OffsetGreenP, chip->AD.OffsetG);

	if (chip->AD.DirectionB)
		Mustek_SendData(chip,ES01_0F_AD9826OffsetBlueN, chip->AD.OffsetB);
	else
		Mustek_SendData(chip,ES01_0E_AD9826OffsetBlueP, chip->AD.OffsetB);

	Mustek_SendData(chip,ES01_B4_StartPixelLSB, LOBYTE(chip->Scan.StartPixel));
	Mustek_SendData(chip,ES01_B5_StartPixelMSB, HIBYTE(chip->Scan.StartPixel));
	XDBG(DEBUG_ASIC,"SetCalibrate StartPixel = %d", chip->Scan.StartPixel);

	Mustek_SendData(chip,ES01_B6_LineWidthPixelLSB, LOBYTE(chip->Scan.LongX));
	Mustek_SendData(chip,ES01_B7_LineWidthPixelMSB, HIBYTE(chip->Scan.LongX));
	XDBG(DEBUG_ASIC,"SetCalibrate Scan.LongX = %d", chip->Scan.LongX);

	Mustek_SendData(chip,ES01_F7_DigitalControl, 0);

	Mustek_SendData(chip,ES01_9E_HorizontalRatio1to15LSB, LOBYTE(chip->Scan.Dpi));
	Mustek_SendData(chip,ES01_9F_HorizontalRatio1to15MSB, HIBYTE(chip->Scan.Dpi));

	Mustek_SendData(chip,ES01_9B_ShadingTableAddrA12_A19, (BYTE)(chip->RamPositions.Shading));

	Mustek_SendData(chip,ES01_A6_MotorOption, 
		                             chip->isMotorMove |  
									 MOTOR_0_ENABLE |        
									 HOME_SENSOR_0_ENABLE |
									 MOTOR_TYPE);
	XDBG(DEBUG_ASIC,"Asic_SetCalibrate: MOTOR_TYPE=%d isMortorMove=%d",MOTOR_TYPE,isMotorMove);

	Mustek_SendData(chip,ES01_F6_MorotControl1, SPEED_NUIT_1_PIXEL_TIME |
									   MOTOR_SYNC_NUIT_1_PIXEL_TIME);

	Mustek_SendData(chip,ES01_E0_MotorAccStep0_7, LOBYTE(1));
	Mustek_SendData(chip,ES01_E1_MotorAccStep8_8, HIBYTE(1));
	
	Mustek_SendData(chip,ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(0));
	Mustek_SendData(chip,ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(0));
	Mustek_SendData(chip,ES01_E4_MotorStepOfMaxSpeed16_19, 0);

	Mustek_SendData(chip,ES01_E5_MotorDecStep, 1);

	Mustek_SendData(chip,ES01_AE_MotorSycnPixelNumberM16LSB, LOBYTE(0));
	Mustek_SendData(chip,ES01_AF_MotorSycnPixelNumberM16MSB, HIBYTE(0));

	Mustek_SendData(chip,ES01_EC_ScanAccStep0_7, LOBYTE(1));
	Mustek_SendData(chip,ES01_ED_ScanAccStep8_8, HIBYTE(1));

	Mustek_SendData(chip,ES01_EE_FixScanStepLSB, LOBYTE(0));
	Mustek_SendData(chip,ES01_8A_FixScanStepMSB, HIBYTE(0));

	Mustek_SendData(chip,ES01_EF_ScanDecStep, 1);

	Mustek_SendData(chip,ES01_E6_ScanBackTrackingStepLSB, LOBYTE(0));
	Mustek_SendData(chip,ES01_E7_ScanBackTrackingStepMSB, HIBYTE(0));

	Mustek_SendData(chip,ES01_E8_ScanRestartStepLSB, LOBYTE(0));
	Mustek_SendData(chip,ES01_E9_ScanRestartStepMSB, HIBYTE(0));

	Mustek_SendData(chip,ES01_EA_ScanBackHomeExtStepLSB, LOBYTE(0));
	Mustek_SendData(chip,ES01_EB_ScanBackHomeExtStepMSB, HIBYTE(0));

	XDBG(DEBUG_ASIC,"Asic_SetCalibrate: chip->ibase2=%d",chip->ibase2);
		Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(chip->ibase2));
		Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(chip->ibase2));

	TotalStep = (chip->Scan.PicHeight) * 1800 / wYResolution * MOTOR_STEP_MULTI + 200;  

	Mustek_SendData(chip,ES01_F0_ScanImageStep0_7, LOBYTE(TotalStep));
	Mustek_SendData(chip,ES01_F1_ScanImageStep8_15, HIBYTE(TotalStep));
	Mustek_SendData(chip,ES01_F2_ScanImageStep16_19, (BYTE) ((TotalStep & 0x00ff0000) >> 16));
	Mustek_SendData(chip,ES01_F8_DRAMType, DRAM_TIMING_1 | DRAM_256Kx16);

	Mustek_SendData(chip,ES01_A0_HostStartAddr0_7, 0x00);
	Mustek_SendData(chip,ES01_A1_HostStartAddr8_15, 0x00);
	Mustek_SendData(chip,ES01_A2_HostStartAddr16_19, 0x00 | ACCESS_DRAM);

	Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, chip->RamPositions.ImageEndAddr_0);
	Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, chip->RamPositions.ImageEndAddr_1);
	Mustek_SendData(chip,ES01_A5_HostEndAddr16_19, chip->RamPositions.ImageEndAddr_2);

	Mustek_SendData(chip,ES01_FB_BufferEmptySize16WordLSB, 0x00);
	Mustek_SendData(chip,ES01_FC_BufferEmptySize16WordMSB, 0x00);

	Mustek_SendData(chip,ES01_F9_BufferFullSize16WordLSB, chip->RamPositions.ImageFullBank_0);
	Mustek_SendData(chip,ES01_FA_BufferFullSize16WordMSB, chip->RamPositions.ImageFullBank_1);

	if (bScanBits == 8)
  		Mustek_SendData(chip,ES01_F5_ScanDataFormat, GRAY |
                                     		_8_BITS |
											GRAY_GREEN|
											SHADING_2_INT_14_DEC |
											SHADING_DISABLE |
											GAMMA_DISABLE
											);
	else if(bScanBits == 24)
		Mustek_SendData(chip,ES01_F5_ScanDataFormat, COLOR |
											_8_BITS |
											GRAY_RED|
											SHADING_2_INT_14_DEC |
											SHADING_DISABLE |
											GAMMA_DISABLE
											);
	else
		Mustek_SendData(chip,ES01_F5_ScanDataFormat, COLOR |
											_16_BITS |
											GRAY_RED|
											SHADING_2_INT_14_DEC |
											SHADING_DISABLE |
											GAMMA_DISABLE
											);

	Mustek_SendData(chip,ES01_FF_TestControl, BYPASS_AFE_DISABLE);

	Mustek_SendData(chip,ES01_F3_ActionOption, 
										  MOTOR_MOVE_TO_FIRST_LINE_DISABLE |
										  MOTOR_BACK_HOME_AFTER_SCAN_DISABLE |
										  SCAN_ENABLE |
										  SCAN_BACK_TRACKING_DISABLE |
										  INVERT_MOTOR_DIRECTION_DISABLE |
				 						  UNIFORM_MOTOR_AND_SCAN_SPEED_ENABLE);
	

	XDBG(DEBUG_ASIC,"Asic_SetCalibrate: Exit");
	return status;
}
