/* sane - Scanner Access Now Easy.
	Copyright (C) 2005 Mustek.
	Originally maintained by Mustek
	Author:Roy 2005.5.24

	This file is part of the SANE package.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston,
	MA 02111-1307, USA.

	As a special exception, the authors of SANE give permission for
	additional uses of the libraries contained in this release of SANE.

	The exception is that, if you link a SANE library with other files
	to produce an executable, this does not by itself cause the
	resulting executable to be covered by the GNU General Public
	License.  Your use of that executable is in no way restricted on
	account of linking the SANE library code into it.

	This exception does not, however, invalidate any other reasons why
	the executable file might be covered by the GNU General Public
	License.

	If you submit changes to SANE to the maintainers to be included in
	a subsequent release, you agree by submitting the changes that
	those changes may be distributed with this exception intact.

	If you write modifications of your own for SANE, it is your choice
	whether to permit this exception to apply to your modifications.
	If you do not wish that, delete this exception notice.

	This file provides a generic USB interface.  */#include "asic_mid.h"
#include "ScanRegDef.h"
#include <math.h>


#define		DPI_1800		0x8000
#define		DPI_900		    0x4000
#define		DPI_450			0x2000
#define     DPI_225         0x1000
#define     DPI_112         0x0800
#define     DPI_72          0x051e
#define     DPI_56          0x0400

#define		DPI_150			0x0aaa
#define		DPI_100			0x071c
#define		DPI_75			0x0555
#define     DPI_50          0x071c

#define		PIXEL_TIME		((double)333 / 1000000)	/*unit : ms*/

BYTE	PHTGTimingAdjustIN900;
BYTE	PH1_0IN900;
BYTE	PH1_1IN900;
BYTE	PH2_0IN900;
BYTE	PH2_1IN900;
BYTE	PH1B_0IN900;
BYTE	PH1B_1IN900;
BYTE	PH2B_0IN900;
BYTE	PH2B_1IN900;
BYTE	PHRS_0IN900;
BYTE	PHRS_1IN900;
BYTE	PHCP_0IN900;
BYTE	PHCP_1IN900;
BYTE	AFE_VSAMP_0IN900;
BYTE	AFE_VSAMP_1IN900;
BYTE	AFE_RSAMP_0IN900;
BYTE	AFE_RSAMP_1IN900;
BYTE	PH1234_IN_DUMMY_TGIN900;
BYTE	CCD_SETUP_REGISTERIN900;
BYTE	HARDWARE_SETTINGIN900;
BYTE	AFEControlIN900;
BYTE	AFE_CHANNEL_RED_LATCH_POSIN900;
BYTE	AFE_CHANNEL_GREEN_LATCH_POSIN900;
BYTE	AFE_CHANNEL_BLUE_LATCH_POSIN900;
BYTE	AFE_MALC_TIMING_ADJUST_0IN900;
BYTE	AFE_MALC_TIMING_ADJUST_1IN900;
BYTE	AD9826ConfigurationIN900;
BYTE	AD9826MuxConfigIN900;
BYTE	PHTGTimingAdjustIN1800;
BYTE	PH1_0IN1800;
BYTE	PH1_1IN1800;
BYTE	PH2_0IN1800;
BYTE	PH2_1IN1800;
BYTE	PH1B_0IN1800;
BYTE	PH1B_1IN1800;
BYTE	PH2B_0IN1800;
BYTE	PH2B_1IN1800;
BYTE	PHRS_0IN1800;
BYTE	PHRS_1IN1800;
BYTE	PHCP_0IN1800;
BYTE	PHCP_1IN1800;
BYTE	AFE_VSAMP_0IN1800;
BYTE	AFE_VSAMP_1IN1800;
BYTE	AFE_RSAMP_0IN1800;
BYTE	AFE_RSAMP_1IN1800;
BYTE	PH1234_IN_DUMMY_TGIN1800;
BYTE	CCD_SETUP_REGISTERIN1800;
BYTE	HARDWARE_SETTINGIN1800;
BYTE	AFEControlIN1800;
BYTE	AFE_CHANNEL_RED_LATCH_POSIN1800;
BYTE	AFE_CHANNEL_GREEN_LATCH_POSIN1800;
BYTE	AFE_CHANNEL_BLUE_LATCH_POSIN1800;
BYTE	AFE_MALC_TIMING_ADJUST_0IN1800;
BYTE	AFE_MALC_TIMING_ADJUST_1IN1800;
BYTE	AD9826ConfigurationIN1800;
BYTE	AD9826MuxConfigIN1800;

STATUS OpenScanChip(PAsic chip)
{
	BYTE x;
	STATUS status;
	
	XDBG(DEBUG_ASIC,"OpenScanChip: Enter");
	x = 0x64;
	status = WriteIOControl(chip, 0x90, 0, 1, &x);
	if (status != STATUS_GOOD)
		return status;
	
	x= 0x65;
	status = WriteIOControl(chip, 0x90, 0, 1, &x);
	if (status != STATUS_GOOD)
		return status;
	
	x = 0x44;
	status = WriteIOControl(chip, 0x90, 0, 1, &x);
	if (status != STATUS_GOOD)
		return status;
	
	x = 0x45;
	status = WriteIOControl(chip, 0x90, 0, 1, &x);
	
	XDBG(DEBUG_ASIC,"OpenScanChip: Exit");
	return status;
}


STATUS CloseScanChip(PAsic chip)
{
	STATUS status;
	BYTE x;
	
	x = 0x64;
	
	status = WriteIOControl(chip,0x90,0,1,&x);
	if (status != STATUS_GOOD)
		return status;
	
	x = 0x65;
	
	status = WriteIOControl(chip,0x90,0,1,&x);
	if (status != STATUS_GOOD)
		return status;
	
	
	x = 0x16;
	
	status = WriteIOControl(chip,0x90,0,1,&x);
	if (status != STATUS_GOOD)
		return status;
	
	x = 0x17;
	
	status = WriteIOControl(chip,0x90,0,1,&x);
	if (status != STATUS_GOOD)
		return status;
	
	return STATUS_GOOD;
}



STATUS SafeInitialChip(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	BYTE temp_f;
	
	XDBG(DEBUG_ASIC,"SafeInitialChip: Enter");
	
	Mustek_SendData(chip,ES01_86_DisableAllClockWhenIdle, CLOSE_ALL_CLOCK_DISABLE);

	Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	
	/*avoid scan not finished*/
	do
	{
		status = Mustek_SendData(chip,ES01_8B_Status, 0x01);
		if (status != STATUS_GOOD)
			return status;
		
		status = Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
		if (status != STATUS_GOOD)
			return status;
		
		temp_f = ES01_8B_Status;
		
		status = Mustek_ReceiveData(chip,&temp_f);
		if (status != STATUS_GOOD)
			return status;
		
	} while ((temp_f & 0x1f) != 0);
	
	if(chip->isFirstOpenChip)
		
	{
		status = DRAM_Test(chip);
		if (status != STATUS_GOOD)
		{
			XDBG(DEBUG_ASIC,"DRAM_Test: Error");
			return status;
		}
		
		chip->isFirstOpenChip=FALSE;
	}
	
	XDBG(DEBUG_ASIC,"SafeInitialChip: exit");
	return STATUS_GOOD;
}


STATUS DRAM_Test(PAsic chip)
{
	XDBG(DEBUG_ASIC,"DRAM_Test: Enter");
	STATUS status;
	unsigned char * temps;
	unsigned int	i;
	
	temps = (unsigned char *) malloc (64);
	
	for (i = 0; i < 64; i++) 
	{
		*(temps + i) = i;
	}
	
	/*Set ram type*/
	status = Mustek_SendData(chip, ES01_F8_DRAMType, DRAM_TIMING_1 | DRAM_256Kx16);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	/*set start address*/
	status = Mustek_SendData(chip, ES01_A0_HostStartAddr0_7, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A1_HostStartAddr8_15, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A2_HostStartAddr16_19, 0x00|ACCESS_DRAM);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	/*set end address*/
	status = Mustek_SendData(chip, ES01_A3_HostEndAddr0_7, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A4_HostEndAddr8_15, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A5_HostEndAddr16_19, 0x3f);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}

	/*add by Roy 2005.9.28*/
	XDBG(DEBUG_ASIC,"before write");
	for (i = 0; i <60; i+=10)
	{
		XDBG(DEBUG_ASIC,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
			*(temps +i),*(temps +i+1),*(temps +i+2),*(temps +i+3),*(temps +i+4),
			*(temps +i+5),*(temps +i+6),*(temps +i+7),*(temps +i+8),*(temps +i+9));
	}
	
	/*	SW_ClearFIFO();*/
	status = Mustek_DMAWrite(chip, 64, (LPBYTE)(temps));
	if (status != STATUS_GOOD)
	{
		XDBG(DEBUG_ASIC,"Mustek_DMAWrite error");
		free(temps);
		return status;
	}
	

	status = Mustek_SendData(chip, ES01_A0_HostStartAddr0_7, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A1_HostStartAddr8_15, 0x00);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A2_HostStartAddr16_19, 0x00| ACCESS_DRAM);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	/*set end address*/
	status = Mustek_SendData(chip, ES01_A3_HostEndAddr0_7, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A4_HostEndAddr8_15, 0xff);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	status = Mustek_SendData(chip, ES01_A5_HostEndAddr16_19, 0x3f);
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}
	
	memset(temps, 0, 64);

	/*	SW_ClearFIFO();*/
	status = Mustek_DMAReadGeneralMode(chip, 64, (LPBYTE)(temps));
	if (status != STATUS_GOOD)
	{
		free(temps);
		return status;
	}

	/*add by Roy 2005.9.28*/
	XDBG(DEBUG_ASIC,"after read");
	for (i = 0; i <60; i+=10)
	{
		XDBG(DEBUG_ASIC,"%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",
			*(temps +i),*(temps +i+1),*(temps +i+2),*(temps +i+3),*(temps +i+4),
			*(temps +i+5),*(temps +i+6),*(temps +i+7),*(temps +i+8),*(temps +i+9));
	}
	

	for (i = 0; i < 64; i++)
	{
		if (*(temps + i) != i)
		{
			fprintf(stderr, "DRAM Test error...\n");
			return STATUS_IO_ERROR;
		}
	}
	
	free (temps);
	
	XDBG(DEBUG_ASIC,"DRAM_Text: Exit");
	return STATUS_GOOD;
}

STATUS ResetGpio(PAsic chip)
{
	STATUS status;
	
	XDBG(DEBUG_ASIC,"ResetGpio: Enter");
	
	Mustek_SendData(chip, ES01_97_GPIOControl0_7, 0x7c);
	if(chip->lightsource==LS_REFLECTIVE)
	{
		Mustek_SendData(chip, ES01_95_GPIOValue0_7, GPIO_95_Config);
	}
	else
	{
		Mustek_SendData(chip, ES01_95_GPIOValue0_7, GPIO_95_Config & 0xdf); 
	}
	
	Mustek_SendData(chip, ES01_98_GPIOControl8_15, 0x00);  
	Mustek_SendData(chip, ES01_96_GPIOValue8_15, 0x00);
	
	XDBG(DEBUG_ASIC,"ResetGpio: Exit");
	return STATUS_GOOD;
}

STATUS ResetTAGpio(PAsic chip)
{
	STATUS status = STATUS_GOOD;

	XDBG(DEBUG_ASIC,"ResetTAGpio: Enter");

	Mustek_SendData(chip, ES01_97_GPIOControl0_7, 0x7c);
	Mustek_SendData(chip, ES01_95_GPIOValue0_7, 0x57);

	Mustek_SendData(chip, ES01_98_GPIOControl8_15, 0x00);  
	Mustek_SendData(chip, ES01_96_GPIOValue8_15, 0x00);

	XDBG(DEBUG_ASIC,"ResetTAGpio: Exit");
	return status;
}

STATUS SetScanParam(PAsic chip,int dpi)
{
	DWORD	temp_s;
	BYTE	Temp_Dummy_Cycle;
	int nMultiResolution; 
	
	nMultiResolution = 1800 / dpi; 
	
	switch (dpi)
	{
	case 1800:
		chip->Scan.Dpi = DPI_1800;
		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_1800DPI;
		}
		else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_1800DPI;
		}else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_1800DPI;
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12;  
		chip->Scan.TG_Wait_Width_Pixel	= 1;   
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;

		if(chip->UsbHost==HT_USB10)
		{
			chip->Scan.Dummy_Cycle			= 1;
		}
		else if(chip->lightsource!=LS_REFLECTIVE)
		{
			chip->Scan.Dummy_Cycle			= 0; 
		}
		else
		{
			chip->Scan.Dummy_Cycle			= 0; 
		}
		
		
		chip->Scan.TG_Times				= 0;		
		Temp_Dummy_Cycle			= 0;
		
		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		
		if(chip->Temp.Line_Pixel>65536)  
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;
		
		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(chip->Scan.Dummy_Cycle + 1);
		chip->Scan.CCD_Dummy_Pixel = 
				(WORD)(chip->Temp.Line_Pixel > temp_s  ? (chip->Temp.Line_Pixel - temp_s): 0);

		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);

		chip->ibase2 = ((DWORD)(chip->Temp.Line_Pixel)) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 /= MOTOR_STEP_MULTI;
		chip->ibase1 = chip->ibase2 - 1000;
		break;
	case 900:
		chip->Scan.Dpi = DPI_900;
		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_900DPI;  
		}else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_900DPI;  
		}else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_900DPI; 
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12;  
		chip->Scan.TG_Wait_Width_Pixel	= 1;   
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		if(chip->UsbHost==HT_USB10 && chip->lightsource==LS_REFLECTIVE)
		{
			if(chip->dwBytesCountPerRow>18000)
				chip->Scan.Dummy_Cycle = 5;
			else
				chip->Scan.Dummy_Cycle = 10; 
		}
		else if(chip->lightsource!=LS_REFLECTIVE)
		{
			if(chip->dwBytesCountPerRow>18000)
				chip->Scan.Dummy_Cycle = 1;
			else
				chip->Scan.Dummy_Cycle = 2;
		}
		else
		{
			if(chip->dwBytesCountPerRow>18000)
				chip->Scan.Dummy_Cycle = 1;
			else
				chip->Scan.Dummy_Cycle = 2;
		}

		chip->Scan.Dummy_Cycle			= 1;
		chip->Scan.TG_Times				= 0;		
		Temp_Dummy_Cycle			= 0;
		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;
		
		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(Temp_Dummy_Cycle + 1);
		
		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s  ? chip->Temp.Line_Pixel - temp_s : 0);
		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);

		chip->ibase2 = ((DWORD)(chip->Temp.Line_Pixel) / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 /= (MOTOR_STEP_MULTI);

		if (chip->UsbHost == HT_USB10)
			chip->ibase2 -= 45;

		chip->ibase1 = chip->ibase2;
		break;
	case 450:
		chip->Scan.Dpi = DPI_450;
		
		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_450DPI;
		}else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_450DPI;
		}else if(chip->UsbHost==HT_USB10)
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_450DPI;
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_450DPI;
		}
		
		chip->Scan.TG_Pulse_Width_Pixel	= 12;  
		chip->Scan.TG_Wait_Width_Pixel	= 1;   
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;

		chip->Scan.Dummy_Cycle		= 0;
		chip->Scan.TG_Times				= 0;
		Temp_Dummy_Cycle			= 0;
		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;
		
		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel	= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);

		chip->ibase2 =(WORD) (chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);

		chip->ibase2 /= MOTOR_STEP_MULTI;
		
		if (chip->UsbHost == HT_USB10)
			chip->ibase2 -= 10;

		chip->ibase1 = chip->ibase2;
		break;

	case 225:
		chip->Scan.Dpi = DPI_225;

		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_225DPI;
		}
		else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_225DPI;
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_225DPI;
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1;  
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		chip->Scan.Dummy_Cycle = 0;
		chip->Scan.TG_Times				= 0;
		Temp_Dummy_Cycle			= 0;
		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);
	
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;

		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(Temp_Dummy_Cycle + 1);
		
		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 /= MOTOR_STEP_MULTI;
		
		chip->ibase1 = chip->ibase2 + 500; 
		break;
	case 112:
		chip->Scan.Dpi = DPI_112;
		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_112DPI;
		}else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_112DPI;
		}else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_112DPI;
		}

		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1;  
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		chip->Scan.Dummy_Cycle			= 0;
		chip->Scan.TG_Times				= 0;
		
		Temp_Dummy_Cycle			= 0;
		
		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;
		
		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(Temp_Dummy_Cycle + 1);
		
		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);

		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 /= MOTOR_STEP_MULTI;
		chip->ibase1 = chip->ibase2 + 1000;
		break;
	case 72:
		chip->Scan.Dpi = DPI_72;
		
		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_72DPI;
		}
		else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_72DPI;
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_72DPI;
		}

		chip->Scan.TG_Pulse_Width_Pixel	= 12;  
		chip->Scan.TG_Wait_Width_Pixel	= 1;   
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		chip->Scan.Dummy_Cycle	= 0;
		chip->Scan.TG_Times				= 0;
		Temp_Dummy_Cycle			= 0;
		
		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;
		
		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);

		
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution )* (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 /= MOTOR_STEP_MULTI;
		chip->ibase1 = chip->ibase2 + 1500;
		break;
	case 56:
		chip->Scan.Dpi = DPI_56;
		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_56DPI;
		}else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_56DPI; 
		}else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_56DPI;
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1;  
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		chip->Scan.Dummy_Cycle			= 0; 
		chip->Scan.TG_Times				= 0;
	
		Temp_Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);

		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;
		
		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(Temp_Dummy_Cycle + 1);
		
		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);
		
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 /= MOTOR_STEP_MULTI;
		chip->ibase1 = chip->ibase2 + 1500;
		break;
		
	case 50:
		chip->Scan.Dpi = DPI_50;
		if(chip->lightsource==LS_POSITIVE)
		{ 
			chip->Temp.Line_Time = POSITIVE_LINE_TIME_50DPI;
		}
		else if(chip->lightsource==LS_NEGATIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_50DPI;
		}else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_50DPI;
		}
		chip->Scan.StartPixel /= 2;
		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1;  
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;

		if(chip->lightsource!=LS_REFLECTIVE)
			chip->Scan.Dummy_Cycle			= 0;  
		else
			chip->Scan.Dummy_Cycle			= 0;

		chip->Scan.TG_Times				= 0;
		
		Temp_Dummy_Cycle			= 0;
		
		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;
		
		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
			(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
			(chip->Scan.CCD_Pixel_Length + 1) +
			(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
			(Temp_Dummy_Cycle + 1);
		
		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		XDBG(DEBUG_ASIC,"SetScanParam: chip->Temp.Line_Pixel = %d",chip->Temp.Line_Pixel);

		
		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / 36) * (chip->Scan.Dummy_Cycle + 1)+ 6400;
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / 36) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 /= MOTOR_STEP_MULTI;
		chip->ibase1= chip->ibase2 + 2000;
		break;
	}
	
	XDBG(DEBUG_ASIC,"SetScanParam: chip->ibase1=%d, chip->ibase2=%d", chip->ibase1, chip->ibase2);
	
	if(dpi>50)
	{
		chip->Timing.PHTGTimingAdjust				= PHTGTimingAdjustIN1800;
		chip->Timing.PH1_0						= PH1_0IN1800;
		chip->Timing.PH1_1						= PH1_1IN1800;
		chip->Timing.PH2_0						= PH2_0IN1800;
		chip->Timing.PH2_1						= PH2_1IN1800;
		chip->Timing.PH1B_0						= PH1B_0IN1800;
		chip->Timing.PH1B_1						= PH1B_1IN1800;
		chip->Timing.PH2B_0						= PH2B_0IN1800;
		chip->Timing.PH2B_1						= PH2B_1IN1800;
		chip->Timing.PHRS_0						= PHRS_0IN1800;
		chip->Timing.PHRS_1						= PHRS_1IN1800;
		chip->Timing.PHCP_0						= PHCP_0IN1800;
		chip->Timing.PHCP_1						= PHCP_1IN1800;
		chip->Timing.AFE_VSAMP_0					= AFE_VSAMP_0IN1800;
		chip->Timing.AFE_VSAMP_1					= AFE_VSAMP_1IN1800;
		chip->Timing.AFE_RSAMP_0					= AFE_RSAMP_0IN1800;
		chip->Timing.AFE_RSAMP_1					= AFE_RSAMP_1IN1800;
		chip->Timing.PH1234_IN_DUMMY_TG			= PH1234_IN_DUMMY_TGIN1800;
		chip->Timing.CCD_SETUP_REGISTER			= CCD_SETUP_REGISTERIN1800;
		chip->Timing.HARDWARE_SETTING				= HARDWARE_SETTINGIN1800;
		chip->Timing.AFEControl					= AFEControlIN1800;
		chip->Timing.AFE_CHANNEL_RED_LATCH_POS	= AFE_CHANNEL_RED_LATCH_POSIN1800;
		chip->Timing.AFE_CHANNEL_GREEN_LATCH_POS	= AFE_CHANNEL_GREEN_LATCH_POSIN1800;
		chip->Timing.AFE_CHANNEL_BLUE_LATCH_POS	= AFE_CHANNEL_BLUE_LATCH_POSIN1800;
		chip->Timing.AFE_MALC_TIMING_ADJUST_0		= AFE_MALC_TIMING_ADJUST_0IN1800;
		chip->Timing.AFE_MALC_TIMING_ADJUST_1		= AFE_MALC_TIMING_ADJUST_1IN1800;
		chip->Timing.AD9826Configuration			= AD9826ConfigurationIN1800;
		chip->Timing.AD9826MuxConfig				= AD9826MuxConfigIN1800;
		XDBG(DEBUG_ASIC,"Set ScanParam: chip->Timing.PHTGTimingAdjust=%d chip->Timing.PH1_0=%d chip->Timing.PH1_1=%d"
		"chip->Timing.PH2_0=%d	chip->Timing.PH2_1=%d 	chip->Timing.PH1B_0=%d	chip->Timing.PH1B_1=%d chip->Timing.PH2B_0=%d	"
		"chip->Timing.PH2B_1",chip->Timing.PHTGTimingAdjust ,chip->Timing.PH1_0, chip->Timing.PH1_1,
		chip->Timing.PH2_0,chip->Timing.PH2_1,	chip->Timing.PH1B_0,	chip->Timing.PH1B_1, chip->Timing.PH2B_0,
		chip->Timing.PH2B_1);
	}
	else
	{
		chip->Timing.PHTGTimingAdjust				= PHTGTimingAdjustIN900;
		chip->Timing.PH1_0						= PH1_0IN900;
		chip->Timing.PH1_1						= PH1_1IN900;
		chip->Timing.PH2_0						= PH2_0IN900;
		chip->Timing.PH2_1						= PH2_1IN900;
		chip->Timing.PH1B_0						= PH1B_0IN900;
		chip->Timing.PH1B_1						= PH1B_1IN900;
		chip->Timing.PH2B_0						= PH2B_0IN900;
		chip->Timing.PH2B_1						= PH2B_1IN900;
		chip->Timing.PHRS_0						= PHRS_0IN900;
		chip->Timing.PHRS_1						= PHRS_1IN900;
		chip->Timing.PHCP_0						= PHCP_0IN900;
		chip->Timing.PHCP_1						= PHCP_1IN900;
		chip->Timing.AFE_VSAMP_0					= AFE_VSAMP_0IN900;
		chip->Timing.AFE_VSAMP_1					= AFE_VSAMP_1IN900;
		chip->Timing.AFE_RSAMP_0					= AFE_RSAMP_0IN900;
		chip->Timing.AFE_RSAMP_1					= AFE_RSAMP_1IN900;
		chip->Timing.PH1234_IN_DUMMY_TG			= PH1234_IN_DUMMY_TGIN900;
		chip->Timing.CCD_SETUP_REGISTER			= CCD_SETUP_REGISTERIN900;
		chip->Timing.HARDWARE_SETTING				= HARDWARE_SETTINGIN900;
		chip->Timing.AFEControl					= AFEControlIN900;
		chip->Timing.AFE_CHANNEL_RED_LATCH_POS	= AFE_CHANNEL_RED_LATCH_POSIN900;
		chip->Timing.AFE_CHANNEL_GREEN_LATCH_POS	= AFE_CHANNEL_GREEN_LATCH_POSIN900;
		chip->Timing.AFE_CHANNEL_BLUE_LATCH_POS	= AFE_CHANNEL_BLUE_LATCH_POSIN900;
		chip->Timing.AFE_MALC_TIMING_ADJUST_0		= AFE_MALC_TIMING_ADJUST_0IN900;
		chip->Timing.AFE_MALC_TIMING_ADJUST_1		= AFE_MALC_TIMING_ADJUST_1IN900;
		chip->Timing.AD9826Configuration			= AD9826ConfigurationIN900;
		chip->Timing.AD9826MuxConfig				= AD9826MuxConfigIN900;
	}

	return STATUS_GOOD;
}

STATUS SetCalibrateParam(PAsic chip,int dpi)
{
	STATUS status = STATUS_GOOD;
	DWORD	temp_s;
	BYTE	Temp_Dummy_Cycle;
	int nMultiResolution; 

	XDBG(DEBUG_ASIC,"SetCalibrateParam: Enter");

	nMultiResolution = 4; 

	switch (dpi)
	{
	case 1800:
		chip->Scan.Dpi = DPI_1800;
		if(chip->lightsource != LS_REFLECTIVE)
		{
			chip->Temp.Line_Time = CALIB_NEGATIVE_LINE_TIME_1800DPI;
		}else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_1800DPI;
		}

		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1;  
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;

		if(chip->UsbHost==HT_USB10 && chip->lightsource == LS_REFLECTIVE)
		{
			if(chip->dwBytesCountPerRow>18000)
				chip->Scan.Dummy_Cycle			= 6;
			else
				chip->Scan.Dummy_Cycle			= 4;
		}
		else if(chip->lightsource!=LS_REFLECTIVE)
		{
			chip->Scan.Dummy_Cycle			= 0;
		}
		else
		{
			chip->Scan.Dummy_Cycle			= 0;
		}

		chip->Scan.TG_Times				= 0;		
		Temp_Dummy_Cycle			= 0;
		chip->Scan.Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536) 
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;


		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (chip->Scan.Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);

		chip->ibase1 = (WORD) chip->Temp.Line_Pixel / nMultiResolution  * (chip->Scan.Dummy_Cycle + 1) + 3000;
		chip->ibase2 = (WORD) chip->Temp.Line_Pixel / nMultiResolution  * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase1/=MOTOR_STEP_MULTI;
		chip->ibase2/=MOTOR_STEP_MULTI;
		break;

	case 900:
		chip->Scan.Dpi = DPI_900;
		if(chip->lightsource != LS_REFLECTIVE)
		{
			chip->Temp.Line_Time = CALIB_NEGATIVE_LINE_TIME_900DPI;
		}else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_900DPI;
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12;  
		chip->Scan.TG_Wait_Width_Pixel	= 1;   
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		if(chip->UsbHost == HT_USB10 && chip->lightsource==LS_REFLECTIVE)
		{
			if(chip->dwBytesCountPerRow>18000)
				chip->Scan.Dummy_Cycle			= 6;
			else
				chip->Scan.Dummy_Cycle			= 4;
		}
		else if(chip->lightsource != LS_REFLECTIVE)
		{
			chip->Scan.Dummy_Cycle			= 0;
		}
		else
		{
			chip->Scan.Dummy_Cycle			= 0;
		}

		chip->Scan.TG_Times				= 0;		
		Temp_Dummy_Cycle			= 0;
		chip->Scan.Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536) 
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;


		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);

		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1) + 3000;
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase1/=MOTOR_STEP_MULTI;
		chip->ibase2/=MOTOR_STEP_MULTI;
		break;
	case 450:
		chip->Scan.Dpi = DPI_450;
		if(chip->lightsource!=LS_REFLECTIVE)
		{
			chip->Temp.Line_Time = CALIB_NEGATIVE_LINE_TIME_450DPI;
		}
		else if(chip->UsbHost==HT_USB10)
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_450DPI;
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_450DPI;
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1; 
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		if(chip->UsbHost==HT_USB10 && chip->lightsource==LS_REFLECTIVE)
		{
				chip->Scan.Dummy_Cycle		= 0;  
		}
		else
		{
			chip->Scan.Dummy_Cycle		= 0;
		}

		chip->Scan.TG_Times				= 0;
		chip->Scan.Dummy_Cycle		= 0;
		
		Temp_Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536) 
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;

		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution + 1000) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase1 /= MOTOR_STEP_MULTI;
		chip->ibase2 /= MOTOR_STEP_MULTI;
		break;
	case 225:
		chip->Scan.Dpi = DPI_225;

		if(chip->lightsource!=LS_REFLECTIVE)
		{
			chip->Temp.Line_Time = CALIB_NEGATIVE_LINE_TIME_225DPI;
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_225DPI;
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12;
		chip->Scan.TG_Wait_Width_Pixel	= 1; 
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		if(chip->lightsource!=LS_REFLECTIVE)
			chip->Scan.Dummy_Cycle			= 1;
		else
			chip->Scan.Dummy_Cycle			= 0;

		chip->Scan.TG_Times				= 0;
		chip->Scan.Dummy_Cycle			= 0;

		Temp_Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536) 
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;

		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);

		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution + 500) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase1/=MOTOR_STEP_MULTI; 
		chip->ibase2/=MOTOR_STEP_MULTI; 
		break;
	case 112:
		chip->Scan.Dpi = DPI_112;

		if(chip->lightsource!=LS_REFLECTIVE)
		{
			chip->Temp.Line_Time = CALIB_NEGATIVE_LINE_TIME_112DPI; 
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_112DPI; 
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1;
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		chip->Scan.Dummy_Cycle			= 0;
		chip->Scan.TG_Times				= 0;
		if(chip->lightsource!=LS_REFLECTIVE)
			chip->Scan.Dummy_Cycle			= 0;
		else
			chip->Scan.Dummy_Cycle			= 0;
			
		Temp_Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;

		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution + 1000) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase1/=MOTOR_STEP_MULTI; 
		chip->ibase2/=MOTOR_STEP_MULTI; 
		break;
	case 72:
		chip->Scan.Dpi = DPI_72;

		if(chip->lightsource!=LS_REFLECTIVE)
		{
			chip->Temp.Line_Time = NEGATIVE_LINE_TIME_72DPI; 
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_72DPI; 
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1; 
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		chip->Scan.Dummy_Cycle			= 0;
		chip->Scan.TG_Times				= 0;

		Temp_Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536)
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;

		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution + 3000) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase1/=MOTOR_STEP_MULTI; 
		chip->ibase2/=MOTOR_STEP_MULTI; 
		break;

	case 56:

		chip->Scan.Dpi = DPI_56;

		if(chip->lightsource!=LS_REFLECTIVE)
		{
			chip->Temp.Line_Time = CALIB_NEGATIVE_LINE_TIME_56DPI;  
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_56DPI; 
		}
		chip->Scan.TG_Pulse_Width_Pixel	= 12;
		chip->Scan.TG_Wait_Width_Pixel	= 1; 
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		chip->Scan.Dummy_Cycle			= 0;
		chip->Scan.TG_Times				= 0;

		Temp_Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536) 
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;

		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);
		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution + 3000) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase1/=MOTOR_STEP_MULTI; 
		chip->ibase2/=MOTOR_STEP_MULTI; 
		break;
	case 50:
		chip->Scan.Dpi = DPI_50;
		if(chip->lightsource!=LS_REFLECTIVE)
		{
			if(chip->lightsource!=LS_POSITIVE)
				chip->Temp.Line_Time = NEGATIVE_LINE_TIME_50DPI;
			else
				chip->Temp.Line_Time = POSITIVE_LINE_TIME_50DPI;
		}
		else
		{
			chip->Temp.Line_Time = REFLECTIVE_LINE_TIME_50DPI; 
		}  
		chip->Scan.StartPixel /= 2;
		chip->Scan.TG_Pulse_Width_Pixel	= 12; 
		chip->Scan.TG_Wait_Width_Pixel	= 1;  
		chip->Scan.Multi_TG_Dummy_Pixel	= 0;
		if(chip->lightsource!=LS_REFLECTIVE)
			chip->Scan.Dummy_Cycle			= 0; 
		else
			chip->Scan.Dummy_Cycle			= 0;
		chip->Scan.TG_Times				= 0;

		Temp_Dummy_Cycle			= 0;

		chip->Temp.Line_Pixel	= (DWORD)(chip->Temp.Line_Time / PIXEL_TIME);
		if(chip->Temp.Line_Pixel>65536) 
			chip->Scan.CCD_Pixel_Length=65536-chip->Scan.TG_Pulse_Width_Pixel-chip->Scan.TG_Wait_Width_Pixel*2;

		temp_s = ((chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
				  (chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
				  (chip->Scan.CCD_Pixel_Length + 1) +
				  (chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0)) *
				  (Temp_Dummy_Cycle + 1);

		chip->Scan.CCD_Dummy_Pixel		= (WORD)(chip->Temp.Line_Pixel > temp_s ? chip->Temp.Line_Pixel - temp_s : 0);

		chip->ibase1 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution + 3000) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2 = (WORD)(chip->Temp.Line_Pixel / nMultiResolution) * (chip->Scan.Dummy_Cycle + 1);
		chip->ibase2/=MOTOR_STEP_MULTI;   
		chip->ibase1=chip->ibase2 + 2000;  
		break;
	}

	if(dpi>50)
	{
		chip->Timing.PHTGTimingAdjust				= PHTGTimingAdjustIN1800;
		chip->Timing.PH1_0						= PH1_0IN1800;
		chip->Timing.PH1_1						= PH1_1IN1800;
		chip->Timing.PH2_0						= PH2_0IN1800;
		chip->Timing.PH2_1						= PH2_1IN1800;
		chip->Timing.PH1B_0						= PH1B_0IN1800;
		chip->Timing.PH1B_1						= PH1B_1IN1800;
		chip->Timing.PH2B_0						= PH2B_0IN1800;
		chip->Timing.PH2B_1						= PH2B_1IN1800;
		chip->Timing.PHRS_0						= PHRS_0IN1800;
		chip->Timing.PHRS_1						= PHRS_1IN1800;
		chip->Timing.PHCP_0						= PHCP_0IN1800;
		chip->Timing.PHCP_1						= PHCP_1IN1800;
		chip->Timing.AFE_VSAMP_0					= AFE_VSAMP_0IN1800;
		chip->Timing.AFE_VSAMP_1					= AFE_VSAMP_1IN1800;
		chip->Timing.AFE_RSAMP_0					= AFE_RSAMP_0IN1800;
		chip->Timing.AFE_RSAMP_1					= AFE_RSAMP_1IN1800;
		chip->Timing.PH1234_IN_DUMMY_TG			= PH1234_IN_DUMMY_TGIN1800;
		chip->Timing.CCD_SETUP_REGISTER			= CCD_SETUP_REGISTERIN1800;

		chip->Timing.HARDWARE_SETTING				= HARDWARE_SETTINGIN1800;

		chip->Timing.AFEControl					= AFEControlIN1800;

		chip->Timing.AFE_CHANNEL_RED_LATCH_POS	= AFE_CHANNEL_RED_LATCH_POSIN1800;
		chip->Timing.AFE_CHANNEL_GREEN_LATCH_POS	= AFE_CHANNEL_GREEN_LATCH_POSIN1800;
		chip->Timing.AFE_CHANNEL_BLUE_LATCH_POS	= AFE_CHANNEL_BLUE_LATCH_POSIN1800;
		chip->Timing.AFE_MALC_TIMING_ADJUST_0		= AFE_MALC_TIMING_ADJUST_0IN1800;
		chip->Timing.AFE_MALC_TIMING_ADJUST_1		= AFE_MALC_TIMING_ADJUST_1IN1800;

		chip->Timing.AD9826Configuration			= AD9826ConfigurationIN1800;
		chip->Timing.AD9826MuxConfig				= AD9826MuxConfigIN1800;
	}
	else
	{
		chip->Timing.PHTGTimingAdjust				= PHTGTimingAdjustIN900;
		chip->Timing.PH1_0						= PH1_0IN900;
		chip->Timing.PH1_1						= PH1_1IN900;
		chip->Timing.PH2_0						= PH2_0IN900;
		chip->Timing.PH2_1						= PH2_1IN900;
		chip->Timing.PH1B_0						= PH1B_0IN900;
		chip->Timing.PH1B_1						= PH1B_1IN900;
		chip->Timing.PH2B_0						= PH2B_0IN900;
		chip->Timing.PH2B_1						= PH2B_1IN900;
		chip->Timing.PHRS_0						= PHRS_0IN900;
		chip->Timing.PHRS_1						= PHRS_1IN900;
		chip->Timing.PHCP_0						= PHCP_0IN900;
		chip->Timing.PHCP_1						= PHCP_1IN900;
		chip->Timing.AFE_VSAMP_0					= AFE_VSAMP_0IN900;
		chip->Timing.AFE_VSAMP_1					= AFE_VSAMP_1IN900;
		chip->Timing.AFE_RSAMP_0					= AFE_RSAMP_0IN900;
		chip->Timing.AFE_RSAMP_1					= AFE_RSAMP_1IN900;
		chip->Timing.PH1234_IN_DUMMY_TG			= PH1234_IN_DUMMY_TGIN900;
		chip->Timing.CCD_SETUP_REGISTER			= CCD_SETUP_REGISTERIN900;

		chip->Timing.HARDWARE_SETTING				= HARDWARE_SETTINGIN900;
		chip->Timing.AFEControl					= AFEControlIN900;

		chip->Timing.AFE_CHANNEL_RED_LATCH_POS	= AFE_CHANNEL_RED_LATCH_POSIN900;
		chip->Timing.AFE_CHANNEL_GREEN_LATCH_POS	= AFE_CHANNEL_GREEN_LATCH_POSIN900;
		chip->Timing.AFE_CHANNEL_BLUE_LATCH_POS	= AFE_CHANNEL_BLUE_LATCH_POSIN900;
		chip->Timing.AFE_MALC_TIMING_ADJUST_0		= AFE_MALC_TIMING_ADJUST_0IN900;
		chip->Timing.AFE_MALC_TIMING_ADJUST_1		= AFE_MALC_TIMING_ADJUST_1IN900;

		chip->Timing.AD9826Configuration			= AD9826ConfigurationIN900;
		chip->Timing.AD9826MuxConfig				= AD9826MuxConfigIN900;
	}

	XDBG(DEBUG_ASIC,"SetCalibrateParam: Exit");
	return status;
}

STATUS SetPowerSave(PAsic chip)
{
	if (chip->firmwarestate < FS_OPENED)
		OpenScanChip(chip);
	
	if (chip->firmwarestate > FS_OPENED)
		Asic_ScanStop(chip);
	
	chip->firmwarestate = FS_OPENED;
	
	return STATUS_GOOD;
}

STATUS SetLineTimeAndExposure(PAsic chip)
{
	if (chip->firmwarestate < FS_OPENED)
		OpenScanChip(chip);
	
	if (chip->firmwarestate > FS_OPENED)
		Mustek_SendData(chip, ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	
	Mustek_SendData(chip, ES01_B0_CCDPixelLSB, LOBYTE( chip->Scan.CCD_Pixel_Length));
	Mustek_SendData(chip, ES01_B1_CCDPixelMSB, HIBYTE( chip->Scan.CCD_Pixel_Length));
	
	Mustek_SendData(chip, ES01_B2_PHTGPulseWidth, chip->Scan.TG_Pulse_Width_Pixel);
	Mustek_SendData(chip, ES01_B3_PHTGWaitWidth, chip->Scan.TG_Wait_Width_Pixel);
	
	Mustek_SendData(chip, ES01_C4_MultiTGTimesRed, 0);
	Mustek_SendData(chip, ES01_C5_MultiTGTimesGreen, 0);
	Mustek_SendData(chip, ES01_C6_MultiTGTimesBlue, 0);
	
	Mustek_SendData(chip, ES01_C7_MultiTGDummyPixelNumberLSB, LOBYTE( chip->Scan.Multi_TG_Dummy_Pixel));
	Mustek_SendData(chip, ES01_C8_MultiTGDummyPixelNumberMSB, HIBYTE( chip->Scan.Multi_TG_Dummy_Pixel));
	
	Mustek_SendData(chip, ES01_C9_CCDDummyPixelNumberLSB, LOBYTE( chip->Scan.CCD_Dummy_Pixel));
	Mustek_SendData(chip, ES01_CA_CCDDummyPixelNumberMSB, HIBYTE( chip->Scan.CCD_Dummy_Pixel));
	
	Mustek_SendData(chip, ES01_CB_CCDDummyCycleNumber,  chip->Scan.Dummy_Cycle);
	
	chip->firmwarestate=FS_OPENED;
	
	return STATUS_GOOD; 
}





STATUS CCDTiming(PAsic chip)
{
	if (chip->firmwarestate<FS_OPENED)
		OpenScanChip(chip);
	
	if (chip->firmwarestate>FS_OPENED)
		Mustek_SendData(chip, ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	
	Mustek_SendData(chip, ES01_CC_PHTGTimingAdjust, MULTI_TG | (chip->Timing.PHTGTimingAdjust & 0xFD));
	
	Mustek_SendData(chip, ES01_CD_PH1_0, chip->Timing.PH1_0);
	Mustek_SendData(chip, ES01_CE_PH1_1, chip->Timing.PH1_1);
	Mustek_SendData(chip, ES01_CF_PH2_0, chip->Timing.PH2_0);
	Mustek_SendData(chip, ES01_D0_PH2_1, chip->Timing.PH2_1);
	Mustek_SendData(chip, ES01_D1_PH1B_0, chip->Timing.PH1B_0);
	Mustek_SendData(chip, ES01_D2_PH1B_1, chip->Timing.PH1B_1);
	Mustek_SendData(chip, ES01_D3_PH2B_0, chip->Timing.PH2B_0);
	Mustek_SendData(chip, ES01_D4_PH2B_1, chip->Timing.PH2B_1);
	Mustek_SendData(chip, ES01_D5_PHRS_0, chip->Timing.PHRS_0);
	Mustek_SendData(chip, ES01_D6_PHRS_1, chip->Timing.PHRS_1);
	Mustek_SendData(chip, ES01_D7_PHCP_0, chip->Timing.PHCP_0);
	Mustek_SendData(chip, ES01_D8_PHCP_1, chip->Timing.PHCP_1);
	Mustek_SendData(chip, ES01_D9_AFE_VSAMP_0, chip->Timing.AFE_VSAMP_0);
	Mustek_SendData(chip, ES01_DA_AFE_VSAMP_1, chip->Timing.AFE_VSAMP_1);
	Mustek_SendData(chip, ES01_DB_AFE_RSAMP_0, chip->Timing.AFE_RSAMP_0);
	Mustek_SendData(chip, ES01_DC_AFE_RSAMP_1, chip->Timing.AFE_RSAMP_1);
	Mustek_SendData(chip, ES01_DD_PH1234_IN_DUMMY_TG, chip->Timing.PH1234_IN_DUMMY_TG);
	Mustek_SendData(chip, ES01_DE_CCD_SETUP_REGISTER, chip->Timing.CCD_SETUP_REGISTER);
	
	chip->firmwarestate=FS_OPENED;
	
	return STATUS_GOOD;
}


STATUS SetADConverter(PAsic chip)
{
	if (chip->firmwarestate < FS_OPENED)
		OpenScanChip(chip);
	
	if (chip->firmwarestate > FS_OPENED)
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
	
#ifdef _DO_HARDWARE_SETTING_
	Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, chip->Timing.HARDWARE_SETTING);
#endif
	
	Mustek_SendData(chip,ES01_9A_AFEControl, chip->Timing.AFEControl);
	
	Mustek_SendData(chip,ES01_81_AFE_CHANNEL_RED_LATCH_POS, chip->Timing.AFE_CHANNEL_RED_LATCH_POS);
	Mustek_SendData(chip,ES01_82_AFE_CHANNEL_GREEN_LATCH_POS, chip->Timing.AFE_CHANNEL_GREEN_LATCH_POS);
	Mustek_SendData(chip,ES01_83_AFE_CHANNEL_BLUE_LATCH_POS, chip->Timing.AFE_CHANNEL_BLUE_LATCH_POS);	
	Mustek_SendData(chip,ES01_84_AFE_MALC_TIMING_ADJUST_0, chip->Timing.AFE_MALC_TIMING_ADJUST_0);
	Mustek_SendData(chip,ES01_85_AFE_MALC_TIMING_ADJUST_1, chip->Timing.AFE_MALC_TIMING_ADJUST_1);
	
	Mustek_SendData(chip,ES01_00_AD9826Configuration, chip->Timing.AD9826Configuration);
	Mustek_SendData(chip,ES01_02_AD9826MuxConfig, chip->Timing.AD9826MuxConfig);
	
	Mustek_SendData(chip,ES01_04_AD9826PGARed, chip->AD.GainR);
	Mustek_SendData(chip,ES01_06_AD9826PGAGreen, chip->AD.GainG);
	Mustek_SendData(chip,ES01_08_AD9826PGABlue, chip->AD.GainB);
	
	if (chip->AD.DirectionR)
		Mustek_SendData(chip,ES01_0B_AD9826OffsetRedN, chip->AD.OffsetR);
	else
		Mustek_SendData(chip,ES01_0A_AD9826OffsetRedP, chip->AD.OffsetR);
	
	if (chip->AD.DirectionG)
		Mustek_SendData(chip,ES01_0D_AD9826OffsetGreenN, chip->AD.OffsetG);
	else
		Mustek_SendData(chip,ES01_0C_AD9826OffsetGreenP, chip->AD.OffsetG);
	
	if (chip->AD.DirectionB)
		Mustek_SendData(chip,ES01_0F_AD9826OffsetBlueN, chip->AD.OffsetB);
	else
		Mustek_SendData(chip,ES01_0E_AD9826OffsetBlueP, chip->AD.OffsetB);

	chip->firmwarestate=FS_OPENED;
}



STATUS GetLineTime(PAsic chip)
{
	BYTE temp;
	DWORD LinePixel;
	double LineTime;
	
	chip->Temp.Line_Pixel = ( (chip->Scan.TG_Pulse_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) +
		(chip->Scan.TG_Wait_Width_Pixel + 1) * (chip->Scan.TG_Times + 1) * 2 +
		(chip->Scan.CCD_Pixel_Length + 1) +
		(chip->Scan.Multi_TG_Dummy_Pixel + 1) * (chip->Scan.TG_Times > 1 ? chip->Scan.TG_Times - 1 : 0) +
		chip->Scan.CCD_Dummy_Pixel + 1) * (chip->Scan.Dummy_Cycle + 1);
	
	chip->Temp.Line_Time	= chip->Temp.Line_Pixel * PIXEL_TIME;
	
	usleep(200000);
	
	Mustek_SendData(chip,ES01_8B_Status, LINE_TIME16_19);
	temp = ES01_8B_Status;
	
	Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	Mustek_ReceiveData(chip,&temp);
	LinePixel = temp;
	
	Mustek_SendData(chip,ES01_8B_Status,LINE_TIME8_15);
	temp = ES01_8B_Status;
	
	Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	Mustek_ReceiveData(chip,&temp);
	LinePixel = (LinePixel << 8) + temp; 
	
	Mustek_SendData(chip,ES01_8B_Status,LINE_TIME0_7);
	temp = ES01_8B_Status;
	
	Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	Mustek_ReceiveData(chip,&temp);
	LinePixel = (LinePixel << 8) + temp + 1;
	
	LineTime = LinePixel * PIXEL_TIME;
	
	if (chip->Temp.Line_Time != LineTime)
		fprintf(stderr, "GetLineTime: LineTime error...");
	
	return STATUS_GOOD;
}

STATUS ReadMotorTable(PAsic chip, LPBYTE lpMotorTable,WORD wYResolution)
{
	WORD i, j;
	WORD wLowSpeed, wHighSpeed;

	if(chip->lightsource != LS_REFLECTIVE)
	{
		wLowSpeed = 2000;
		wHighSpeed =400;
	}
	else
	{
		wLowSpeed = 2000;
		wHighSpeed =200;
	}
	
	/* For every resolution (Back home)
	//Motor T0 Acc Table*/
	for (i = 0; i < 512; i++)
	{
		long double y;			
		y = (wLowSpeed - wHighSpeed);  
		y *= sin(3.1415926*i/(504*2));
		y = wLowSpeed - y;       
		*((LPWORD)lpMotorTable + i			) = (WORD)y; 
		if(i > 504)
			*((LPWORD)lpMotorTable + i			) = wHighSpeed;
	}
	
	/*Motor T1 Dec Table*/
	for (i = 0; i < 255; i++)
	{
		long double y;
		y = (wLowSpeed - wHighSpeed);   
		y *= sin(3.1415926*i/(248*2)+3.1415926/2);
		y = wHighSpeed + y;
		*((LPWORD)lpMotorTable + i + 512	) = (WORD)(y);  
		if(i >= 248)
			*((LPWORD)lpMotorTable + i + 512	) = wLowSpeed; 
	}
	
	if (wYResolution == 56)
		XDBG(DEBUG_ASIC,"ReadMotorTable: T6");
	/*Motor T6 Acc Table*/ 
	for (i = 0; i < 512; i++)
	{
		long double y;
		y = (wLowSpeed - wHighSpeed);     
		y *= sin(3.1415926*i/(504*2));
		y = wLowSpeed - y;           
		*((LPWORD)lpMotorTable + i + 512 * 6) = (WORD)(y);
		if(i>=504)
			*((LPWORD)lpMotorTable + i + 512 * 6) = (WORD)(y);
	}
	
	if (wYResolution == 56)
		XDBG(DEBUG_ASIC,"ReadMotorTable: T7");
	/*Motor T7 Dec Table*/
	for (i = 0; i < 256; i++)
	{
		*((LPWORD)lpMotorTable + i + 512 * 7) = 1000 + i * 1000 / 256;
	}
	
	switch(wYResolution)
		
	{
	case 1800:
		/*Motor T0 Acc Table*/
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y=(2000 - 300);   
			y*=sin(3.1415926*i/(504*2));
			y=2000 - y;        
			*((LPWORD)lpMotorTable + i			) = (WORD)y;  
			if(i>=504)
				*((LPWORD)lpMotorTable + i			) = 300; 
		}

		/*Motor T1 Dec Table*/
		for (i = 0; i < 256; i++)
		{
			long double y;
			y=(2000 - 300);     
			y*=sin(3.1415926*i/(248*2)+3.1415926/2);
			y=300 + y;          
			*((LPWORD)lpMotorTable + i + 512	) = (WORD)(y); 
			if(i>=248)
				*((LPWORD)lpMotorTable + i + 512	) = 2000; 
		}
		
		j = (WORD)chip->ibase1;
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(120*2));
			y = chip->ibase1-y;
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y; 
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y; 
			if(i>=120 ||((chip->UsbHost == HT_USB10 && chip->lightsource == LS_REFLECTIVE) ||chip->lightsource != LS_REFLECTIVE) ) 
			{
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2;  /*T2*/
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2;  /*T4*/
			}
		}
		
		j = (WORD)chip->ibase2;
		for (i = 0; i < 255; i++)
		{
			long double y;
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(120*2)+3.1415926/2);
			y = chip->ibase2+y;
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);  /*T3*/
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y);  /*T5*/
			if(i>=120 || ((chip->UsbHost == HT_USB10 && chip->lightsource==LS_REFLECTIVE) ||chip->lightsource!=LS_REFLECTIVE))
			{
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase2;/*ibase1;*/
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase2;/*ibase1;*/
			}
		}
		break;
		
	case 900:
    case 450:
		
		/*Motor T0 Acc Table*/
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (2000 - 400); 
			y *=sin(3.1415926*i/(504*2));
			y = 2000 - y;      
			*((LPWORD)lpMotorTable + i			) = (WORD)y; 
			if(i >= 504)
				*((LPWORD)lpMotorTable + i			) = 400; 
		}
		
		/*Motor T1 Dec Table*/
		for (i = 0; i < 256; i++)
		{
			long double y;
			y = (2000 - 400);  
			y *= sin(3.1415926*i/(248*2)+3.1415926/2);
			y = 400 + y ;   
			*((LPWORD)lpMotorTable + i + 512	) = (WORD)(y); 
			if(i >= 248)
				*((LPWORD)lpMotorTable + i + 512	) = 2000;
		}

		j = (WORD)chip->ibase1;
		XDBG(DEBUG_ASIC,"SetScanParam: chip->ibase1=%d, chip->ibase2=%d",chip->ibase1,chip->ibase2);
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(504*2));
			y = chip->ibase1-y;
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y;  /*T2*/
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y;  /*T4*/
			if(i>=504 )  
			{
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2;  /*T2*/
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2;  /*T4*/
			}
		}
		
		j = (WORD)chip->ibase2;
		for (i = 0; i < 255; i++)
		{
			long double y;
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(248*2)+3.1415926/2);
			y = chip->ibase2+y;
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y);
			if(i>=248 || ((chip->UsbHost == HT_USB10 && chip->lightsource==LS_REFLECTIVE) ||chip->lightsource!=LS_REFLECTIVE))
			{
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase2;/*ibase1;  T3*/
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase2;/*ibase1;  T5*/
			}
		}
		break;
	case 300:
	case 225:
		/*Motor T0 Acc Table*/
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (2000 - 300);    
			y *= sin(3.1415926*i/(504*2));
			y = 2000 - y;       
			*((LPWORD)lpMotorTable + i			) = (WORD)y;  
			if(i >= 504)
				*((LPWORD)lpMotorTable + i			) = 400;   
		}
		
		/*Motor T1 Dec Table*/
		for (i = 0; i < 256; i++)
		{
			long double y;
			y = (2000 - 300);    
			y *= sin(3.1415926*i/(248*2)+3.1415926/2);
			y = 300 + y;        
			*((LPWORD)lpMotorTable + i + 512	) = (WORD)(y);  
			if(i>=248)
				*((LPWORD)lpMotorTable + i + 512	) = 2000;  
		}
	
		if((chip->UsbHost==HT_USB20 && chip->lightsource==LS_REFLECTIVE))
		{
			j = (WORD)chip->ibase1;
			for (i = 0; i < 512; i++)
			{
				long double y;			
				y = (chip->ibase1-chip->ibase2);
				y *= sin(3.1415926*i/(504*2));
				y = chip->ibase1-y;
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y;  /*T2*/
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y;  /*T4*/
				if(i>=504)
				{
					*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2;  /*T2*/
					*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2;  /*T4*/
				}
			}
			
			j = (WORD)chip->ibase2;
			for (i = 0; i < 255; i++)
			{
				long double y;
				y = (chip->ibase1-chip->ibase2);
				y *= sin(3.1415926*i/(248*2)+3.1415926/2);
				y = chip->ibase2+y;
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);  /*T3*/
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y);  /*T5*/
				if(i>=248)
				{
					*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase1;  /*T3*/
					*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase1;  /*T5*/
				}
			}
		}
		else
		{
			j = (WORD)chip->ibase1;
			for (i = 0; i < 512; i++)
			{
				long double y;			
				y = (chip->ibase1-chip->ibase2);
				y *= sin(3.1415926*i/(504*2));
				y = chip->ibase1-y;
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y;  /*T2*/
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y;  /*T4*/
				if(i>=504)
				{
					*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2;  /*T2*/
					*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2;  /*T4*/
				}
			}
			
			j = (WORD)chip->ibase2;
			for (i = 0; i < 255; i++)
			{
				long double y;
				y = (chip->ibase1-chip->ibase2);
				y *= sin(3.1415926*i/(248*2)+3.1415926/2);
				y = chip->ibase2+y;
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);  /*T3*/
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y);  /*T5*/
				if(i>=248)
				{
					*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase1;  /*T3*/
					*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase1;  /*T5*/
				}
			}
		}
		break;
		
	case 150:
	case 112:
		/*Motor T0 Acc Table*/
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y =( 2000 - 200);   
			y *= sin(3.1415926*i/(504*2));
			y = 2000 - y;        
			*((LPWORD)lpMotorTable + i			) = (WORD)y; 
			if(i >= 504)
				*((LPWORD)lpMotorTable + i			) = 200; 
		}
		
		/*Motor T1 Dec Table*/
		for (i = 0; i < 256; i++)
		{
			long double y;
			y = (2000 - 200); 
			y *= sin(3.1415926*i/(256*2)+3.1415926/2);
			y = 200 + y;       
			*((LPWORD)lpMotorTable + i + 512	) = (WORD)(y); 
			if(i >= 248)
				*((LPWORD)lpMotorTable + i + 512	) = 2000;
		}
		
		j = (WORD)chip->ibase1;
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(300*2));
			y = chip->ibase1-y;
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y;  /*T2*/
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y;  /*T4*/
			if(i>=300)
			{
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2;  /*T2*/
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2;  /*T4*/
			}
		}
		
		j = (WORD)chip->ibase2;
		if((chip->ibase1+5000) < 65535)
			chip->ibase1 +=5000;
		else
			chip->ibase1 = 65535;		
		for (i = 0; i < 255; i++)
		{
			long double y;
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(120*2)+3.1415926/2);
			y = chip->ibase2+y;
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);  /*T3*/
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y);  /*T5*/
			if(i >= 120)
			{
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase1;  /*T3*/
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase1;  /*T5*/
			}
		}
		break;
	case 100:
	case 72:
		/*Motor T0 Acc Table*/
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (2000 - 200);   
			y *= sin(3.1415926*i/(504*2));
			y = 2000 - y;         
			*((LPWORD)lpMotorTable + i			) = (WORD)y; 
			if(i >= 504)
				*((LPWORD)lpMotorTable + i			) = 200;  
		}
		/*Motor T1 Dec Table*/
		for (i = 0; i < 256; i++)
		{
			long double y;
			y = (2000 - 200);  
			y *= sin(3.1415926*i/(248*2)+3.1415926/2);
			y = 200 + y;   
			*((LPWORD)lpMotorTable + i + 512	) = (WORD)(y); 
			if(i>=248)
				*((LPWORD)lpMotorTable + i + 512	) = 2000; 
		}
		
		j = (WORD)chip->ibase1;
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(300*2));
			y = chip->ibase1-y;
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y; 
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y; 
			if(i >= 300)
			{
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2; 
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2; 
			}
		}
		
		j = (WORD)chip->ibase2;
		if((chip->ibase1+6400)<65535)
			chip->ibase1 += 6400;
		else
			chip->ibase1 = 65535;		
		for (i = 0; i < 255; i++)
		{
			long double y;
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(100*2)+3.1415926/2);
			y = chip->ibase2+y;
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);  
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y); 
			if(i >= 100)
			{
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase1; 
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase1; 
			}
		}
		break;
		
	case 75:
		j = (WORD)chip->ibase1;
		for (i = 0; i < 512; i++)
		{
			long double y;			
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(504*2));
			y = chip->ibase1-y;
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y; 
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y;  
			*((LPWORD)lpMotorTable + i + 512 * 6) = (WORD)y;  
			if(i>=504)
			{
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2; 
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2; 
				*((LPWORD)lpMotorTable + i + 512 * 6) = (WORD)chip->ibase2; 
			}
		}
		
		j = (WORD)chip->ibase2;
		for (i = 0; i < 255; i++)
		{
			long double y;
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(248*2)+3.1415926/2);
			y = chip->ibase2+y;
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y); 
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y); 
			*((LPWORD)lpMotorTable + i + 512 * 7) = (WORD)(y); 
			if(i >= 248)
			{
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase1; 
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase1; 
				*((LPWORD)lpMotorTable + i + 512 * 7) = (WORD)chip->ibase1; 
			}
		}
		break;
		
	case 50:
	case 56:
	case 0:
		for (i = 0; i < 512; i++)  
		{
			long double y;			
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(504*2));
			y = chip->ibase1-y;
			*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)y; 
			*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)y; 
			if(i>=504)
			{
				*((LPWORD)lpMotorTable + i + 512 * 2) = (WORD)chip->ibase2; 
				*((LPWORD)lpMotorTable + i + 512 * 4) = (WORD)chip->ibase2; 
			}
		}
		

		for (i = 0; i < 255; i++)
		{
			long double y;
			y = (chip->ibase1-chip->ibase2);
			y *= sin(3.1415926*i/(248*2)+3.1415926/2);
			y = chip->ibase2+y;
			*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)(y);
			*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)(y);
			if(i>=248)
			{
				*((LPWORD)lpMotorTable + i + 512 * 3) = (WORD)chip->ibase1;
				*((LPWORD)lpMotorTable + i + 512 * 5) = (WORD)chip->ibase1;
			}
		}
		break;
	}
	return STATUS_GOOD;
}




STATUS WriteMotorTable(PAsic chip, LPBYTE lpMotorTable)
{
	/*Set ram type*/
	Mustek_SendData(chip,ES01_F8_DRAMType, DRAM_TIMING_1 | DRAM_256Kx16);
	
	/*set motor start address*/
	Mustek_SendData(chip,ES01_A0_HostStartAddr0_7, chip->RamPositions.Motor_0);
	Mustek_SendData(chip,ES01_A1_HostStartAddr8_15, chip->RamPositions.Motor_1);
	Mustek_SendData(chip,ES01_A2_HostStartAddr16_19, chip->RamPositions.Motor_2 | ACCESS_DRAM);
	
	/*set motor end address*/
	Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, 0xff);
	Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, 0xff);
	Mustek_SendData(chip,ES01_A5_HostEndAddr16_19, 0xff);
	
	Mustek_SendData(chip,ES01_9D_MotorTableAddrA12_A19, (BYTE)(chip->RamPositions.Motor));
	
	XDBG(DEBUG_ASIC,"WriteMotorTable: RamPositions.Motor_0=%d",(BYTE)(chip->RamPositions.Motor_0));
	XDBG(DEBUG_ASIC,"WriteMotorTable: RamPositions.Motor_1=%d",(BYTE)(chip->RamPositions.Motor_1));
	XDBG(DEBUG_ASIC,"WriteMotorTable: RamPositions.Motor_2=%d",(BYTE)(chip->RamPositions.Motor_2)| ACCESS_DRAM);
	XDBG(DEBUG_ASIC,"WriteMotorTable: RamPositions.Motor=%d",(BYTE)(chip->RamPositions.Motor));
	
	/*SW_ClearFIFO();*/
	Mustek_DMAWrite(chip,512 * 8 * 2, (LPBYTE)(lpMotorTable));

	/*Steal read 2 byte,by Roy 2005.10.7*/
	usleep(2000);
	BYTE temps[2];
	Mustek_DMAReadGeneralMode(chip, 2, temps);  
	
	return STATUS_GOOD;
	
}

STATUS MotorStepTableForScan(PAsic chip, WORD wYResolution, WORD wStartY, DWORD dwScanImageSteps)
{
	STATUS status;

	WORD wAccSteps=504;    /*T1,*/
	DWORD wForwardSteps=0;  /*T2*/
	BYTE bDecSteps=248;   /*T3*/
	WORD wMotorSycnPixelNumber=0; /*T4*/
	WORD wScanAccSteps=504;   /*T5 ,T9, T14*/
	BYTE bScanDecSteps=248;   /*T16 ,T8, T11*/
	WORD wFixScanSteps=64;   /*T6*/
	WORD wScanBackTrackingSteps=40;   /*T10*/
	WORD wScanRestartSteps=40;       /*T12*/
	WORD wScanBackHomeExtSteps=64;  /*T13*/

	dwScanImageSteps+=2;

	switch(wYResolution)
	{
	case 1800:
		wAccSteps=1;
		bDecSteps=1;
		wScanBackTrackingSteps=0;
		wScanRestartSteps=40;
		wScanBackHomeExtSteps=1;
		wMotorSycnPixelNumber=0;
		if((chip->UsbHost==HT_USB10 && chip->lightsource==LS_REFLECTIVE)||chip->lightsource!=LS_REFLECTIVE)
		{
			wScanAccSteps=504;
			bScanDecSteps=248;
			chip->Scan.TotalMotorSteps = dwScanImageSteps*MOTOR_STEP_MULTI;
			if(wStartY*MOTOR_STEP_MULTI<1256)  /*not including T0, T1 steps*/
			{
				wFixScanSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wScanAccSteps)>0 ? 
							  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wScanAccSteps):0 ;	
			    wAccSteps=1;
			    wForwardSteps=0;
			    bDecSteps=1;
			}
			else
			{
				wFixScanSteps=64;
				/*if motor 1/4 steps*/
     			wForwardSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps)>0 ?  
							  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps):0;
			}
		}else
		{
			wScanAccSteps=504;
			bScanDecSteps=248;

			chip->Scan.TotalMotorSteps = dwScanImageSteps * MOTOR_STEP_MULTI;

			if(wStartY * MOTOR_STEP_MULTI < 1256)
			{
				wFixScanSteps=(chip->Scan.StartLine-wScanAccSteps)>0 ? 
							  (chip->Scan.StartLine-wScanAccSteps):0 ;	
				wAccSteps=1;
			    wForwardSteps=0;
			    bDecSteps=1;				
			}
			else
			{
				wFixScanSteps=64;
     			wForwardSteps=((DWORD)chip->Scan.StartLine* MOTOR_STEP_MULTI-(DWORD)wAccSteps-(DWORD)bDecSteps-(DWORD)wScanAccSteps-(DWORD)wFixScanSteps)>0 ?  
							  ((DWORD)chip->Scan.StartLine* MOTOR_STEP_MULTI-(DWORD)wAccSteps-(DWORD)bDecSteps-(DWORD)wScanAccSteps-(DWORD)wFixScanSteps):0;

			}
		}
		
		wForwardSteps = DISTSTEP* MOTOR_STEP_MULTI;
		chip->Scan.TotalMotorSteps = dwScanImageSteps*MOTOR_STEP_MULTI+wForwardSteps;
		
		
		Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(chip->ibase2));
		Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(chip->ibase2));
		break;
	case 900:
		wAccSteps=504;
		bDecSteps=248;
		wScanBackTrackingSteps=40;
		wScanRestartSteps=40;
		wScanBackHomeExtSteps=64;
		wMotorSycnPixelNumber=0;

		if((chip->UsbHost==HT_USB10 && chip->lightsource==LS_REFLECTIVE) ||chip->lightsource!=LS_REFLECTIVE)
		{
			wScanAccSteps=504;
			bScanDecSteps=248;
			chip->Scan.TotalMotorSteps = dwScanImageSteps * MOTOR_STEP_MULTI; 
			if(wStartY*MOTOR_STEP_MULTI<1256)  
			{
				wFixScanSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wScanAccSteps)>0 ? 
							  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wScanAccSteps):0 ;		
				wAccSteps=1;
			    wForwardSteps=0;
			    bDecSteps=1;
			}
			else
			{
				wFixScanSteps=64;
				wForwardSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps)>0 ? 
							  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps):0;
			}
		}
		else
		{
			wScanAccSteps=504;
			bScanDecSteps=248;

			chip->Scan.TotalMotorSteps = dwScanImageSteps * MOTOR_STEP_MULTI;  
			if(wStartY*MOTOR_STEP_MULTI<1256)  
			{
				wFixScanSteps=(chip->Scan.StartLine * MOTOR_STEP_MULTI - wScanAccSteps)>0 ? 
							  (chip->Scan.StartLine * MOTOR_STEP_MULTI - wScanAccSteps):0 ;		
				wAccSteps=1;
			    wForwardSteps=0;
			    bDecSteps=1;
			}
			else
			{
				DWORD dwTempForwardSteps;
				wFixScanSteps=64;
				dwTempForwardSteps = (DWORD)chip->Scan.StartLine * MOTOR_STEP_MULTI;
				dwTempForwardSteps = dwTempForwardSteps - (DWORD)wAccSteps - (DWORD)bDecSteps - (DWORD)wScanAccSteps - (DWORD)wFixScanSteps;
				if(dwTempForwardSteps > 0)
					wForwardSteps = dwTempForwardSteps;
				else 
					wForwardSteps = 0;

			}
		}
		chip->Scan.TotalMotorSteps+= wAccSteps;
		chip->Scan.TotalMotorSteps+= wForwardSteps;
		chip->Scan.TotalMotorSteps+= bDecSteps;

		chip->Scan.TotalMotorSteps+= wScanAccSteps;
		chip->Scan.TotalMotorSteps+= wFixScanSteps; 
		chip->Scan.TotalMotorSteps+= bScanDecSteps;
		Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(chip->ibase2));
		Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(chip->ibase2));
		break;

	case 450:
	case 300:
	case 225:
		wAccSteps=504;
		bDecSteps=248;

		wScanBackHomeExtSteps=64;
		wMotorSycnPixelNumber=0;

		chip->Scan.TotalMotorSteps = dwScanImageSteps * MOTOR_STEP_MULTI; 

		
		if(wStartY*MOTOR_STEP_MULTI<1256) 
		{
			wFixScanSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wScanAccSteps)>0 ? 
						  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wScanAccSteps):0 ;		
			wAccSteps=1;
			wForwardSteps=0;
			bDecSteps=1;
		}
		else
		{
			wFixScanSteps=64;
			wForwardSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps)>0 ? 
					   	  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps):0;
		
		}

		chip->Scan.TotalMotorSteps+= wAccSteps;
		chip->Scan.TotalMotorSteps+= wForwardSteps;
		chip->Scan.TotalMotorSteps+= bDecSteps;

		chip->Scan.TotalMotorSteps+= wScanAccSteps;
		chip->Scan.TotalMotorSteps+= wFixScanSteps;
		chip->Scan.TotalMotorSteps+= bScanDecSteps;
		break;
	
	case 150:
	case 112:
		wMotorSycnPixelNumber=0;
		wScanAccSteps=504;
		bScanDecSteps=248;
		wScanBackTrackingSteps=64;
		wScanRestartSteps=64;
		wScanBackHomeExtSteps=64;
		
		chip->Scan.TotalMotorSteps = dwScanImageSteps*MOTOR_STEP_MULTI;
		
		if(wStartY*MOTOR_STEP_MULTI<1256)  
		{
			wFixScanSteps=(chip->Scan.StartLine-wScanAccSteps)>0 ? 
				          (chip->Scan.StartLine-wScanAccSteps):0 ;		
			wAccSteps=1;
			wForwardSteps=0;
			bDecSteps=1;
		}
		else
		{
			wFixScanSteps=64;
			wForwardSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps)>0 ? 
						  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps):0;
			
		}
		chip->Scan.TotalMotorSteps+= wAccSteps;
		chip->Scan.TotalMotorSteps+= wForwardSteps;
		chip->Scan.TotalMotorSteps+= bDecSteps;

		chip->Scan.TotalMotorSteps+= wScanAccSteps;
		chip->Scan.TotalMotorSteps+= wFixScanSteps;
		chip->Scan.TotalMotorSteps+= bScanDecSteps;
		break;

	case 100:
	case 72:
	case 56:
		wMotorSycnPixelNumber=0;
		wScanAccSteps=504; 
		bScanDecSteps=244;  
		wScanBackTrackingSteps=64;
		wScanRestartSteps=64;
		wScanBackHomeExtSteps=64;
		
		chip->Scan.TotalMotorSteps = dwScanImageSteps * MOTOR_STEP_MULTI;
		
		if(wStartY * MOTOR_STEP_MULTI<1256)  
		{
			wFixScanSteps=(chip->Scan.StartLine-wScanAccSteps)>0 ? 
				          (chip->Scan.StartLine-wScanAccSteps):0 ;		
			wAccSteps=1;
			wForwardSteps=0;
			bDecSteps=1;
		}
		else
		{
			wFixScanSteps=64;
			wForwardSteps=(chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps)>0 ? 
						  (chip->Scan.StartLine*MOTOR_STEP_MULTI-wAccSteps-(WORD)bDecSteps-wScanAccSteps-wFixScanSteps):0;
			
		}

		chip->Scan.TotalMotorSteps+= wAccSteps;
		chip->Scan.TotalMotorSteps+= wForwardSteps;
		chip->Scan.TotalMotorSteps+= bDecSteps;

		chip->Scan.TotalMotorSteps+= wScanAccSteps;
		chip->Scan.TotalMotorSteps+= wFixScanSteps;
		chip->Scan.TotalMotorSteps+= bScanDecSteps;
		break;

	case 75:
	case 50:
		wAccSteps=504;
		bDecSteps=248;
		wScanAccSteps=504;
		bScanDecSteps=248;
		wFixScanSteps=64;

		wForwardSteps=(chip->Scan.StartLine * MOTOR_STEP_MULTI - wAccSteps - (WORD)bDecSteps - wScanAccSteps - wFixScanSteps)>0 ? 
					  (chip->Scan.StartLine * MOTOR_STEP_MULTI - wAccSteps - (WORD)bDecSteps - wScanAccSteps - wFixScanSteps):0;
		
		wScanBackTrackingSteps=64;
		wScanRestartSteps=64;  
		wScanBackHomeExtSteps=64;
		wMotorSycnPixelNumber=0;


		chip->Scan.TotalMotorSteps = dwScanImageSteps*MOTOR_STEP_MULTI; 

		

		chip->Scan.TotalMotorSteps+= wAccSteps;
		chip->Scan.TotalMotorSteps+= wForwardSteps;
		chip->Scan.TotalMotorSteps+= bDecSteps;
		chip->Scan.TotalMotorSteps+= wScanAccSteps;
		chip->Scan.TotalMotorSteps+= wFixScanSteps;
		chip->Scan.TotalMotorSteps+= bScanDecSteps;
		break;
	}
	
	Mustek_SendData(chip,ES01_E0_MotorAccStep0_7, LOBYTE(wAccSteps));
	Mustek_SendData(chip,ES01_E1_MotorAccStep8_8, HIBYTE(wAccSteps));

	Mustek_SendData(chip,ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(wForwardSteps));
	Mustek_SendData(chip,ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(wForwardSteps));
	Mustek_SendData(chip,ES01_E4_MotorStepOfMaxSpeed16_19, (BYTE)((wForwardSteps & 0x00ff0000) >> 16));

	Mustek_SendData(chip,ES01_E5_MotorDecStep, bDecSteps);
	Mustek_SendData(chip,ES01_AE_MotorSycnPixelNumberM16LSB, LOBYTE(wMotorSycnPixelNumber));
	Mustek_SendData(chip,ES01_AF_MotorSycnPixelNumberM16MSB, HIBYTE(wMotorSycnPixelNumber));
	Mustek_SendData(chip,ES01_EC_ScanAccStep0_7, LOBYTE(wScanAccSteps));
	Mustek_SendData(chip,ES01_ED_ScanAccStep8_8, HIBYTE(wScanAccSteps));
	Mustek_SendData(chip,ES01_EE_FixScanStepLSB, LOBYTE(wFixScanSteps));
	Mustek_SendData(chip,ES01_8A_FixScanStepMSB, HIBYTE(wFixScanSteps));
	Mustek_SendData(chip,ES01_EF_ScanDecStep, bScanDecSteps);
	Mustek_SendData(chip,ES01_E6_ScanBackTrackingStepLSB, LOBYTE(wScanBackTrackingSteps));
	Mustek_SendData(chip,ES01_E7_ScanBackTrackingStepMSB, HIBYTE(wScanBackTrackingSteps));
	Mustek_SendData(chip,ES01_E8_ScanRestartStepLSB, LOBYTE(wScanRestartSteps));
	Mustek_SendData(chip,ES01_E9_ScanRestartStepMSB, HIBYTE(wScanRestartSteps));
	Mustek_SendData(chip,ES01_EA_ScanBackHomeExtStepLSB, LOBYTE(wScanBackHomeExtSteps));
	Mustek_SendData(chip,ES01_EB_ScanBackHomeExtStepMSB, HIBYTE(wScanBackHomeExtSteps));
	
	Mustek_SendData(chip,ES01_F0_ScanImageStep0_7, LOBYTE(chip->Scan.TotalMotorSteps));
	Mustek_SendData(chip,ES01_F1_ScanImageStep8_15, HIBYTE(chip->Scan.TotalMotorSteps));
	Mustek_SendData(chip,ES01_F2_ScanImageStep16_19, (BYTE) ((chip->Scan.TotalMotorSteps & 0x00ff0000) >> 16));
	return STATUS_GOOD;
}


STATUS LoadProfile(PAsic chip)
{
	return STATUS_GOOD;
}

STATUS IsCarriageHome(PAsic chip,BOOL* LampHome ,BOOL* TAHome)
{
	STATUS status = STATUS_GOOD;
	BYTE temp;
	XDBG(DEBUG_ASIC,"Asic_IsTAHomePosition: Enter");
	
	
	Mustek_SendData(chip,ES01_8B_Status, 0x02);
	Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	temp = ES01_8B_Status;
	status = Mustek_ReceiveData(chip,&temp);
	if(STATUS_GOOD != status)
	{
		fprintf(stderr,"Asic_IsTAHomePostion: CarriageHome Failed\n");
		return STATUS_IO_ERROR;
	}

	if(temp & 0x10)
		*TAHome = TRUE;
	else 
	{
		*TAHome = FALSE;
	}
	
	Mustek_SendData(chip,ES01_8B_Status, 0x01);
	Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
	temp = ES01_8B_Status;
	status = Mustek_ReceiveData(chip,&temp);
	if(STATUS_GOOD != status)
	{
		fprintf(stderr,"Asic_IsTAHomePostion: CarriageHome Failed\n");
		return STATUS_IO_ERROR;
	}

	if((temp & 0x1f)!=0)
		*LampHome = TRUE;
	else 
	{
		*LampHome = FALSE;
	}

	XDBG(DEBUG_ASIC,"Asic_IsTAHomePosition: Exit");
	return status;
}

STATUS TALAMPWaitCarriageHome(PAsic chip)
{
	STATUS status = STATUS_GOOD;
	BYTE temp;

	XDBG(DEBUG_ASIC, "TALampWaitCarriageHome: Enter");

	if(chip->motorstate == MS_STILL)
		return status;

	for (;;)
	{
		/*Test TA is Home?*/
		status = Mustek_SendData(chip,ES01_8B_Status, 0x02);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"TALampWaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
	
		status = Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"Asic_WaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
		temp = ES01_8B_Status;
		status = Mustek_ReceiveData(chip,&temp);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"Asic_WaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
		if ((temp&0x10)!= 0)
		{
			XDBG(DEBUG_ASIC,"TA is in home");
			break;
		}
		
		/*Test Lamp is Home*/
		status = Mustek_SendData(chip,ES01_8B_Status, 0x01);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"TALampWaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
	
		status = Mustek_WriteAddressLineForRegister(chip,ES01_8B_Status);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"Asic_WaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
		temp = ES01_8B_Status;
		status = Mustek_ReceiveData(chip,&temp);
		if(status != STATUS_GOOD)
		{
			fprintf(stderr,"Asic_WaitCarriageHome: CarriageHome Failed\n");
			return STATUS_IO_ERROR;
		}
		if ((temp&0x1f)== 0)
		{
			XDBG(DEBUG_ASIC,"Lamp is in home");
			break;
		}

	}
	Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_DISABLE);


	chip->firmwarestate = FS_OPENED;
	chip->motorstate = MS_STILL;

	XDBG(DEBUG_ASIC,"Asic_WaitCarriageHome: Exit");
	return status;
}

STATUS DoCarriageHome(PAsic chip, BOOL isTA)
{
	STATUS status = STATUS_GOOD;
	LPBYTE lpMotorTable;
	BYTE bMotorType = MOTOR_TYPE;
	long lMotorStep;
	
	if (!isTA)
	{
		if(chip->motorstate == MS_MOVED)
			return status;
		
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
		ResetGpio(chip);

		
		lpMotorTable = (unsigned char *) malloc (512 * 8 * 2);
		memset(lpMotorTable, 0, 512 * 8 * sizeof(WORD));
		
		chip->ibase1=CARRIAGE_HOME_iBASE1;
		chip->ibase2=CARRIAGE_HOME_iBASE2;
		ReadMotorTable(chip,lpMotorTable,0);
		
#ifdef _DO_HARDWARE_SETTING_
		Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, chip->Timing.HARDWARE_SETTING);
#endif

		Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(300));
		Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(300));
		Mustek_SendData(chip,ES01_A6_MotorOption, 
				MOTOR_0_ENABLE |
				MOTOR_1_ENABLE |
				HOME_SENSOR_0_ENABLE |
				BIPOLAR_FULL_3955_3966);
				
		Mustek_SendData(chip,ES01_E0_MotorAccStep0_7, LOBYTE(504)); 
		Mustek_SendData(chip,ES01_E1_MotorAccStep8_8, HIBYTE(504)); 
		
		lMotorStep = MOTOR_CARRIAGE_HOME_STEPS * MOTOR_STEP_MULTI - 504 - 248;
		Mustek_SendData(chip,ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(lMotorStep));
		Mustek_SendData(chip,ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(lMotorStep));
		Mustek_SendData(chip,ES01_E4_MotorStepOfMaxSpeed16_19, (BYTE)((lMotorStep & 0x00ff0000) >> 16));
		
		Mustek_SendData(chip,ES01_EA_ScanBackHomeExtStepLSB, LOBYTE(0));
		Mustek_SendData(chip,ES01_EB_ScanBackHomeExtStepMSB, HIBYTE(0));
		
		Mustek_SendData(chip,ES01_E5_MotorDecStep, 248);/*255 steps*/
		
		Mustek_SendData(chip,ES01_F8_DRAMType, DRAM_TIMING_1 | DRAM_256Kx16);
		
		Mustek_SendData(chip,ES01_A0_HostStartAddr0_7, 0x00);
		Mustek_SendData(chip,ES01_A1_HostStartAddr8_15, 0x00);
		Mustek_SendData(chip,ES01_A2_HostStartAddr16_19, 0x01 | ACCESS_DRAM);
		
		Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, 0xff);
		Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, 0xff);
		Mustek_SendData(chip,ES01_A5_HostEndAddr16_19, 0x03);
		
		/*tell scan chip the motor table address, unit is 2^13 bytes*/
		Mustek_SendData(chip,ES01_9D_MotorTableAddrA12_A19, 0x10);
		
		Mustek_DMAWrite(chip,512 * 2 * 8, ((unsigned char *)lpMotorTable));
		
		/*Steal read 2 byte,by Roy 2005.10.7*/
		usleep(2000);
		BYTE temps[2];
		Mustek_DMAReadGeneralMode(chip, 2, temps);  
		
		Mustek_SendData(chip,ES01_F6_MorotControl1, SPEED_NUIT_1_PIXEL_TIME |
									   MOTOR_SYNC_NUIT_1_PIXEL_TIME);
		
		
		Mustek_SendData(chip,ES01_F3_ActionOption, 
					MOTOR_MOVE_TO_FIRST_LINE_DISABLE |
					MOTOR_BACK_HOME_AFTER_SCAN_ENABLE |
					SCAN_DISABLE |
					SCAN_BACK_TRACKING_DISABLE |
					INVERT_MOTOR_DIRECTION_DISABLE |
				    UNIFORM_MOTOR_AND_SCAN_SPEED_DISABLE|
				    MOTOR_TEST_LOOP_DISABLE);
		
		Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_ENABLE);
		
		chip->motorstate = MS_MOVED;
		free(lpMotorTable);
		
	}
	else 
	{
		BOOL ishome;
		Mustek_SendData(chip,ES01_F4_ActiveTriger,ACTION_TRIGER_DISABLE);
		
		ResetTAGpio(chip);
		Asic_IsTAHomePosition(chip,&ishome);
		if(ishome)
			return status;
		
		ResetTAGpio(chip);
	

		lpMotorTable = (unsigned char *) malloc (512 * 8 * 2);
		memset(lpMotorTable, 0, 512 * 8 * sizeof(WORD));
		
		chip->ibase1=CARRIAGE_HOME_iBASE1;
		chip->ibase2=CARRIAGE_HOME_iBASE2;
		ReadMotorTable(chip,lpMotorTable,0);
		
#ifdef _DO_HARDWARE_SETTING_
		Mustek_SendData(chip,ES01_74_HARDWARE_SETTING, chip->Timing.HARDWARE_SETTING);
#endif
		Mustek_SendData(chip,ES01_FD_MotorFixedspeedLSB, LOBYTE(CARRIAGE_HOME_FIX_MOTOR_SPEED));
		Mustek_SendData(chip,ES01_FE_MotorFixedspeedMSB, HIBYTE(CARRIAGE_HOME_FIX_MOTOR_SPEED));


		Mustek_SendData(chip,ES01_A6_MotorOption, 
				MOTOR_0_ENABLE |
				MOTOR_1_ENABLE |
				HOME_SENSOR_1_ENABLE |
				BIPOLAR_FULL_3955_3966);

		Mustek_SendData(chip,ES01_E0_MotorAccStep0_7, LOBYTE(504)); 
		Mustek_SendData(chip,ES01_E1_MotorAccStep8_8, HIBYTE(504)); 
		
		lMotorStep = MOTOR_CARRIAGE_HOME_STEPS * MOTOR_STEP_MULTI - 504 - 248;
		Mustek_SendData(chip,ES01_E2_MotorStepOfMaxSpeed0_7, LOBYTE(lMotorStep));
		Mustek_SendData(chip,ES01_E3_MotorStepOfMaxSpeed8_15, HIBYTE(lMotorStep));
		Mustek_SendData(chip,ES01_E4_MotorStepOfMaxSpeed16_19, (BYTE)((lMotorStep & 0x00ff0000) >> 16));
		
		Mustek_SendData(chip,ES01_EA_ScanBackHomeExtStepLSB, LOBYTE(0));
		Mustek_SendData(chip,ES01_EB_ScanBackHomeExtStepMSB, HIBYTE(0));
		
		Mustek_SendData(chip,ES01_E5_MotorDecStep, 248);
		
		Mustek_SendData(chip,ES01_F8_DRAMType, DRAM_TIMING_1 | DRAM_256Kx16);
		
		Mustek_SendData(chip,ES01_A0_HostStartAddr0_7, 0x00);
		Mustek_SendData(chip,ES01_A1_HostStartAddr8_15, 0x00);
		Mustek_SendData(chip,ES01_A2_HostStartAddr16_19, 0x01 | ACCESS_DRAM);
		
		Mustek_SendData(chip,ES01_A3_HostEndAddr0_7, 0xff);
		Mustek_SendData(chip,ES01_A4_HostEndAddr8_15, 0xff);
		Mustek_SendData(chip,ES01_A5_HostEndAddr16_19, 0x03);
		
		Mustek_SendData(chip,ES01_9D_MotorTableAddrA12_A19, 0x10);
		
		Mustek_DMAWrite(chip,512 * 2 * 8, ((unsigned char *)lpMotorTable));
		/*Steal read 2 byte,by Roy 2005.10.7*/
		usleep(2000);
		BYTE temps[2];
		Mustek_DMAReadGeneralMode(chip, 2, temps);  
		
		
		Mustek_SendData(chip,ES01_F6_MorotControl1, SPEED_NUIT_1_PIXEL_TIME |
									   MOTOR_SYNC_NUIT_1_PIXEL_TIME);
		
		
		Mustek_SendData(chip,ES01_F3_ActionOption, 
					MOTOR_MOVE_TO_FIRST_LINE_DISABLE |
					MOTOR_BACK_HOME_AFTER_SCAN_ENABLE |
					SCAN_DISABLE |
					SCAN_BACK_TRACKING_DISABLE |
					INVERT_MOTOR_DIRECTION_DISABLE |
				    UNIFORM_MOTOR_AND_SCAN_SPEED_DISABLE|
				    MOTOR_TEST_LOOP_DISABLE);
		
		Mustek_SendData(chip,ES01_F4_ActiveTriger, ACTION_TRIGER_ENABLE);
		
		chip->motorstate = MS_MOVED;
		free(lpMotorTable);
	}	
	return status;
}
