/* sane - Scanner Access Now Easy.
	Copyright (C) 2005 Mustek.
	Originally maintained by Mustek
	Author: Roy 2005.09

	This file is part of the SANE package.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston,
	MA 02111-1307, USA.

	As a special exception, the authors of SANE give permission for
	additional uses of the libraries contained in this release of SANE.

	The exception is that, if you link a SANE library with other files
	to produce an executable, this does not by itself cause the
	resulting executable to be covered by the GNU General Public
	License.  Your use of that executable is in no way restricted on
	account of linking the SANE library code into it.

	This exception does not, however, invalidate any other reasons why
	the executable file might be covered by the GNU General Public
	License.

	If you submit changes to SANE to the maintainers to be included in
	a subsequent release, you agree by submitting the changes that
	those changes may be distributed with this exception intact.

	If you write modifications of your own for SANE, it is your choice
	whether to permit this exception to apply to your modifications.
	If you do not wish that, delete this exception notice.

	This file provides a generic USB interface.  */
#ifndef _DEBUG_H_
#define _DEBUG_H_

#include <stdarg.h>
#include <stdio.h>

/*#define _DEBUG_*/
#undef _DEBUG_

#define DEBUG_ASIC 1
#define DEBUG_SPICALL 2
#define DEBUG_SCANOBJ 4
#define DEBUG_PANEL 8
#define DEBUG_SANE 16
#define DEBUG_LIBUSB 32

#define DEBUG_LEVEL 3
/*	(DEBUG_ASIC  | DEBUG_SPICALL | DEBUG_SCANOBJ | DEBUG_PANEL | DEBUG_SANE)*/

#ifdef _DEBUG_

#define XDBG(level,fmt,msg...) PDebugInfo(level,fmt, ##msg)

#else

#define XDBG(level,fmt,msg...)

#endif

static void PDebugInfo(int level, const char* fmt,...)
{
	va_list ap; 
	
	if (level == 0)
		return;
	
	va_start(ap,fmt);

	if (0 == (level & DEBUG_LEVEL)) 
	{	
		va_end(ap);
		return;
	}

	if (0 != (level & DEBUG_ASIC))
	{
		fprintf(stderr,"[ASIC]: ");
		vfprintf(stderr,fmt,ap);
		fprintf(stderr,"\n");
	}

	if (0 != (level & DEBUG_SPICALL))
	{
		fprintf(stderr,"[SPICALL]: ");
		vfprintf(stderr,fmt,ap);
		fprintf(stderr,"\n");
	}
	
	if (0 != (level & DEBUG_SCANOBJ))
	{
		fprintf(stderr,"[SCANOBJ]: ");
		vfprintf(stderr,fmt,ap);
		fprintf(stderr,"\n");
	}

	if (0 != (level & DEBUG_PANEL))
	{
		fprintf(stderr,"[PANEL]: ");
		vfprintf(stderr,fmt,ap);
		fprintf(stderr,"\n");		
	}
	
	if (0 != (level & DEBUG_SANE))
	{
		fprintf(stderr,"[SANE]: ");
		vfprintf(stderr,fmt,ap);
		fprintf(stderr,"\n");		
	}

	if (0 != (level & DEBUG_LIBUSB))
	{
		fprintf(stderr,"[LIBUSB]: ");
		vfprintf(stderr,fmt,ap);
		fprintf(stderr,"\n");
	}
  
	va_end(ap);
	
	return;
}


#endif /* End of file debug.h */
