/* sane - Scanner Access Now Easy.
	Copyright (C) 2005 Mustek.
	Originally maintained by Mustek
	Author:Jason 2003.04  Roy 2005.09

	This file is part of the SANE package.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston,
	MA 02111-1307, USA.

	As a special exception, the authors of SANE give permission for
	additional uses of the libraries contained in this release of SANE.

	The exception is that, if you link a SANE library with other files
	to produce an executable, this does not by itself cause the
	resulting executable to be covered by the GNU General Public
	License.  Your use of that executable is in no way restricted on
	account of linking the SANE library code into it.

	This exception does not, however, invalidate any other reasons why
	the executable file might be covered by the GNU General Public
	License.

	If you submit changes to SANE to the maintainers to be included in
	a subsequent release, you agree by submitting the changes that
	those changes may be distributed with this exception intact.

	If you write modifications of your own for SANE, it is your choice
	whether to permit this exception to apply to your modifications.
	If you do not wish that, delete this exception notice.

	This file provides a generic USB interface.  */
#include <stdlib.h>
#include <unistd.h>

#include "../include/typedef.h"
#include "../include/asic.h"
#include "../include/spicall.h"
#include "../include/public.h"
#include "../include/ini.h"
#include "MustScanner.h"
#include "ScannerTransparent.h"
#include "ScannerDefine.h"
#include "spicall_ex.h"


BOOL Transparent_Reset()
{
	XDBG(DEBUG_SPICALL,"Transparent_Reset: call in ");
	if (g_bOpened) 
	{
		XDBG(DEBUG_SPICALL,"Transparent_Reset: scanner has been opened");
		return FALSE;
	}
	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"Transparent_Reset: can not open scanner");
		return FALSE;
	}

	Asic_Reset(&g_chip);
	if(STATUS_GOOD!=Asic_TurnTA(&g_chip,TRUE))
	{
		XDBG(DEBUG_SPICALL,"ransparent_Reset: Asic_TurnTA return error");
		return FALSE;
	}

	Asic_Close(&g_chip);
	
	g_Y=0;
	g_wLineartThreshold=128;
	g_dwTotalTotalXferLines=0;
	g_bFirstReadImage=TRUE;

	g_pGammaTable=NULL;

	
	return TRUE;
}

BOOL Transparent_ScanSuggest(PTARGETIMAGE pTarget, PSUGGESTSETTING pSuggest)
{
	WORD wMaxWidth,	wMaxHeight;
	int i;

	XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: call in");
		
	for (i=0;s_wOpticalYDpi[i]!=0;i++)
	{
		if (s_wOpticalYDpi[i] <= pTarget->wDpi) 
		{
			pSuggest->wYDpi=s_wOpticalYDpi[i];
			break;
		}
	}
	if (s_wOpticalYDpi[i]==0) 
	{
		i--;
		pSuggest->wYDpi=s_wOpticalYDpi[i];
	}

	for (i=0;s_wOpticalXDpi[i]!=0;i++) 
	{
		if (s_wOpticalXDpi[i] <= pTarget->wDpi)
		{
			pSuggest->wXDpi=s_wOpticalXDpi[i];
			break;
		}
	}
	if (s_wOpticalXDpi[i]==0) 
	{
		i--;
		pSuggest->wXDpi=s_wOpticalXDpi[i];
	}

	pSuggest->wX=(WORD)(((DWORD)(pTarget->wX)*(DWORD)(pSuggest->wXDpi))/(DWORD)(pTarget->wDpi));
	pSuggest->wY=(WORD)(((DWORD)(pTarget->wY)*(DWORD)(pSuggest->wYDpi))/(DWORD)(pTarget->wDpi));
	pSuggest->wWidth=(WORD)(((DWORD)(pTarget->wWidth)*(DWORD)(pSuggest->wXDpi))/(DWORD)(pTarget->wDpi));
	pSuggest->wHeight=(WORD)(((DWORD)(pTarget->wHeight)*(DWORD)(pSuggest->wYDpi))/(DWORD)(pTarget->wDpi));

	if(pTarget->cmColorMode==CM_TEXT)
	{
		pSuggest->wWidth =((pSuggest->wWidth+7)>>3)<<3;
		if( pSuggest->wWidth < 8)
			pSuggest->wWidth = 8;
	}

	if(pSuggest->wXDpi==56)
		g_Width=(((pSuggest->wWidth*2+511)>>9)<<9)>>1;  
	else
		g_Width=((pSuggest->wWidth+511)>>9)<<9;  
	g_Height=pSuggest->wHeight;

	wMaxWidth=(MAX_SCANNING_WIDTH*pSuggest->wXDpi)/300; 
	wMaxHeight=(MAX_SCANNING_HEIGHT*pSuggest->wYDpi)/300; 
	
	if(pTarget->cmColorMode==CM_TEXT)
		wMaxWidth=(wMaxWidth>>3)<<3;		

	if (pSuggest->wWidth>wMaxWidth)
		pSuggest->wWidth=wMaxWidth;
	if (pSuggest->wHeight>wMaxHeight)
		pSuggest->wHeight=wMaxHeight;

	if (pTarget->isOptimalSpeed) 
	{
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: isOptimalSpeed  is true");

		switch (pTarget->cmColorMode) 
		{
		case CM_RGB48:
			pSuggest->cmScanMode=CM_RGB48;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth)*6);
			break;
		case CM_RGB24:
			pSuggest->cmScanMode=CM_RGB24ext;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth)*3);
			break;
		case CM_GRAY16:
			pSuggest->cmScanMode=CM_GRAY16ext;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth)*2);
			break;
		case CM_GRAY8:
			pSuggest->cmScanMode=CM_GRAY8ext;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth));
			break;
		case CM_TEXT:
			pSuggest->cmScanMode=CM_TEXT;
			pSuggest->dwBytesPerRow=(DWORD)(pSuggest->wWidth)/8;
			break;
		}
	}
	else 
	{	
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: isOptimalSpeed  not true");

		switch (pTarget->cmColorMode)
		{
		case CM_RGB48:
			pSuggest->cmScanMode=CM_RGB48;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth)*6);
			break;
		case CM_RGB24:
			pSuggest->cmScanMode=CM_RGB24ext;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth)*3);
			break;
		case CM_GRAY16:
			pSuggest->cmScanMode=CM_GRAY16ext;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth)*2);
			break;
		case CM_GRAY8:
			pSuggest->cmScanMode=CM_GRAY8ext;
			pSuggest->dwBytesPerRow=(DWORD)((pSuggest->wWidth));
			break;
		case CM_TEXT:
			pSuggest->cmScanMode=CM_TEXT;
			pSuggest->dwBytesPerRow=(DWORD)(pSuggest->wWidth)/8;
			break;
		}
	}

	return TRUE;
}

BOOL Transparent_SetupScan(COLORMODE ColorMode,WORD XDpi,WORD YDpi,BOOL isInvert,WORD X,WORD Y,WORD Width,WORD Height)
{
	BOOL hasTA;
	WORD wTAShadingMinus=0;
	
	
	XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: call in");

	if (g_bOpened) 
	{
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: scanner has been opened");
		return FALSE;
	}

	if (!g_bPrepared) 
	{
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: scanner not prepared");
		return FALSE;
	}

	g_ScanMode=ColorMode;
	g_XDpi=XDpi;
	g_YDpi=YDpi;
	g_SWWidth=Width;	
	g_SWHeight=Height;

	switch(g_YDpi)
	{
	case 1800:
		g_wPixelDistance=4;  
		g_wLineDistance=32; 
		g_Height+=g_wPixelDistance;
		break;
	case 900:
		g_wPixelDistance=0;     
		g_wLineDistance=16;   
		g_Height+=g_wPixelDistance;
		break;
	case 450:
		g_wPixelDistance=0;  
		g_wLineDistance=8;   
		break;
	case 225:
		g_wPixelDistance=0;  
		g_wLineDistance=4;   
		break;
	case 112:
		g_wPixelDistance=0; 
		g_wLineDistance=2;  
		break;
	case 100:
	case 72:
	case 56:
		g_wPixelDistance=0; 
		g_wLineDistance=1;  
		break;
	case 50:
		g_wPixelDistance=0; 
		g_wLineDistance=1;
		break;
	default:
		g_wLineDistance=0;
	}

	XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: g_YDpi=%d",g_YDpi);
	XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: g_wLineDistance=%d",g_wLineDistance);
	XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: g_wPixelDistance=%d",g_wPixelDistance);



	switch(g_ScanMode)
	{
	case CM_RGB48:
		g_BytesPerRow=6*g_Width;		/* ASIC limit : width must be 8x*/
		g_SWBytesPerRow=6*g_SWWidth;		/* ASIC limit : width must be 8x*/
		g_bScanBits = 48;
		g_Height+=g_wLineDistance*2;
		break;
	case CM_RGB24ext:
		g_BytesPerRow=3*g_Width;		/* ASIC limit : width must be 8x*/
		g_SWBytesPerRow=3*g_SWWidth;	
		g_bScanBits = 24;
		g_Height+=g_wLineDistance*2;
		break;
	case CM_GRAY16ext:
		g_BytesPerRow=2*g_Width;		/* ASIC limit : width must be 8x*/
		g_SWBytesPerRow=2*g_SWWidth;	
		g_bScanBits = 16;
		break;

	case CM_GRAY8ext:
	case CM_TEXT:
		g_BytesPerRow=g_Width;		/* ASIC limit : width must be 8x*/
		g_SWBytesPerRow=g_SWWidth;		
		g_bScanBits = 8;
		break;
	}


		
	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: Asic_Open return error");
		return FALSE;
	}

	g_bOpened=TRUE;
	if(Asic_TurnTA(&g_chip,TRUE)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: Asic_TurnTA return error");
		return FALSE;
	}
	
	if(Asic_IsTAConnected(&g_chip,&hasTA)!=STATUS_GOOD)
	{	
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: Asic_IsTAConnected return error");
		return FALSE;
	}
	if(!hasTA)
	{
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: no TA device");
		return FALSE;
	}
	
	
	if(Asic_CarriageHome(&g_chip,TRUE)!=STATUS_GOOD)
	{
		XDBG(DEBUG_SPICALL,"Transparent_ScanSuggest: Asic_CarriageHome return error");
		return FALSE;
	}

	Asic_WaitCarriageHome(&g_chip,TRUE);
	Asic_WaitUnitReady(&g_chip);
	
	
	Asic_MotorMove(&g_chip,TRUE, 250 * MOTOR_STEP_MULTI);
	Transparent_AdjustAD();
	
	
	Asic_MotorMove(&g_chip,TRUE, 1100 * MOTOR_STEP_MULTI);
	Transparent_FindTopLeft(&g_X,&g_Y);
	
	
	g_X=(g_X*1800/FIND_LEFT_TOP_CALIBRATE_RESOLUTION)+(X*1800/g_XDpi) -480;  

	Asic_MotorMove(&g_chip,FALSE , 1000 * MOTOR_STEP_MULTI ); 
	Transparent_LineCalibration16Bits(wTAShadingMinus);
	
	g_Y = (Y * 1800 / g_YDpi) + 996 - BEFORE_SCANNING_MOTOR_FORWARD_PIXEL - LINE_WHITE_CALIBRATION__16B_HEIGHT;  
	
	if(g_YDpi == 1800)
	{
		Asic_MotorMove(&g_chip,TRUE, (g_Y-DISTSTEP) * MOTOR_STEP_MULTI); 
	}
	
	Asic_SetWindow(&g_chip,g_bScanBits, g_XDpi, g_YDpi, g_X, g_Y, g_Width, g_Height);
	
	
	return Transparent_PrepareScan();
}

BOOL Transparent_StopScan()
{
	XDBG(DEBUG_SPICALL,"Transparent_StopScan: call in");

	if (!g_bOpened) 
	{
		return FALSE;
	}
	if (!g_bPrepared)
	{
		return FALSE;
	}
		

	g_isCanceled=TRUE;   

	pthread_cancel(g_threadid_readimage);
	pthread_join(g_threadid_readimage,NULL);

	XDBG(DEBUG_SPICALL,"Transparent_StopScan: thread exit");

	Asic_ScanStop(&g_chip);
	Asic_Close(&g_chip);
	g_bOpened=FALSE;

	return TRUE;

}


BOOL Transparent_GetRows(BYTE* lpBlock,WORD* Rows,BOOL isOrderInvert)
{

	XDBG(DEBUG_SPICALL,"Transparent_GetRows: call in");

	if (!g_bOpened) 
	{
		return FALSE;
	}
	if (!g_bPrepared)
	{
		return FALSE;
	}

	switch(g_ScanMode)
	{		
	case CM_RGB48:
		if(g_XDpi==1800)
			return MustScanner_GetRgb48BitLine1800DPI(lpBlock,isOrderInvert,Rows);
		else
			return MustScanner_GetRgb48BitLine(lpBlock,isOrderInvert,Rows);
		
	case CM_RGB24ext:
		if(g_XDpi==1800)
			return MustScanner_GetRgb24BitLine1800DPI(lpBlock,isOrderInvert,Rows);
		else			
			return MustScanner_GetRgb24BitLine(lpBlock,isOrderInvert,Rows);
		
	case CM_GRAY16ext:
		if(g_XDpi==1800)
			return MustScanner_GetMono16BitLine1800DPI(lpBlock,isOrderInvert,Rows);
		else
			return MustScanner_GetMono16BitLine(lpBlock,isOrderInvert,Rows);
		
	case CM_GRAY8ext:
		if(g_XDpi==1800)
			return MustScanner_GetMono8BitLine1800DPI(lpBlock,isOrderInvert,Rows);
		else			
			return MustScanner_GetMono8BitLine(lpBlock,isOrderInvert,Rows);
						
	case CM_TEXT:
		if(g_XDpi==1800)
			return MustScanner_GetMono1BitLine1800DPI(lpBlock,isOrderInvert,Rows);
		else
			return MustScanner_GetMono1BitLine(lpBlock,isOrderInvert,Rows);		
	}

}

BOOL Transparent_AdjustAD()
{
	LPBYTE lpCalData;
	WORD wCalWidth;
	int nTimesOfCal,k;
	float fRFactor=1.0;
	float fGFactor=1.0;
	float fBFactor=1.0;
	BYTE bDarkMaxLevel;
	BYTE bDarkMinLevel;
	WORD wMaxValueR, wMinValueR, wMaxValueG, wMinValueG, wMaxValueB, wMinValueB;
	BYTE bLastMinR, bLastROffset, bROffsetUpperBound = 255, bROffsetLowerBound = 0;
	BYTE bLastMinG, bLastGOffset, bGOffsetUpperBound = 255, bGOffsetLowerBound = 0;
	BYTE bLastMinB, bLastBOffset, bBOffsetUpperBound = 255, bBOffsetLowerBound = 0;

	XDBG(DEBUG_SPICALL,"Transparent_AdjustAD: call in");

	if (!g_bOpened) 
	{
		return FALSE;
	}
	if (!g_bPrepared)
	{
		return FALSE;
	}


	g_chip.AD.DirectionR=R_DIRECTION;
	g_chip.AD.DirectionG=G_DIRECTION;
	g_chip.AD.DirectionB=B_DIRECTION;
	g_chip.AD.GainR=R_GAIN;
	g_chip.AD.GainG=G_GAIN;
	g_chip.AD.GainB=B_GAIN;
	g_chip.AD.OffsetR=R_OFFSET;
	g_chip.AD.OffsetG=G_OFFSET;
	g_chip.AD.OffsetB=B_OFFSET;

	if(g_XDpi<=900)
		wCalWidth=(((CALIBRATION_PIXEL_WIDTH + 20) * g_XDpi / 1800 + 511)>>9)<<9;
	else
		wCalWidth=(((CALIBRATION_PIXEL_WIDTH + 20) / 2 + 511) >> 9) << 9; 

	lpCalData = (LPBYTE)malloc(sizeof(BYTE)*wCalWidth*3);
	
	if(lpCalData==NULL)
	{
		return FALSE;
	}

	Asic_SetMotorType(&g_chip,FALSE,TRUE);
	
	Asic_SetCalibrate(&g_chip,24, g_XDpi, g_XDpi, 0, 0, wCalWidth, 1, FALSE);
	
	nTimesOfCal=0;
	
	bDarkMaxLevel = TRAN_DARK_MAX_LEVEL - 10;
	bDarkMinLevel = TRAN_DARK_MIN_LEVEL - 10;
	
	do{
		Asic_SetADConverter(&g_chip);
		Asic_ScanStart(&g_chip);
		Asic_ReadCalibrationData(&g_chip,lpCalData,wCalWidth*3,24);
		Asic_ScanStop(&g_chip);
		
		Asic_TACalculateMaxMin(&g_chip,lpCalData,&wMaxValueR,&wMinValueR,g_XDpi);
		Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth,&wMaxValueG,&wMinValueG,g_XDpi);
		Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth*2,&wMaxValueB,&wMinValueB,g_XDpi);
		

		if(nTimesOfCal != 0)  
		{
			if(wMaxValueR == 255)
				g_chip.AD.OffsetR = 0;
			else MustScanner_AdjustOffset(nTimesOfCal-1, &(g_chip.AD.DirectionR), &(g_chip.AD.OffsetR), &bLastMinR, &bLastROffset, &wMinValueR, &bROffsetUpperBound, &bROffsetLowerBound, TRAN_DARK_MIN_LEVEL, TRAN_DARK_MAX_LEVEL);
			
			if(wMaxValueG == 255)
				g_chip.AD.OffsetG = 0;
			else MustScanner_AdjustOffset(nTimesOfCal-1, &(g_chip.AD.DirectionG), &(g_chip.AD.OffsetG), &bLastMinG, &bLastGOffset, &wMinValueG, &bGOffsetUpperBound, &bGOffsetLowerBound, TRAN_DARK_MIN_LEVEL, TRAN_DARK_MAX_LEVEL);
			
			if(wMaxValueB == 255)
				g_chip.AD.OffsetB = 0;
			else MustScanner_AdjustOffset(nTimesOfCal-1, &(g_chip.AD.DirectionB), &(g_chip.AD.OffsetB), &bLastMinB, &bLastBOffset, &wMinValueB, &bBOffsetUpperBound, &bBOffsetLowerBound, TRAN_DARK_MIN_LEVEL, TRAN_DARK_MAX_LEVEL);
		}
		
		nTimesOfCal++;
		if(nTimesOfCal>10)
			break;
		
	}while(wMinValueR>bDarkMaxLevel || wMinValueR<bDarkMinLevel
		||wMinValueG>bDarkMaxLevel || wMinValueG<bDarkMinLevel
		||wMinValueB>bDarkMaxLevel || wMinValueB<bDarkMinLevel);
	
	Asic_SetADConverter(&g_chip);
	Asic_ScanStart(&g_chip);
	Asic_ReadCalibrationData(&g_chip,lpCalData,wCalWidth*3,24);
	Asic_ScanStop(&g_chip);
	
	Asic_TACalculateMaxMin(&g_chip,lpCalData,&wMaxValueR,&wMinValueR,g_XDpi);
	Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth,&wMaxValueG,&wMinValueG,g_XDpi);
	Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth*2,&wMaxValueB,&wMinValueB,g_XDpi);
	

	g_chip.AD.GainR=1-(double)(wMaxValueR-wMinValueR)/TRAN_MIN_LEVEL_RANGE>0 ? 
		(BYTE)(((1-(double)(wMaxValueR-wMinValueR)/TRAN_MIN_LEVEL_RANGE))*63*6/5) : 0 ;
	g_chip.AD.GainG=1-(double)(wMaxValueG-wMinValueG)/TRAN_MIN_LEVEL_RANGE>0 ? 
		(BYTE)(((1-(double)(wMaxValueG-wMinValueG)/TRAN_MIN_LEVEL_RANGE))*63*6/5) : 0 ;
	g_chip.AD.GainB=1-(double)(wMaxValueB-wMinValueB)/TRAN_MIN_LEVEL_RANGE>0 ? 
		(BYTE)(((1-(double)(wMaxValueB-wMinValueB)/TRAN_MIN_LEVEL_RANGE))*63*6/5) : 0 ;
	
	if(g_chip.AD.GainR>63) g_chip.AD.GainR=63;
	if(g_chip.AD.GainG>63) g_chip.AD.GainG=63;
	if(g_chip.AD.GainB>63) g_chip.AD.GainB=63;
	

	Asic_SetADConverter(&g_chip);
	Asic_ScanStart(&g_chip);
	Asic_ReadCalibrationData(&g_chip,lpCalData,wCalWidth*3,24);
	Asic_ScanStop(&g_chip);
	

	Asic_CalculateMaxMin(&g_chip,lpCalData,&wMaxValueR,&wMinValueR,g_XDpi);
	Asic_CalculateMaxMin(&g_chip,lpCalData+wCalWidth,&wMaxValueG,&wMinValueG,g_XDpi);
	Asic_CalculateMaxMin(&g_chip,lpCalData+wCalWidth*2,&wMaxValueB,&wMinValueB,g_XDpi);
	
	for(k=0;k < 2; k++)
	{
		nTimesOfCal=0;
		do
		{
			/*R Channel*/
			if((wMaxValueR-wMinValueR)>TRAN_MAX_LEVEL_RANGE)
			{
				if(g_chip.AD.GainR>0)
					g_chip.AD.GainR--;
			}
			else
			{
				if((wMaxValueR-wMinValueR)<TRAN_MIN_LEVEL_RANGE)
				{
					if(wMaxValueR<TRAN_WHITE_MIN_LEVEL)
					{
						g_chip.AD.GainR++;
						if(g_chip.AD.GainR>63) g_chip.AD.GainR=63;
					} 
					else
					{
						if(wMaxValueR>TRAN_WHITE_MAX_LEVEL)
						{
							if(g_chip.AD.GainR< 1) g_chip.AD.GainR=0;
							else g_chip.AD.GainR--;
						}
						else
						{
							if(g_chip.AD.GainR > 63) g_chip.AD.GainR=63;
							else g_chip.AD.GainR++;
						}
					}
				}
				else 
				{
					if(wMaxValueR>TRAN_WHITE_MAX_LEVEL)
					{
						if(g_chip.AD.GainR< 1) g_chip.AD.GainR=0;
						else g_chip.AD.GainR--;
					}
					
					if(wMaxValueR<TRAN_WHITE_MIN_LEVEL)
					{
						if(g_chip.AD.GainR > 63) g_chip.AD.GainR=63;
						else g_chip.AD.GainR++;
					}
				}
			}
			
			/*G Channel */
			if((wMaxValueG-wMinValueG)>TRAN_MAX_LEVEL_RANGE)
			{
				if(g_chip.AD.GainG>0)
					g_chip.AD.GainG--;
			}
			else 
			{
				if((wMaxValueG-wMinValueG)<TRAN_MIN_LEVEL_RANGE)
				{
					if(wMaxValueG<TRAN_WHITE_MIN_LEVEL)
					{
						g_chip.AD.GainG++;
						if(g_chip.AD.GainG>63) g_chip.AD.GainG=63;
					}
					else
					{
						if(wMaxValueG>TRAN_WHITE_MAX_LEVEL)
						{
							if(g_chip.AD.GainG< 1) g_chip.AD.GainG=0;
							else g_chip.AD.GainG--;
						}
						else
						{
							if(g_chip.AD.GainG > 63) g_chip.AD.GainG=63;
							else g_chip.AD.GainG++;
						}
					}
				}
				else
				{
					if(wMaxValueG>TRAN_WHITE_MAX_LEVEL)
					{
						if(g_chip.AD.GainG< 1) g_chip.AD.GainG=0;
						else g_chip.AD.GainG--;
					}
					
					if(wMaxValueG<TRAN_WHITE_MIN_LEVEL)
					{
						if(g_chip.AD.GainG > 63) g_chip.AD.GainG=63;
						else g_chip.AD.GainG++;
					}
				}
			}
			
			/* B Channel*/
			if((wMaxValueB-wMinValueB)>TRAN_MAX_LEVEL_RANGE)
			{
				if(g_chip.AD.GainB>0)
					g_chip.AD.GainB--;
			}
			else 
			{
				if((wMaxValueB-wMinValueB)<TRAN_MIN_LEVEL_RANGE)
				{
					if(wMaxValueB<TRAN_WHITE_MIN_LEVEL)
					{
						g_chip.AD.GainB++;
						if(g_chip.AD.GainB>63) g_chip.AD.GainB=63;
					}
					else
					{
						if(wMaxValueB>TRAN_WHITE_MAX_LEVEL)
						{
							if(g_chip.AD.GainB< 1) g_chip.AD.GainB=0;
							else g_chip.AD.GainB--;
						}
						else
						{
							if(g_chip.AD.GainB > 63) g_chip.AD.GainB=63;
							else g_chip.AD.GainB++;
						}
					}
				}
				else
				{
					if(wMaxValueB>TRAN_WHITE_MAX_LEVEL)
					{
						if(g_chip.AD.GainB< 1) g_chip.AD.GainB=0;
						else g_chip.AD.GainB--;
					}
					
					if(wMaxValueB<TRAN_WHITE_MIN_LEVEL)
					{
						if(g_chip.AD.GainB > 63) g_chip.AD.GainB=63;
						else g_chip.AD.GainB++;
					}
				}
			}
			
			Asic_SetADConverter(&g_chip);
			Asic_ScanStart(&g_chip);
			Asic_ReadCalibrationData(&g_chip,lpCalData,wCalWidth*3,24);
			Asic_ScanStop(&g_chip);
			
			
			Asic_TACalculateMaxMin(&g_chip,lpCalData,&wMaxValueR,&wMinValueR,g_XDpi);			    
			Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth,&wMaxValueG,&wMinValueG,g_XDpi);
			Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth*2,&wMaxValueB,&wMinValueB,g_XDpi);
			
			
			nTimesOfCal++;
			if(nTimesOfCal>10)
				break;
		}while((wMaxValueR-wMinValueR)>TRAN_MAX_LEVEL_RANGE || (wMaxValueR-wMinValueR)<TRAN_MIN_LEVEL_RANGE
			||(wMaxValueG-wMinValueG)>TRAN_MAX_LEVEL_RANGE || (wMaxValueG-wMinValueG)<TRAN_MIN_LEVEL_RANGE
			||(wMaxValueB-wMinValueB)>TRAN_MAX_LEVEL_RANGE || (wMaxValueB-wMinValueB)<TRAN_MIN_LEVEL_RANGE);
		
		
		/* Adjust Offset 2nd*/
		nTimesOfCal=0;
		do
		{
			Asic_SetADConverter(&g_chip);
			Asic_ScanStart(&g_chip);
			Asic_ReadCalibrationData(&g_chip,lpCalData,wCalWidth*3,24);
			Asic_ScanStop(&g_chip);
			
			Asic_TACalculateMaxMin(&g_chip,lpCalData,&wMaxValueR,&wMinValueR,g_XDpi);
			Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth,&wMaxValueG,&wMinValueG,g_XDpi);
			Asic_TACalculateMaxMin(&g_chip,lpCalData+wCalWidth*2,&wMaxValueB,&wMinValueB,g_XDpi);
			
			MustScanner_SecondAdjustOffset(nTimesOfCal, &(g_chip.AD.DirectionR), &(g_chip.AD.OffsetR), &bLastMinR, &bLastROffset, &wMinValueR, &bROffsetUpperBound, &bROffsetLowerBound, TRAN_DARK_MIN_LEVEL, TRAN_DARK_MAX_LEVEL);
			
			MustScanner_SecondAdjustOffset(nTimesOfCal, &(g_chip.AD.DirectionG), &(g_chip.AD.OffsetG), &bLastMinG, &bLastGOffset, &wMinValueG, &bGOffsetUpperBound, &bGOffsetLowerBound, TRAN_DARK_MIN_LEVEL, TRAN_DARK_MAX_LEVEL);
			
			MustScanner_SecondAdjustOffset(nTimesOfCal, &(g_chip.AD.DirectionB), &(g_chip.AD.OffsetB), &bLastMinB, &bLastBOffset, &wMinValueB, &bBOffsetUpperBound, &bBOffsetLowerBound, TRAN_DARK_MIN_LEVEL, TRAN_DARK_MAX_LEVEL);
			
			nTimesOfCal++;
			if(nTimesOfCal>8)
				break;
			
		}while(wMinValueR>TRAN_DARK_MAX_LEVEL || wMinValueR<TRAN_DARK_MIN_LEVEL
			||wMinValueG>TRAN_DARK_MAX_LEVEL || wMinValueG<TRAN_DARK_MIN_LEVEL
			||wMinValueB>TRAN_DARK_MAX_LEVEL || wMinValueB<TRAN_DARK_MIN_LEVEL);
		
	}
		
	free(lpCalData);
}

BOOL Transparent_FindTopLeft(WORD* lpwStartX, WORD* lpwStartY)
{
	WORD wCalWidth=TA_FIND_LEFT_TOP_WIDTH_IN_DIP;   
	WORD wCalHeight=TA_FIND_LEFT_TOP_HEIGHT_IN_DIP;  

	int i,j;
	WORD wLeftSide;
	WORD wTopSide;
	int nScanBlock;
	LPBYTE lpCalData;
	DWORD dwTotalSize;


	XDBG(DEBUG_SPICALL,"Transparent_FindTopLeft: call in");

	*lpwStartX=187;  
	*lpwStartY=43;   

	if (!g_bOpened) 
	{
		XDBG(DEBUG_SPICALL,"Transparent_FindTopLeft: scanner not opened");

		return FALSE;
	}
	if (!g_bPrepared)
	{
		XDBG(DEBUG_SPICALL,"Transparent_FindTopLeft: scanner not prepared");
		return FALSE;
	}


	lpCalData = (LPBYTE)malloc(sizeof(BYTE)*wCalWidth * wCalHeight);
	if(lpCalData==NULL)
	{
		XDBG(DEBUG_SPICALL,"Transparent_FindTopLeft: lpCalData malloc fail");
		return FALSE;
	}

	dwTotalSize=wCalWidth * wCalHeight ;
	nScanBlock=(int)(dwTotalSize/g_dwCalibrationSize);
	
	
	Asic_TurnLampEngineTA(&g_chip,TRUE);  
	sleep(1);
	Asic_SetMotorType(&g_chip,TRUE,TRUE);
	
	Asic_SetCalibrate(&g_chip,8, 450, 450, 0, 0, wCalWidth, wCalHeight, FALSE);
	Asic_ScanStart(&g_chip);
	
	for(i=0; i<nScanBlock; i++)
		Asic_ReadCalibrationData(&g_chip,lpCalData+i*g_dwCalibrationSize,g_dwCalibrationSize,8);
	
	Asic_ReadCalibrationData(&g_chip,lpCalData+(nScanBlock)*g_dwCalibrationSize, (dwTotalSize-g_dwCalibrationSize*nScanBlock) , 8);
	Asic_ScanStop(&g_chip);
	
	Asic_TurnTA(&g_chip,TRUE); 
	
	wLeftSide=0;
	wTopSide=0;
	
	for(i=(wCalWidth-1); i>0; i--)
	{
		wLeftSide=*(lpCalData+i);
		wLeftSide+=*(lpCalData+wCalWidth*2+i);
		wLeftSide+=*(lpCalData+wCalWidth*4+i);
		wLeftSide+=*(lpCalData+wCalWidth*6+i);
		wLeftSide+=*(lpCalData+wCalWidth*8+i);
		wLeftSide/=5;
		if(wLeftSide<60)
		{
			if(i==(wCalWidth-1)) 
				break;  
			
			*lpwStartX=i;
			break;
		}
	}
	
	for(j=0 ;j<wCalHeight; j++)
	{
		wTopSide=*(lpCalData+wCalWidth*j+i-2);
		wTopSide+=*(lpCalData+wCalWidth*j+i-4);
		wTopSide+=*(lpCalData+wCalWidth*j+i-6);
		wTopSide+=*(lpCalData+wCalWidth*j+i-8);
		wTopSide+=*(lpCalData+wCalWidth*j+i-10);
		wTopSide/=5;
		if(wTopSide>60)
		{
			if(j==0)
				break;
			
			*lpwStartY=j;
			break;
		}
	}
	
	if((*lpwStartX < 100) || (*lpwStartX > 250))
	{
		*lpwStartX = 187;
	}
	
	if((*lpwStartY < 10) ||(*lpwStartY > 100))
	{
		*lpwStartY = 43;
	}
	
	Asic_MotorMove(&g_chip,FALSE,(wCalHeight-*lpwStartY + TA_MOTOR_BACK_STEP_AFTER_FIND_BOUNDARY)*MOTOR_STEP_MULTI);
	Asic_WaitCarriageHome(&g_chip,FALSE);
	
	free(lpCalData);

	return TRUE;
}

BOOL Transparent_LineCalibration16Bits()
{		
	WORD *lpWhiteShading;
	WORD *lpDarkShading;
	double wRWhiteLevel=0;
	double wGWhiteLevel=0;
	double wBWhiteLevel=0;
	DWORD dwRDarkLevel=0;
	DWORD dwGDarkLevel=0;
	DWORD dwBDarkLevel=0;
	DWORD dwREvenDarkLevel=0;
	DWORD dwGEvenDarkLevel=0;
	DWORD dwBEvenDarkLevel=0;
	LPWORD lpRWhiteSort;
	LPWORD lpGWhiteSort;
	LPWORD lpBWhiteSort;
	LPWORD lpRDarkSort;
	LPWORD lpGDarkSort;
	LPWORD lpBDarkSort;
	int i,j;
	
	LPBYTE lpWhiteData;
	LPBYTE lpDarkData;
	DWORD dwWhiteTotalSize;
	DWORD dwDarkTotalSize;
	WORD wWhiteCalHeight = TA_WHITE_CALIBRATION__16B_HEIGHT;  
	WORD wCalWidth;
	WORD wNoUseImageData;
	WORD wDarkCalHeight = TA_DARK_CALIBRATION__16B_HEIGHT; 

	BOOL isDoSoftware = FALSE;
	int FiltLower_LBound;
	int FileLower_UBound;
	
	BOOL bUniDark;
	BOOL b900UniDark;
	

	XDBG(DEBUG_SPICALL,"Transparent_LineCalibration16Bits: call in");

	if (!g_bOpened) 
	{
		XDBG(DEBUG_SPICALL,"Transparent_LineCalibration16Bits: scanner not opened");
		return FALSE;
	}
	if (!g_bPrepared)
	{
		XDBG(DEBUG_SPICALL,"Transparent_LineCalibration16Bits: scanner not prepared");
		return FALSE;
	}
	
	
	wCalWidth=g_Width;
	
	dwWhiteTotalSize=wCalWidth*wWhiteCalHeight*3*2;
	dwDarkTotalSize=wCalWidth*wDarkCalHeight*3*2;
	lpWhiteData = (LPBYTE)malloc(sizeof(BYTE)*dwWhiteTotalSize);
	lpDarkData = (LPBYTE)malloc(sizeof(BYTE)*dwDarkTotalSize);
	if( lpWhiteData==NULL || lpDarkData==NULL )
	{
		XDBG(DEBUG_SPICALL,"Transparent_LineCalibration16Bits: lpWhiteData or lpDarkData malloc fail");
		return FALSE;
	}
	
	Asic_SetMotorType(&g_chip,TRUE,TRUE);
	
	
#ifdef HARDWARE_SHADING_SWITCH_ON
	if((g_XDpi > 900) && (g_Width > HARDWARE_SHADING_WIDTH * g_XDpi)  && (g_bScanBits >= 8))
#else
	if((g_XDpi > 900) && (g_bScanBits >= 8)) 
#endif
		Asic_SetCalibrate(&g_chip,48, g_XDpi, 900, g_X, 0, wCalWidth, wWhiteCalHeight, FALSE);
	else
		Asic_SetCalibrate(&g_chip,48, g_XDpi, 900, g_X, 0, wCalWidth, wWhiteCalHeight, TRUE);
		
	Asic_ScanStart(&g_chip);
	
	Asic_ReadCalibrationData(&g_chip,lpWhiteData,dwWhiteTotalSize,8);
	
	
	wNoUseImageData=(WORD)(dwWhiteTotalSize%512);  
	if(wNoUseImageData > 0)
		Asic_ReadNoUseImage(&g_chip,wNoUseImageData);
	
	Asic_ScanStop(&g_chip);
	
	
	Asic_MotorMove(&g_chip,TRUE, TA_MOTOR_FORWARD_STEP_AFTER_READ_WHITE_DATA * MOTOR_STEP_MULTI);
	
	Asic_SetMotorType(&g_chip,FALSE,TRUE);
		
#ifdef HARDWARE_SHADING_SWITCH_ON
	if((g_XDpi > 900) && (g_Width > HARDWARE_SHADING_WIDTH * g_XDpi)  && (g_bScanBits >= 8)) 
#else
	if((g_XDpi > 900) && (g_bScanBits >= 8)) 
#endif
		Asic_SetCalibrate(&g_chip,48, g_XDpi, 900, g_X, 0, wCalWidth, wDarkCalHeight, FALSE);
	else
		Asic_SetCalibrate(&g_chip,48, g_XDpi, 900, g_X, 0, wCalWidth, wDarkCalHeight, TRUE);
			
	usleep(500000);
	
	Asic_ScanStart(&g_chip);
	Asic_ReadCalibrationData(&g_chip,lpDarkData,dwDarkTotalSize,8);
	
	wNoUseImageData=(WORD)(dwDarkTotalSize%512); 
	if(wNoUseImageData > 0)
		Asic_ReadNoUseImage(&g_chip,wNoUseImageData);
	
	Asic_ScanStop(&g_chip);
	
	Asic_TurnTA(&g_chip,TRUE);
	
	
	lpWhiteShading=(WORD*)malloc(sizeof(WORD)*wCalWidth*3);
	lpDarkShading=(WORD*)malloc(sizeof(WORD)*wCalWidth*3);
	
	lpRWhiteSort=(WORD*)malloc(sizeof(WORD)*wWhiteCalHeight);
	lpGWhiteSort=(WORD*)malloc(sizeof(WORD)*wWhiteCalHeight);
	lpBWhiteSort=(WORD*)malloc(sizeof(WORD)*wWhiteCalHeight);
	lpRDarkSort=(WORD*)malloc(sizeof(WORD)*wDarkCalHeight);
	lpGDarkSort=(WORD*)malloc(sizeof(WORD)*wDarkCalHeight);
	lpBDarkSort=(WORD*)malloc(sizeof(WORD)*wDarkCalHeight);
	
	if( lpWhiteShading==NULL || lpDarkShading==NULL 
		|| lpRWhiteSort==NULL || lpGWhiteSort==NULL || lpBWhiteSort==NULL
		|| lpRDarkSort==NULL || lpGDarkSort==NULL || lpBDarkSort==NULL)
	{
		XDBG(DEBUG_SPICALL,"Transparent_LineCalibration16Bits: malloc fail");

		free(lpWhiteData);
		free(lpDarkData);
		return FALSE;
	}
		
		
#ifdef HARDWARE_SHADING_SWITCH_ON
	if((g_XDpi > 900) && (g_Width > HARDWARE_SHADING_WIDTH * g_XDpi)  && (g_bScanBits >= 8)) 
#else
	if((g_XDpi > 900) && (g_bScanBits >= 8)) 
#endif
		isDoSoftware = TRUE;
				

	dwRDarkLevel=0;
	dwGDarkLevel=0;
	dwBDarkLevel=0;
	dwREvenDarkLevel=0;
	dwGEvenDarkLevel=0;
	dwBEvenDarkLevel=0;
				
	if(wDarkCalHeight == 10)
	{
		FiltLower_LBound = 6;
		FileLower_UBound = 10;
	}
	else
	{
		if(wDarkCalHeight == 20)
		{
			FiltLower_LBound = 10;
			FileLower_UBound = 20;
		}
		else
		{
			FiltLower_LBound = 20;
			FileLower_UBound = 40;;
			
		}
				
	}
				
	bUniDark = OptionReadInt("[spicall]","bUniDark",0,INIFILENAME);
	b900UniDark = OptionReadInt("[spicall]","b900UniDark",0,INIFILENAME);
				
	for(i=0; i<wCalWidth; i++)
	{
		for(j=0; j<wDarkCalHeight; j++)
		{
			lpRDarkSort[j] =(WORD)(*(lpDarkData+j*wCalWidth*6+i*6+0));
			lpRDarkSort[j]+=(WORD)(*(lpDarkData+j*wCalWidth*6+i*6+1)<<8);
			
			lpGDarkSort[j] =(WORD)(*(lpDarkData+j*wCalWidth*6+i*6+2));
			lpGDarkSort[j]+=(WORD)(*(lpDarkData+j*wCalWidth*6+i*6+3)<<8);
			
			lpBDarkSort[j] =(WORD)(*(lpDarkData+j*wCalWidth*6+i*6+4));
			lpBDarkSort[j]+=(WORD)(*(lpDarkData+j*wCalWidth*6+i*6+5)<<8);
		}
		
		if(g_XDpi==1800)
		{
			if(bUniDark)
			{
				if(i%2)
				{
					dwRDarkLevel+=(DWORD)MustScanner_FiltLower(lpRDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
					dwGDarkLevel+=(DWORD)MustScanner_FiltLower(lpGDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
					dwBDarkLevel+=(DWORD)MustScanner_FiltLower(lpBDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				}
				else
				{
					dwREvenDarkLevel+=(DWORD)MustScanner_FiltLower(lpRDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
					dwGEvenDarkLevel+=(DWORD)MustScanner_FiltLower(lpGDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
					dwBEvenDarkLevel+=(DWORD)MustScanner_FiltLower(lpBDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				}
			} 
			else
			{
				*(lpDarkShading+i*3+0)=(WORD)MustScanner_FiltLower(lpRDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				*(lpDarkShading+i*3+1)=(WORD)MustScanner_FiltLower(lpGDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				*(lpDarkShading+i*3+2)=(WORD)MustScanner_FiltLower(lpBDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
			}
		}
		else
		{
			if(b900UniDark)
			{
				dwRDarkLevel+=(DWORD)MustScanner_FiltLower(lpRDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				dwGDarkLevel+=(DWORD)MustScanner_FiltLower(lpGDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				dwBDarkLevel+=(DWORD)MustScanner_FiltLower(lpBDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
			}
			{
				*(lpDarkShading+i*3+0)=(WORD)MustScanner_FiltLower(lpRDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				*(lpDarkShading+i*3+1)=(WORD)MustScanner_FiltLower(lpGDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
				*(lpDarkShading+i*3+2)=(WORD)MustScanner_FiltLower(lpBDarkSort,wDarkCalHeight,FiltLower_LBound,FileLower_UBound);
			}
		}
				
	}
				
	if((g_XDpi==1800) && (bUniDark))
	{
		dwRDarkLevel=(DWORD)(dwRDarkLevel/(wCalWidth/2));
		dwGDarkLevel=(DWORD)(dwGDarkLevel/(wCalWidth/2));
		dwBDarkLevel=(DWORD)(dwBDarkLevel/(wCalWidth/2));
		
		dwREvenDarkLevel=(DWORD)(dwREvenDarkLevel/(wCalWidth/2));
		dwGEvenDarkLevel=(DWORD)(dwGEvenDarkLevel/(wCalWidth/2));
		dwBEvenDarkLevel=(DWORD)(dwBEvenDarkLevel/(wCalWidth/2));
		
		for(i=0; i<wCalWidth; i++)
		{
			if(i%2)
			{
				*(lpDarkShading+i*3+0)=(WORD)dwRDarkLevel;
				*(lpDarkShading+i*3+1)=(WORD)dwGDarkLevel;
				*(lpDarkShading+i*3+2)=(WORD)dwBDarkLevel;
			}
			else
			{
				*(lpDarkShading+i*3+0)=(WORD)dwREvenDarkLevel;
				*(lpDarkShading+i*3+1)=(WORD)dwGEvenDarkLevel;
				*(lpDarkShading+i*3+2)=(WORD)dwBEvenDarkLevel;
			}
		}				
	}
				
	if((g_XDpi < 1800) && (b900UniDark))
	{
		dwRDarkLevel=(DWORD)(dwRDarkLevel / wCalWidth);
		dwGDarkLevel=(DWORD)(dwGDarkLevel / wCalWidth);
		dwBDarkLevel=(DWORD)(dwBDarkLevel / wCalWidth);
		
		for(i=0; i<wCalWidth; i++)
		{
			*(lpDarkShading+i*3+0) =(*(lpDarkShading+i*3+0) + (WORD)dwRDarkLevel) >> 2;
			*(lpDarkShading+i*3+1) =(*(lpDarkShading+i*3+1) + (WORD)dwGDarkLevel) >> 2;
			*(lpDarkShading+i*3+2) =(*(lpDarkShading+i*3+2) + (WORD)dwBDarkLevel) >> 2;
			

		}
				
	}

	if(wWhiteCalHeight == 20)
	{
		FiltLower_LBound = 5;
		FileLower_UBound = 15;				
	}
	else
	{
		if(wWhiteCalHeight == 40)
		{
			FiltLower_LBound = 10;
			FileLower_UBound = 30;
		}				
		else
		{
			FiltLower_LBound = 30;
			FileLower_UBound = 50;
		}			
	}
				
	for(i=0; i<wCalWidth; i++)
	{
		wRWhiteLevel=0;
		wGWhiteLevel=0;
		wBWhiteLevel=0;
					
		for(j=0; j<wWhiteCalHeight; j++)
		{
			lpRWhiteSort[j] =(WORD)(*(lpWhiteData+j*wCalWidth*2*3+i*6+0));
			lpRWhiteSort[j]+=(WORD)(*(lpWhiteData+j*wCalWidth*2*3+i*6+1)<<8);
						
			lpGWhiteSort[j] =(WORD)(*(lpWhiteData+j*wCalWidth*2*3+i*6+2));
			lpGWhiteSort[j]+=(WORD)(*(lpWhiteData+j*wCalWidth*2*3+i*6+3)<<8);
						
			lpBWhiteSort[j] =(WORD)(*(lpWhiteData+j*wCalWidth*2*3+i*6+4));
			lpBWhiteSort[j]+=(WORD)(*(lpWhiteData+j*wCalWidth*2*3+i*6+5)<<8);
		}
					
					
		wRWhiteLevel=(double)(MustScanner_FiltLower(lpRWhiteSort,wWhiteCalHeight,FiltLower_LBound,FileLower_UBound)-*(lpDarkShading+i*3+0));
		wGWhiteLevel=(double)(MustScanner_FiltLower(lpGWhiteSort,wWhiteCalHeight,FiltLower_LBound,FileLower_UBound)-*(lpDarkShading+i*3+1));
		wBWhiteLevel=(double)(MustScanner_FiltLower(lpBWhiteSort,wWhiteCalHeight,FiltLower_LBound,FileLower_UBound)-*(lpDarkShading+i*3+2));
					
		if(isDoSoftware) 
		{
			*(lpWhiteShading+i*3+0)=(WORD)wRWhiteLevel;  
			*(lpWhiteShading+i*3+1)=(WORD)wGWhiteLevel;  
			*(lpWhiteShading+i*3+2)=(WORD)wBWhiteLevel;   
		}
		else
		{
			if(g_ssScanSource==SS_Negative)
			{
				if(wRWhiteLevel>0)
					*(lpWhiteShading+i*3+0)=(WORD)((float)65536/wRWhiteLevel*0x4000); 
				else
					*(lpWhiteShading+i*3+0)=0x4000; 
				
				if(wGWhiteLevel>0)
					*(lpWhiteShading+i*3+1)=(WORD)((float)65536/wGWhiteLevel*0x4000); 
				else
					*(lpWhiteShading+i*3+1)=0x4000; 
				
				if(wBWhiteLevel>0)
					*(lpWhiteShading+i*3+2)=(WORD)((float)65536/wBWhiteLevel*0x4000);  
				else
					*(lpWhiteShading+i*3+2)=0x4000;  
			}
			else
			{
				if(wRWhiteLevel>0)
					*(lpWhiteShading+i*3+0)=(WORD)((float)65536/wRWhiteLevel*0x4000); 
				else
					*(lpWhiteShading+i*3+0)=0x4000;  
				
				if(wGWhiteLevel>0)
					*(lpWhiteShading+i*3+1)=(WORD)((float)65536/wGWhiteLevel*0x4000);
				else
					*(lpWhiteShading+i*3+1)=0x4000;  
				
				if(wBWhiteLevel>0)
					*(lpWhiteShading+i*3+2)=(WORD)((float)65536/wBWhiteLevel*0x4000); 
				else
					*(lpWhiteShading+i*3+2)=0x4000; 
			}
		}				
	}
				
	free(lpWhiteData);
	free(lpDarkData);
	free(lpRWhiteSort);
	free(lpGWhiteSort);
	free(lpBWhiteSort);
	free(lpRDarkSort);
	free(lpGDarkSort);
	free(lpBDarkSort);
				
#ifdef HARDWARE_SHADING_SWITCH_ON
	if((g_XDpi > 900) && (g_Width > HARDWARE_SHADING_WIDTH * g_XDpi)  && (g_bScanBits >= 8))
#else
	if((g_XDpi > 900) && (g_bScanBits >= 8))
#endif
	{
		Asic_SetSoftwareShadingTable(&g_chip,lpWhiteShading, lpDarkShading, g_XDpi, wCalWidth, 0); 
	}
	else
	{
		Asic_SetShadingTable(&g_chip,lpWhiteShading, lpDarkShading, g_XDpi, wCalWidth, 0);
	}
					
	free(lpWhiteShading);
	free(lpDarkShading);
					
	return TRUE;
}


BOOL Transparent_PrepareScan()
{
	g_wScanLinesPerBlock = (WORD)(g_dwBufferSize/g_BytesPerRow);
	g_wMaxScanLines=(WORD)(g_dwImageBufferSize/g_BytesPerRow);
	g_wMaxScanLines=(g_wMaxScanLines/g_wScanLinesPerBlock)*g_wScanLinesPerBlock;
	g_isCanceled=FALSE;

	g_dwScannedTotalLines=0;
	g_wReadedLines=0;
	g_wtheReadyLines=0;
	g_wReadImageLines=0;

	g_wReadyShadingLine = 0;
	g_wStartShadingLinePos = 0;
	
	
	XDBG(DEBUG_SPICALL,"Transparent_PrepareScan: call in");

	switch(g_ScanMode)
	{
	case CM_RGB48:
		
		g_wtheReadyLines=g_wLineDistance*2+g_wPixelDistance;
		
		g_lpReadImageHead=(LPBYTE)malloc(g_dwImageBufferSize);
		if(g_lpReadImageHead==NULL)
		{
			XDBG(DEBUG_SPICALL,"Transparent_PrepareScan:malloc fail");
			return FALSE;
		}
		break;
		
	case CM_RGB24ext:		
		g_wtheReadyLines=g_wLineDistance*2+g_wPixelDistance;
		g_lpReadImageHead=(LPBYTE)malloc(g_dwImageBufferSize);

		if(g_lpReadImageHead==NULL)
		{
			XDBG(DEBUG_SPICALL,"Transparent_PrepareScan:malloc fail");
			return FALSE;
		}
		break;

	case CM_GRAY16ext:		
		g_wtheReadyLines=g_wPixelDistance;
		g_lpReadImageHead=(LPBYTE)malloc(g_dwImageBufferSize);
		if(g_lpReadImageHead==NULL)
		{
			XDBG(DEBUG_SPICALL,"Transparent_PrepareScan:malloc fail");
			return FALSE;
		}
		break;
		
	case CM_GRAY8ext:		
		g_wtheReadyLines=g_wPixelDistance;
		g_lpReadImageHead=(LPBYTE)malloc(g_dwImageBufferSize);
		if(g_lpReadImageHead==NULL)
		{
			XDBG(DEBUG_SPICALL,"Transparent_PrepareScan:malloc fail");
			return FALSE;
		}
		break;
		
	case CM_TEXT:		
		g_wtheReadyLines=g_wPixelDistance;
		g_lpReadImageHead=(LPBYTE)malloc(g_dwImageBufferSize);
		if(g_lpReadImageHead==NULL)
		{
			XDBG(DEBUG_SPICALL,"Transparent_PrepareScan:malloc fail");
			return FALSE;
		}
		break;
	default:
		return FALSE;		
	}
	
	Asic_ScanStart(&g_chip);
	return TRUE;
}

