/* sane - Scanner Access Now Easy.
	Copyright (C) 2005 Mustek.
	Originally maintained by Mustek
	Author:Jason 2003.04  Roy 2005.09

	This file is part of the SANE package.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of the
	License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston,
	MA 02111-1307, USA.

	As a special exception, the authors of SANE give permission for
	additional uses of the libraries contained in this release of SANE.

	The exception is that, if you link a SANE library with other files
	to produce an executable, this does not by itself cause the
	resulting executable to be covered by the GNU General Public
	License.  Your use of that executable is in no way restricted on
	account of linking the SANE library code into it.

	This exception does not, however, invalidate any other reasons why
	the executable file might be covered by the GNU General Public
	License.

	If you submit changes to SANE to the maintainers to be included in
	a subsequent release, you agree by submitting the changes that
	those changes may be distributed with this exception intact.

	If you write modifications of your own for SANE, it is your choice
	whether to permit this exception to apply to your modifications.
	If you do not wish that, delete this exception notice.

	This file provides a generic USB interface.  */

#include <stdlib.h>
#include <math.h>

#include "../include/spicall.h"
#include "../include/typedef.h"
#include "../include/ini.h"
#include "../include/asic.h"
#include "../include/debug.h"
#include "ScannerDefine.h"

#include "spicall_ex.h"
#include "MustScanner.h"
#include "ScannerReflective.h"
#include "ScannerTransparent.h"



void AutoLevel(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine);
void QBETDetectAutoLevel(void* pDIB, DWORD ImageWidth, DWORD ImageHeight);
void QBetChange(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine);


BOOL _Open()
{
	XDBG(DEBUG_SPICALL,"_Open:call in ");
	g_pDeviceFile = (LPBYTE)malloc(200);

	if(NULL==g_pDeviceFile)
	{
		return FALSE;
	}

	OptionReadString("[spicall]","DeviceFileName","/dev/usb/scanner0",g_pDeviceFile,200,INIFILENAME);

	XDBG(DEBUG_SPICALL,"_Open:leave _Open");
	return MustScanner_Init();	
}


BOOL _Close()
{
	XDBG(DEBUG_SPICALL,"_Close:call in ");
	if(g_pDeviceFile!=NULL)
	{
		free(g_pDeviceFile);
		g_pDeviceFile=NULL;
	}
	XDBG(DEBUG_SPICALL,"_Close:leave _Close");

	return TRUE;


}


BOOL GetDeviceStatus()
{
	XDBG(DEBUG_SPICALL,"GetDeviceStatus:call in ");
	return MustScanner_GetScannerState();

}


BOOL PowerControl(BOOL isLampOn, BOOL isTALampOn)
{
	XDBG(DEBUG_SPICALL,"PowerControl:call in ");

	return MustScanner_PowerControl(isLampOn,isTALampOn);


}


BOOL CarriageHome()
{
	XDBG(DEBUG_SPICALL,"CarriageHome:call in ");

	return MustScanner_BackHome();
}

BOOL GetGammaInfo(LPGAMMAINFO pGammaInfo)
{
	XDBG(DEBUG_SPICALL,"GetGammaInfo:call in ");

	switch(pGammaInfo->smScanMode) 
	{
		case SM_GRAY:
			pGammaInfo->wInputGammaBits=12;
			pGammaInfo->wOutputGammaBits=8;
			break;
		case SM_RGB24:
			pGammaInfo->wInputGammaBits=12;
			pGammaInfo->wOutputGammaBits=8;
			break;
		case SM_GRAY16:
			pGammaInfo->wInputGammaBits=16;
			pGammaInfo->wOutputGammaBits=16;
			break;
		case SM_RGB48:
			pGammaInfo->wInputGammaBits=16;
			pGammaInfo->wOutputGammaBits=16;
			break;
		default:
			pGammaInfo->wInputGammaBits=0;
			pGammaInfo->wOutputGammaBits=0;
			return FALSE;
	}

	return TRUE;
}


BOOL SetParameters (LPSETPARAMETERS pSetParameters)
{
	WORD X1inTargetDpi;
	WORD Y1inTargetDpi;
	WORD X2inTargetDpi;
	WORD Y2inTargetDpi;


	XDBG(DEBUG_SPICALL,"SetParameters:call in ");

	if(ST_Reflective==g_ScanType)
	{
		Reflective_Reset();			 
	}
	else
	{
		Transparent_Reset();			
	}

	switch(pSetParameters->smScanMode) 
	{
		case SM_TEXT:
			g_tiTarget.cmColorMode = CM_TEXT;
			break;
		case SM_GRAY:
			g_tiTarget.cmColorMode = CM_GRAY8;
			break;
		case SM_GRAY16:
			g_tiTarget.cmColorMode = CM_GRAY16;
			break;
		case SM_RGB24:
			g_tiTarget.cmColorMode = CM_RGB24;
			break;
		case SM_RGB48:
			g_tiTarget.cmColorMode = CM_RGB48;
			break;
		default:
			return FALSE;
	}

	g_ssScanSource = pSetParameters->ssScanSource;
	g_tiTarget.bScanSource = pSetParameters->ssScanSource;

	if (pSetParameters->ssScanSource==SS_Reflective)
	{
		g_ScanType = ST_Reflective;
	}
	else if(pSetParameters->ssScanSource==SS_Positive
		  ||pSetParameters->ssScanSource==SS_Negative
		  ||pSetParameters->ssScanSource==SS_ADF)
	{
		g_ScanType = ST_Transparent;
	}
	else
	{
		XDBG(DEBUG_SPICALL,"SetParameters: ScanSource error");

		return FALSE;
	}

	if (pSetParameters->pfPixelFlavor==PF_BlackIs0
		||pSetParameters->pfPixelFlavor==PF_WhiteIs0) 
	{	
		g_PixelFlavor = pSetParameters->pfPixelFlavor;		
	}
	else
	{
		XDBG(DEBUG_SPICALL,"SetParameters: PixelFlavor error");
		return FALSE;
	}


	if(pSetParameters->fmArea.x1>=pSetParameters->fmArea.x2) 
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: x1 > x2, error");
		return FALSE;
	}
	if(pSetParameters->fmArea.y1>=pSetParameters->fmArea.y2) 
	{
		XDBG(DEBUG_SPICALL,"SetParameters: y1 >= y2, error");
		return FALSE;
	}
	if(pSetParameters->fmArea.x2>MAX_SCANNING_WIDTH) 
	{
		XDBG(DEBUG_SPICALL,"SetParameters: x2 > MAX_SCANNING_WIDTH, error");
		return FALSE;
	}
	if (pSetParameters->fmArea.y2>MAX_SCANNING_HEIGHT)
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: y2 > MAX_SCANNING_HEIGHT, error");
		return FALSE;
	}

	X1inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.x1)*(DWORD)(pSetParameters->wTargetDPI)/300L);
	Y1inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.y1)*(DWORD)(pSetParameters->wTargetDPI)/300L);
	X2inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.x2)*(DWORD)(pSetParameters->wTargetDPI)/300L);
	Y2inTargetDpi=(WORD)((DWORD)(pSetParameters->fmArea.y2)*(DWORD)(pSetParameters->wTargetDPI)/300L);
	g_tiTarget.isOptimalSpeed=TRUE;

	g_tiTarget.wDpi=pSetParameters->wTargetDPI;
	g_tiTarget.wX=X1inTargetDpi;
	g_tiTarget.wY=Y1inTargetDpi;
	g_tiTarget.wWidth = X2inTargetDpi-X1inTargetDpi;           
	g_tiTarget.wHeight = Y2inTargetDpi-Y1inTargetDpi;
	
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wDpi=%d",g_tiTarget.wDpi);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wX=%d",g_tiTarget.wX);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wY=%d",g_tiTarget.wY);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wWidth=%d",g_tiTarget.wWidth);
	XDBG(DEBUG_SPICALL,"SetParameters: g_tiTarget.wHeight=%d",g_tiTarget.wHeight);



	if(MustScanner_Prepare(g_tiTarget.bScanSource)==FALSE)
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: MustScanner_Prepare fail");

		return FALSE;
	}
	if(pSetParameters->wLinearThreshold>256&&pSetParameters->smScanMode==SM_TEXT)
	{
		XDBG(DEBUG_SPICALL,"SetParameters: LinearThreshold error");
		return FALSE;
	}
	else
	{
		g_wLineartThreshold = pSetParameters->wLinearThreshold;
	}



	if(pSetParameters->pGammaTable!=NULL)
	{
		XDBG(DEBUG_SPICALL,"SetParameters: IN gamma table not NULL");

		g_pGammaTable = pSetParameters->pGammaTable;
		g_isSelfGamma = FALSE;
	}

	else if(pSetParameters->smScanMode==SM_GRAY || pSetParameters->smScanMode==SM_RGB24)
	{
		WORD i;
		BYTE byGammaData;
		double pow_d;
		double pow_z=(double)10/20.0;

		g_pGammaTable = (WORD*)malloc(sizeof(WORD)*4096*3);
		
		XDBG(DEBUG_SPICALL,"SetParameters: gamma table malloc %d Bytes ",sizeof(WORD)*4096*3);

		XDBG(DEBUG_SPICALL,"SetParameters: address of g_pGammaTable=%d",g_pGammaTable);

		if(g_pGammaTable==NULL)
		{
			XDBG(DEBUG_SPICALL,"SetParameters: gamma table malloc fail");
			return FALSE;
		}
		g_isSelfGamma = TRUE;

		for(i=0;i<4096;i++)
		{
			pow_d=(double)i/(double)4096;
			
			byGammaData=(BYTE)( pow(pow_d,pow_z)*255 );

			*(g_pGammaTable+i) = byGammaData;
			*(g_pGammaTable+i+4096) = byGammaData;
			*(g_pGammaTable+i+8192) = byGammaData;
		}

	}
	else if(pSetParameters->smScanMode==SM_GRAY16 || pSetParameters->smScanMode==SM_RGB48)
	{
		unsigned int i,wGammaData;
		g_pGammaTable = (WORD*)malloc(sizeof(WORD)*65536*3);	

		if(g_pGammaTable==NULL)
		{
			XDBG(DEBUG_SPICALL,"SetParameters: gamma table malloc fail");
			return FALSE;
		}
		g_isSelfGamma = TRUE;

		for(i=0;i<65536;i++)
		{
			wGammaData=(WORD)(pow((((float)i)/65536.0),(((float)10)/16.0))*65535);

			*(g_pGammaTable+i) = wGammaData;
			*(g_pGammaTable+i+65536)  = wGammaData;
			*(g_pGammaTable+i+65536*2) = wGammaData;
		}
		
	}
	else
	{	
		XDBG(DEBUG_SPICALL,"SetParameters: set g_pGammaTable to NULL");
		g_pGammaTable = NULL;
	}
	
	g_bSharpen = OptionReadInt("[spicall]","SharpenEnable",0,INIFILENAME);
	
	return TRUE;
}


BOOL GetParameters(LPGETPARAMETERS pGetParameters)
{

	XDBG(DEBUG_SPICALL,"GetParameters: call in ");

	if(ST_Reflective==g_ScanType)
	{
		if(FALSE==Reflective_ScanSuggest(&g_tiTarget,&g_ssSuggest))
		{
			XDBG(DEBUG_SPICALL,"GetParameters: Reflective_ScanSuggest error");
			return FALSE;			 
		}
	}
	else
	{
		if(FALSE==Transparent_ScanSuggest(&g_tiTarget,&g_ssSuggest))
		{
			XDBG(DEBUG_SPICALL,"GetParameters: Transparent_ScanSuggest error");
			return FALSE;			
		}
	}

	pGetParameters->wSourceXDPI=g_ssSuggest.wXDpi;
	pGetParameters->wSourceYDPI=g_ssSuggest.wYDpi;
	pGetParameters->dwLength=(DWORD) g_ssSuggest.wHeight;
	pGetParameters->dwLineByteWidth=g_ssSuggest.dwBytesPerRow;

	XDBG(DEBUG_SPICALL,"GetParameters: leave GetParameters");

	return TRUE;
}


BOOL StartScan()
{
	XDBG(DEBUG_SPICALL,"StartScan: call in ");
	if(ST_Reflective==g_ScanType)
	{
		XDBG(DEBUG_SPICALL,"StartScan: g_ScanType==ST_Reflective ");

		return Reflective_SetupScan(
			g_ssSuggest.cmScanMode,
			g_ssSuggest.wXDpi,
			g_ssSuggest.wYDpi,
			PF_BlackIs0,
			g_ssSuggest.wX,
			g_ssSuggest.wY,
			g_ssSuggest.wWidth,
			g_ssSuggest.wHeight);
	}
	else
	{
		XDBG(DEBUG_SPICALL,"StartScan: g_ScanType==ST_Transparent ");

		return Transparent_SetupScan(
			g_ssSuggest.cmScanMode,
			g_ssSuggest.wXDpi,
			g_ssSuggest.wYDpi,
			PF_BlackIs0,
			g_ssSuggest.wX,
			g_ssSuggest.wY,
			g_ssSuggest.wWidth,
			g_ssSuggest.wHeight);

	}

}


BOOL ReadScannedData(LPIMAGEROWS pImageRows)
{

	BOOL isRGBInvert;
    WORD Rows;
	BYTE* lpBlock = (BYTE*)pImageRows->pBuffer;
	int i;

	XDBG(DEBUG_SPICALL,"ReadScannedData: call in ");

	if( pImageRows->roRgbOrder == RO_RGB )
		isRGBInvert = FALSE;
	else
		isRGBInvert = TRUE;

	Rows = pImageRows->wWantedLineNum;	
	XDBG(DEBUG_SPICALL,"ReadScannedData: wanted Rows = %d",Rows );

	if(ST_Reflective==g_ScanType)
	{
		if(FALSE==Reflective_GetRows(lpBlock,&Rows,isRGBInvert))
			return FALSE;			 
	}
	else
	{
		if(FALSE==Transparent_GetRows(lpBlock,&Rows,isRGBInvert))
			return FALSE;			
	}

	pImageRows->wXferedLineNum = Rows;

	if (g_PixelFlavor==PF_WhiteIs0 || g_ScanMode==CM_TEXT || g_ssScanSource==SS_Negative) 
	{
		int TotalSize= Rows*g_ssSuggest.dwBytesPerRow;
		for(i=0;i<TotalSize;i++) 
		{
			*(lpBlock++)^=0xff;
		}
	}

	if(g_ssScanSource==SS_Negative)
	{
		AutoLevel(lpBlock, g_ScanMode, Rows, g_ssSuggest.dwBytesPerRow);
		QBetChange(lpBlock, g_ScanMode, Rows, g_ssSuggest.dwBytesPerRow);
	}

	XDBG(DEBUG_SPICALL,"ReadScannedData: leave ReadScannedData");

	return TRUE;


}

BOOL StopScan()
{

	BOOL rt;
	int i;

	XDBG(DEBUG_SPICALL,"StopScan: call in ");

	if(ST_Reflective==g_ScanType)
	{
		rt = Reflective_StopScan();			 
	}
	else
	{
		rt = Transparent_StopScan();			 
	}
	
	if(g_isSelfGamma && g_pGammaTable!=NULL)
	{
		for(i=0;i<20;i++)
		{
			if(!g_isScanning)
			{
				free(g_pGammaTable);
				g_pGammaTable=NULL;				
				break;
			}
			else
			{
				sleep(1); 
			}
		}
	}

	if(g_lpReadImageHead != NULL) 
	{
		free(g_lpReadImageHead); 
		g_lpReadImageHead = NULL;
	}

	XDBG(DEBUG_SPICALL,"StopScan: call out");
	
	return rt;
}

BOOL IsTAConnected()
{	
	BOOL hasTA;
	XDBG(DEBUG_SPICALL,"IsTAConnected: call in ");

	if(Asic_Open(&g_chip,g_pDeviceFile)!=STATUS_GOOD)
	{
		return FALSE;
	}

	if(Asic_IsTAConnected(&g_chip,&hasTA)!=STATUS_GOOD)
	{
		Asic_Close(&g_chip);
		return FALSE;
	}
	
	Asic_Close(&g_chip);

	return hasTA;


}


BOOL GetKeyStatus(BYTE* pKey)
{
	return FALSE;  

}


void AutoLevel(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine)
{
   int    ii;
   DWORD  i,j;
   DWORD  tLines,CountPixels,TotalImgSize;
   WORD   R,G,B,max_R,max_G,max_B,min_R,min_G,min_B;
   float  fmax_R,fmax_G,fmax_B;
   DWORD  sum_R = 0, sum_G = 0, sum_B = 0;
   float  mean_R, mean_G, mean_B;
   DWORD  hisgram_R[256], hisgram_G[256], hisgram_B[256];
      
   DWORD iWidth=BytesPerLine/3;
   DWORD iHeight=ScanLines;
   BYTE* pbmpdata=(BYTE*)lpSource;   

   
   DWORD   tmp = 0;
   WORD   imin_threshold[3];
   WORD   imax_threshold[3];

   if (scanMode != SM_RGB24) 
   {
	   return;
   }
   
   i = j = 0;
   tLines = CountPixels = TotalImgSize = 0;


   TotalImgSize = iWidth * iHeight;
   for(i=0; i<256; i++){
          
      hisgram_R[i] = 0; hisgram_G[i] = 0; hisgram_B[i] = 0;
   }
   
   max_R = max_G = max_B =0;
   min_R = min_G = min_B =255;
   tLines = 0;
   for(j=0; j<iHeight; j++)
   {
          tLines = j * iWidth *3;
		  for(i=0;i<iWidth;i++)
		  {
			 R = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			 G = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			 B = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));

			 max_R = _MAX(R,max_R);
			 max_G = _MAX(G,max_G);
			 max_B = _MAX(B,max_B);

             min_R = _MIN(R,min_R);
             min_G = _MIN(G,min_G);
			 min_B = _MIN(B,min_B);

			 hisgram_R[(BYTE)R]++; hisgram_G[(BYTE)G]++; hisgram_B[(BYTE)B]++;

			 sum_R += R; sum_G += G; sum_B += B;

			 *(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
			 *(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
			 *(pbmpdata + (tLines + i * 3))     = (BYTE)B;

             CountPixels++;
		  }
          
   }
   mean_R = (float)(sum_R / TotalImgSize);
   mean_G = (float)(sum_G / TotalImgSize);
   mean_B = (float)(sum_B / TotalImgSize);


   imin_threshold[0] = 0;    imin_threshold[1] = 0;    imin_threshold[2] = 0;
   imax_threshold[0] = 0;    imax_threshold[1] = 0;    imax_threshold[2] = 0;

   for(ii = 0; ii < 256; ii++)
   {
	  if(hisgram_R[ii] > 0)
		if(hisgram_R[ii] >= imin_threshold[0])
		{
		  min_R = ii;
		  break;
		}
   }

   tmp = 0;
   for (ii = 255; ii >= 0; ii--)
   {
	   if (hisgram_R[ii] > 0)
		if(hisgram_R[ii] >= imax_threshold[0])
		{
		   max_R = ii;
		   break;
		}
	   
   }

   tmp = 0;
   for (ii = 0; ii < 256; ii++)
   {
	   if(hisgram_G[ii] > 0)
	    if(hisgram_G[ii] >= imin_threshold[1])
		{
		  min_G = ii;
		  break;
		}
   }

   tmp = 0;
   for (ii = 255; ii >= 0; ii--)
   {
	   if(hisgram_G[ii] > 0)
		if(hisgram_G[ii] >= imax_threshold[1])
		{
		   max_G = ii;
		   break;
		}
   }

   tmp = 0;
   for (ii = 0; ii < 256; ii++)
   {
	   if(hisgram_B[ii] > 0)
	    if(hisgram_B[ii]  >= imin_threshold[2])
		{
		  min_B = ii;
		  break;
		}
   }

   tmp = 0;
   for (ii = 255; ii >= 0; ii--)
   {
	   if(hisgram_B[ii] > 0)
	    if(hisgram_B[ii] >= imax_threshold[2])
		{
		  max_B = ii;
		  break;
		}
   }


   sum_R = max_R-min_R;
   sum_G = max_G-min_G;
   sum_B = max_B-min_B;

   
   for(j=0; j<iHeight; j++){
          tLines = j * iWidth *3;
		  for(i=0;i<iWidth;i++)
		  {
			 R = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			 G = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			 B = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
			 
			 if(sum_R == 0) R = max_R;
			 else if (R < min_R) R = 0;
			 else if(R<=255){
				 fmax_R = ((float)((R - min_R)*255)/(float)sum_R); 
				 R = (WORD)fmax_R;
				 fmax_R = (fmax_R - R)*10;

				if(fmax_R >=5)
					 R++;
			 }
			 if (R > 255)   R = 255;

			 if(sum_G == 0) G = max_G;
			 else if (G < min_G) G = 0;
			 else if(G<=255){
				 fmax_G = ((float)((G - min_G)*255)/(float)sum_G); 
				 G = (WORD)fmax_G;
				 fmax_G = (fmax_G - G)*10;
				 if(fmax_G >=5)
					 G++;
				 
			 }
			 if (G > 255)   G = 255;

			 if(sum_B == 0) B = max_B;
			 else if (B < min_B) B = 0;
			 else if(B<=255){
				 fmax_B = ((float)(B - min_B)*255/(float)sum_B); 
				 B = (WORD)fmax_B;
				 fmax_B = (fmax_B - B)*10;
				 if(fmax_B >= 5)
					 B++;
			 }
			 if (B > 255)   B = 255;

			 hisgram_R[(BYTE)R]++; hisgram_G[(BYTE)G]++; hisgram_B[(BYTE)B]++;

			*(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
			*(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
			*(pbmpdata + (tLines + i * 3))     = (BYTE)B;

		  }

	   }
	return;
}


void QBETDetectAutoLevel(void* pDIB, DWORD ImageWidth, DWORD ImageHeight)
{
   WORD	  *pbmpdata;
   float  fRPercent = 0.0;
   float  fGPercent = 0.0;
   float  fBPercent = 0.0;
   float  fRSum,fGSum,fBSum;

   int    i,j;
   DWORD  tLines,CountPixels,TotalImgSize;
   WORD   R,G,B,max_R,max_G,max_B,min_R,min_G,min_B;
   WORD   wIndexR,wIndexG,wIndexB;
   float  fmax_R,fmax_G,fmax_B;
   DWORD  sum_R = 0, sum_G = 0, sum_B = 0;
   DWORD  hisgram_R[1024], hisgram_G[1024], hisgram_B[1024];
   
   if(!pDIB)
   {
	   return;
   }

   pbmpdata    = (WORD *) pDIB;

      

   CountPixels = 0;
   TotalImgSize = ImageWidth * ImageHeight;
  
   for(i=0; i<1024; i++){
          
      hisgram_R[i] = 0; hisgram_G[i] = 0; hisgram_B[i] = 0;
   }
   
   max_R = max_G = max_B =0;
   min_R = min_G = min_B =1023;
   tLines = 0;

   for(j=0; j<(int)ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<(int)ImageWidth;i++)
		  {
			 R = *(pbmpdata + (tLines + i * 3 + 2));
			 G = *(pbmpdata + (tLines + i * 3 + 1));
			 B = *(pbmpdata + (tLines + i * 3));

			 max_R = _MAX(R,max_R);
			 max_G = _MAX(G,max_G);
			 max_B = _MAX(B,max_B);

             min_R = _MIN(R,min_R);
             min_G = _MIN(G,min_G);
			 min_B = _MIN(B,min_B);

			 hisgram_R[R]++; hisgram_G[G]++; hisgram_B[B]++;

			 sum_R += R; sum_G += G; sum_B += B;

			 *(pbmpdata + (tLines + i * 3 + 2)) = R;
			 *(pbmpdata + (tLines + i * 3 + 1)) = G;
			 *(pbmpdata + (tLines + i * 3))     = B;

             CountPixels++;
		  }
          
   }

   fRSum = 0.0;
   fGSum = 0.0;
   fBSum = 0.0;

   wIndexR = 511;
   wIndexG = 511;
   wIndexB = 511;

   for(i=0;i<1024;i++){
     fRSum += (float)hisgram_R[i];
     fRPercent = (fRSum / CountPixels)*100;
	 if(fRPercent > 50) 
     { 
       wIndexR = i;
	   break;
	 }
	 
   }

   for(i=0;i<1024;i++){
     fGSum += (float)hisgram_G[i];
     fGPercent = (fGSum / CountPixels) * 100;
	 if(fGPercent > 50) 
     { 
       wIndexG = i;
	   break;
	 }
	 
   }

   for(i=0;i<1024;i++){
     fBSum += (float)hisgram_B[i];
     fBPercent = (fBSum / CountPixels) * 100;
	 if(fBPercent > 50) 
     { 
       wIndexB = i;
	   break;
	 }
	 
   }

   fRSum = 0.0;
   for(i=wIndexR;i>=0;i--){
     fRSum += (float)hisgram_R[i];
     fRPercent = (fRSum / CountPixels) * 100;
	 if(fRPercent >= 48) 
     { 
       min_R = i;
	   break;
	 }
	 
   }

   fRSum = 0.0;
   for(i=wIndexR;i<1024;i++){
     fRSum += (float)hisgram_R[i];
     fRPercent = (fRSum / CountPixels) * 100;
	 if(fRPercent >= 47) 
     { 
       max_R = i;
	   break;
	 }
	 
   }

   fGSum = 0.0;
   for(i=wIndexG;i>=0;i--){
     fGSum += (float)hisgram_G[i];
     fGPercent = (fGSum / CountPixels) * 100;
	 if(fGPercent >= 48) 
     { 
       min_G = i;
	   break;
	 }
	 
   }

   fGSum = 0.0;
   for(i=wIndexG;i<1024;i++){
     fGSum += (float)hisgram_G[i];
     fGPercent = (fGSum / CountPixels) * 100;
	 if(fGPercent >= 47) 
     { 
       max_G = i;
	   break;
	 }
	 
   }

   fBSum = 0.0;
   for(i=wIndexB;i>=0;i--){
     fBSum += (float)hisgram_B[i];
     fBPercent = (fBSum / CountPixels) * 100;
	 if(fBPercent >= 46)
     { 
       min_B = i;
	   break;
	 }
	 
   }

   fBSum = 0.0;
   for(i=wIndexB;i<1024;i++){
     fBSum += (float)hisgram_B[i];
     fBPercent = (fBSum / CountPixels) * 100;
	 if(fBPercent >= 47)
     { 
       max_B = i;
	   break;
	 }
	 
   }


   
   sum_R = max_R- min_R;
   sum_G = max_G- min_G;
   sum_B = max_B- min_B;
   
   for(j=0; j<(int)ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<(int)ImageWidth;i++)
		  {
			 R = *(pbmpdata + (tLines + i * 3 + 2));
			 G = *(pbmpdata + (tLines + i * 3 + 1));
			 B = *(pbmpdata + (tLines + i * 3));
			 
		

			 if(sum_R == 0) R = max_R;
			 else if(R<min_R){
				  
				  R=0;
			 }
			 else if((R>=min_R) && (R<=1023)){
				 fmax_R = ((float)((R - min_R)*923)/(float)sum_R)+100; 
				 R = (WORD)fmax_R;
				 fmax_R = (fmax_R - R)*10;
				 if(fmax_R >=5)
				    R++;
			 }
			 if (R > 1023)   R = 1023;


			 if(sum_G == 0) G = max_G;
			 else if(G<min_G){
                 
				 G=0;
			 }
			 else if((G>=min_G) && (G<=1023)){
				 fmax_G = ((float)((G - min_G)*923)/(float)sum_G)+100; 
				 G = (WORD)fmax_G;
				 fmax_G = (fmax_G - G)*10;
				 if(fmax_G >=5)
					 G++;
			 }
			 if (G > 1023)   G = 1023;


			 if(sum_B == 0) B = max_B;
			 else if(B<min_R){
				 
				 B=0;
			 }
			 else if((B>=min_B) && (R<=1023)){
				 fmax_B = ((float)(B - min_B)*923/(float)sum_B)+100; 
				 B = (WORD)fmax_B;
				 fmax_B = (fmax_B - B)*10;
				 if(fmax_B >= 5)
					 B++;
			 }
			 if (B > 1023)   B = 1023;
			 
			*(pbmpdata + (tLines + i * 3 + 2)) = R;
			*(pbmpdata + (tLines + i * 3 + 1)) = G;
			*(pbmpdata + (tLines + i * 3))     = B;

		  }

	   }
   return;

}



void QBetChange(LPSTR lpSource, SCANMODE scanMode, WORD ScanLines, DWORD BytesPerLine)
{
   WORD			i,j;
   DWORD		tLines,TotalImgSize;
   WORD			R1,G1,B1,R,G,B,R2,G2,B2,QBET_RGB,PointF,PointB;
   WORD*		pwRGB;
   
   int k;

   DWORD ImageWidth=BytesPerLine/3;
   DWORD ImageHeight=ScanLines;
   BYTE* pbmpdata=(BYTE*)lpSource; 
   
   if (scanMode != SM_RGB24)
   {
	   return;
   }


   TotalImgSize = ImageWidth * ImageHeight *3 *2;
   if((pwRGB = (WORD*) malloc (TotalImgSize) ) == NULL)
   {
	  return ;
   }      

   for(j=0; j<ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<ImageWidth;i++)
		  {
             if(i == 0){ 
				 R1= R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			     G1= G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			     B1= B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));

				 R2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 2));
			     G2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 1));
			     B2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3));
			 }
			 else if(i == (ImageWidth-1)){
				 R1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 2));
			     G1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 1));
			     B1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3));

				 R2= R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			     G2= G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			     B2= B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));
				 
			     
			 }
			 else{
			    R1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 2));
			    G1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3 + 1));
			    B1 = (WORD)(BYTE)*(pbmpdata + (tLines + (i-1) * 3));

				R  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 2));
			    G  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3 + 1));
			    B  = (WORD)(BYTE)*(pbmpdata + (tLines + i * 3));

				R2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 2));
			    G2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3 + 1));
			    B2 = (WORD)(BYTE)*(pbmpdata + (tLines + (i+1) * 3));
			 }



             R1 = R1 & 0x0003;
			 G1 = G1 & 0x0003;
			 B1 = B1 & 0x0003;

			 R2 = R2 & 0x0003;
			 G2 = G2 & 0x0003;
			 B2 = B2 & 0x0003;
			 for(k=0;k<3;k++)
			 {
			 	if(k == 0)
				{ 
			       PointF = R1;
			       PointB = R2;
				}
				else if(k == 1)
				{
				   PointF = G1;
				   PointB = G2;
				}
				else if(k == 2)
				{
				   PointF = B1;
				   PointB = B2;
				}
                      
				switch(PointF){
				   case 0:
				   case 1:	
					   if(PointB == 0)
						   QBET_RGB = 0xFFFC;
					   else if(PointB == 1)
						   QBET_RGB = 0xFFFC;
					   else if(PointB == 2)
						   QBET_RGB = 0xFFFD;
					   else if(PointB == 3)
						   QBET_RGB = 0xFFFE;
				       break;
				   case 2:
					   if(PointB == 0)
						   QBET_RGB = 0xFFFD;
					   else if(PointB == 1)
						   QBET_RGB = 0xFFFD;
					   else if(PointB == 2)
						   QBET_RGB = 0xFFFF;
					   else if(PointB == 3)
						   QBET_RGB = 0xFFFF;
					   break;
				   case 3:
					   if(PointB == 0)
						   QBET_RGB = 0xFFFE;
					   else if(PointB == 1)
						   QBET_RGB = 0xFFFE;
					   else if(PointB == 2)
						   QBET_RGB = 0xFFFF;
					   else if(PointB == 3)
						   QBET_RGB = 0xFFFF;
					   break;
				   default:
					   break;	
				}

                if(k==0){
                    R = R << 2;
                    R = R + 0x0003;
                    R = R & QBET_RGB;
				}
				else if(k==1){
					G = G << 2;
                    G = G + 0x0003;
                    G = G & QBET_RGB;
				}
				else if(k==2){
                    B = B << 2;
                    B = B + 0x0003;
                    B = B & QBET_RGB;
				}
			 }
             
			 *(pwRGB + (tLines + i * 3 + 2)) = R;
			 *(pwRGB + (tLines + i * 3 + 1)) = G;
			 *(pwRGB + (tLines + i * 3))     = B;
			
		  }
          
   }

   QBETDetectAutoLevel(pwRGB,ImageWidth,ImageHeight); 
    for(j=0; j<ImageHeight; j++){
          tLines = j * ImageWidth *3;
		  for(i=0;i<ImageWidth;i++)
		  {
			  R = *(pwRGB + (tLines + i * 3 + 2)) ;
			  G = *(pwRGB + (tLines + i * 3 + 1)) ;
			  B = *(pwRGB + (tLines + i * 3))     ;
            
			  R = R >> 2;
			  G = G >> 2;
			  B = B >> 2;
			  if(R>255) R = 255;
              		  if(G>255) G = 255;
			  if(B>255) B = 255;

			  *(pbmpdata + (tLines + i * 3 + 2)) = (BYTE)R;
			  *(pbmpdata + (tLines + i * 3 + 1)) = (BYTE)G;
			  *(pbmpdata + (tLines + i * 3))     = (BYTE)B;
			
		  }
          
   }

   if(pwRGB!=NULL){
     free(pwRGB);
   }

   
   return;

}

